<?php
namespace App\Classes\Finance;

use App\Classes\Finance\DailyJournalHelper;


use App\Models\Finance\MultiVoucher;
use App\Models\Sales\Customer;
use App\Models\Main\Bank;
use App\Models\Finance\Tax;
use App\Models\Finance\Treasury;
use App\Models\Finance\ExpenseCode;
use App\Models\Sales\Delegate;
use App\Models\Finance\DailyJournal;
use App\Models\Main\Supplier;
use Illuminate\Support\Facades\DB;

class MultiVoucherHelper
{
    function get_next_codes($date, $year, $branch)
    {
        $data['year_ser'] = intval(DB::table('multi_voucher_m')
                ->where('finance_year_id', $year)
                ->max('year_ser')) + 1;

        $data['month_ser'] = intval(DB::table('multi_voucher_m')
                ->where('finance_year_id', $year)
                ->max('month_ser')) + 1;

        $data['branch_ser'] = intval(DB::table('multi_voucher_m')
                ->where('branch_id', $branch)
                ->max('branch_ser')) + 1;

        return $data;
    }

    function can_edit($id)
    {
        return true;
    }

    function can_delete($id)
    {

        $master         = MultiVoucher::get_multi_voucher_m($id , 'en');
        $details        = MultiVoucher::get_multi_voucher_d($id , 'en');
        if(!isset($master->id) ){
            return [false , 'item not found !'];
        }

        if(collect($details)->where('invoice_id' , 0 )->count() != count($details) ){
            return [false , 'some item has invoices !'];
        }

        return [true];
        return true;
    }

    function generate_journal($id)
    {

        $master         = MultiVoucher::get_multi_voucher_m($id , 'en');
        $details        = MultiVoucher::get_multi_voucher_d($id , 'en');
        // $files          = MultiVoucher::get_attachs($id);

        if($master->multi_voucher_active == 0 && $master->daily_journal_m_id > 0 ){
            DailyJournal::delete_daily_journal_m($master->daily_journal_m_id);
            return array( true ,  '')  ;
        }


        $items          = [];
        $journal_data   = array(
            'finance_year_id'   => $master->finance_year_id  ,
            'branch_id'         => $master->branch_id ,
            'des'               => $master->description  ,
            'journal_date'      => $master->action_date ,
            'document_id'       => $id ,
            'daily_type_id'     => 20 ,
            'auto_generated'    => 1 ,
            'journal_active'    => 1 ,
            'confirmed'         => 1 ,
            'document_id'       => $id ,
            'daily_journal_id'  => $master->daily_journal_m_id
        );

        foreach ($details as $key => $row) {

             switch ($row->dis_type) {
                case '1':
                    $account_tree_id = Treasury::get_treasury($row->dis_treasury_id, 'en')->account_tree_id ?? 0;
                    $dis_type = 'Treasury';
                break;
                case '2':
                    $account_tree_id = Bank::get_bank($row->dis_bank_id , 'en')->account_tree_id ?? 0;
                    $dis_type = 'Bank';
                    break;
                case '3':
                    $account_tree_id = Customer::get_customer($row->dis_customer_id, 'en')->account_tree_id ?? 0;
                    $dis_type = 'Customer';
                break;
                case '4':
                    $account_tree_id = Supplier::get_purchases_vendor($row->dis_supplier_id, 'en')->account_tree_id ?? 0;
                    $dis_type = 'PurchasesVendor';
                break;
                case '5':
                    $account_tree_id = $row->dis_account_tree_id;
                break;
                case '6':
                    $account_tree_id = Delegate::get_delegate($row->dis_delegate_id, 'en')->account_tree_id ?? 0;
                    $dis_type = 'Delegate';
                break;
                case '7':
                    $account_tree_id = Delegate::get_delegate($row->dis_delegate_id, 'en')->account_tree_id ?? 0;
                    $dis_type = 'Delegate';
                break;
                case '8':
                    $account_tree_id = ExpenseCode::get_expense_code($row->dis_expense_code_id, 'en')->account_tree_id ?? 0;
                    $dis_type = 'ExpenseCode';
                break;
                case '9':
                    $account_tree_id = Delegate::get_delegate($row->dis_delegate_id, 'en')->account_tree_id ?? 0;
                    $dis_type = 'Delegate';
                break;
                default:
                    $account_tree_id = 0;
                    $dis_type = '';
                break;
            }
            if(  $account_tree_id == 0 ){
                return array( false ,  trans('error.'.($dis_type ?? '').' Account tree id not found') , '' ,[] );
            }

            $items[]    = array(
                'account_tree_id'   => $account_tree_id ,
                'cost_center_id'    => $row->cost_centers ?? '' ,
                'debit'             => ($row->voucher_type == 1 ? $row->amount_without_vat : 0 ) ,
                'credit'            => ($row->voucher_type == 2 ? $row->amount_without_vat : 0 ) ,
                'description'       => $row->description
            );

            if($row->tax_id > 0 ){
                $tax = Tax::get_tax($row->tax_id , 'en');
                $items[]    = array(
                    'account_tree_id'   => $tax->account_tree_id ,
                    'cost_center_id'    => '' ,
                    'debit'             => ($row->voucher_type == 1 ? $row->total_vat : 0 ) ,
                    'credit'            => ($row->voucher_type == 2 ? $row->total_vat : 0 ) ,
                    'description'       => $row->description
                );
            }

        }

        $journal = (new DailyJournalHelper)->generate_journal($journal_data , $items );

        if($journal[0]){
            DB::table('multi_voucher_m')
                ->where('multi_voucher_m_id' , $id)
                ->update( ['daily_journal_m_id' => $journal[2]] );
        }

        return $journal;

    }
    function check_invoices($clearance_id  = null ,$invoice_id = null, $invoice_type = null ) //check if the detail has invoice to update it
    {

        if($invoice_id > 0){
            if($invoice_type == 'claim'){
                DB::table('clearance_invoices_claim_m')->where('invoice_id' , $invoice_id)->update(['need_update' => 1]);
            }elseif($invoice_type == 'invoice'){
              DB::table('clearance_invoices_m')->where('invoice_id' , $invoice_id)->update(['need_update' => 1]);

            }
        }
        if($clearance_id > 0){
            $last_claim = DB::table('clearance_invoices_claim_m')->where('clearance_id' , $clearance_id)->orderBy('invoice_id', 'desc')->first();
            if(isset($last_claim) && $last_claim->invoice_id > 0){
                DB::table('clearance_invoices_claim_m')->where('invoice_id' , $last_claim->invoice_id)->update(
                    ['need_update' => 1]
                );

            }else{
                $last_invoice = DB::table('clearance_invoices_m')->where('clearance_id' , $clearance_id)->orderBy('invoice_id', 'desc')->first();
                if(isset($last_invoice) && $last_invoice->invoice_id > 0){
                    DB::table('clearance_invoices_m')->where('invoice_id' , $last_invoice->invoice_id)->update(
                        ['need_update' => 1]
                    );
                }
            }

        }

        return true;
    }
}
