<?php

namespace App\Classes\Sales;
use App\Classes\Finance\DailyJournalHelper;
use App\Models\Finance\DailyJournal;
use App\Models\Finance\Tax;
use App\Models\Sales\NoticeCreditor;
use App\Models\Main\Setting;
use Illuminate\Support\Facades\DB;

class NoticeCreditorHelper {


    function calculate_master_values($id){
        $details    = NoticeCreditor::get_notice_creditor_d($id, 'en');
        $update = [
            'total_vat'          => collect($details)->sum('vat_amount'),
            'total_without_vat'  => collect($details)->sum('amount_without_vat'),
            'total_with_vat'     => collect($details)->sum('amount_with_vat'),
        ];

        NoticeCreditor::update_notice_creditor_m($update , $id);
    }



    function generate_journal($id){
        $check  = Setting::get_main_setting_by_code('creditor_notice_journal')->value ?? false ;
        if(!$check)
            return array( true ,  trans("error.add_fails") ) ;

        $master     = NoticeCreditor::get_notice_creditor_m($id, 'en');

        if(!isset($master->active) || $master->active == 0 ){
            DailyJournal::delete_daily_journal_m($master->daily_journal_id);
            return array( true ,  trans("error.add_fails") ) ;
        }
        $details    = NoticeCreditor::get_notice_creditor_d($id, 'en');

        $items      = [];
        $journal_data  = array(
            'finance_year_id'   => $master->finance_year_id  ,
            'branch_id'         => $master->branch_id ,
            'des'               => "اشعار دائن لفاتوره رقم " . $master->invoice_m_id . " معامله " . $master->clearance_id . " بيان " . $master->notice_creditor_m_id  ,
            'journal_date'      => $master->invoice_date ,
            'document_id'       => $id ,
            'daily_type_id'     => 15 ,
            'auto_generated'    => 1 ,
            'journal_active'    => 1 ,
            'confirmed'         => 1
        );

        if($master->daily_journal_id > 0 ){
            $journal_data['daily_journal_id'] = $master->daily_journal_id;
        }
        if($master->customer_account_tree_id < 1){
            $error_msg = trans('error.customer_account_not_found');
        }
            //check account shared
            $shared = DB::table('account_tree')
                ->where('account_tree_id' , $master->customer_account_tree_id )
                ->where('shared_account' , 1 )
                ->where('shared_type_id'  , 1 ) // customer
                ->first();
            if(isset($shared) ){
                $shared_profile_id = $master->customer_id;
            }
            $items[]   = array(
                'account_tree_id'   => $master->customer_account_tree_id ,
                'shared_profile_id' => $shared_profile_id ?? 0 ,
                'cost_center_id'    => 0 ,
                'debit'             => 0   ,
                'credit'            => collect($details)->sum('amount_with_vat') ,
                'description'       => 'قيمة اشعار دائن رقم '.$master->notice_creditor_m_id
            );


        foreach($details as $item){
            $items[]    = array(
                'account_tree_id'   => $item->account_tree_id ?? 0 ,
                'cost_center_id'    => 0 ,
                'debit'             => $item->amount_without_vat ,
                'credit'            => 0  ,
                'description'       => 'قيمة اشعار دائن رقم '.$master->notice_creditor_m_id . " , ".$item->revenue_name
            );
            if($item->account_tree_id < 1){
                $error_msg = trans('error.revenue_account_not_found');

            }
        }

        $tax = Setting::get_main_setting_by_code('default_sales_vat')->value ?? 0;
        $sales_taxs_account = Tax::get_tax($tax , 'en')->account_tree_id ?? 0;
        if($sales_taxs_account < 1){
            $error_msg = trans('error.sales_taxs_account_not_found');
        }

        $items[]    = array(
            'account_tree_id'   => $sales_taxs_account ,
            'cost_center_id'    => 0 ,
            'debit'             => collect($details)->sum('vat_amount')  ,
            'credit'            => 0 ,
            'description'       => 'ضريبة اشعار دائن رقم '.$master->notice_creditor_m_id
        );
        $journal = (new DailyJournalHelper)->generate_journal($journal_data , $items);

        if($journal[0]){
            DB::table('notice_creditors_m')->where('notice_creditor_m_id' , $id)->update( ['daily_journal_id' => $journal[2]] );
            return $journal;

        }else{

            $response = [
               'check' => false,
               'data'  => $journal ,

               'msg'   => $error_msg ?? $journal[1] ?? 'مشكلة في الحسابات المرتبطة بالقيد '
            ];
            header('Content-Type: application/json; charset=utf-8; ');
            header('Access-Control-Allow-Origin:*');
            header('Access-Control-Allow-Headers:*');
            echo json_encode($response);
            die();

        }

    }



}
