<?php

namespace App\Exports;

use App\Models\Sales\Classification;
use Maatwebsite\Excel\Concerns\FromArray;
use PhpOffice\PhpSpreadsheet\Cell\Cell;

use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithMapping;
// use Maatwebsite\Excel\Concerns\WithTitle;
use PhpOffice\PhpSpreadsheet\Style\NumberFormat;
use Maatwebsite\Excel\Concerns\WithColumnFormatting;
use PhpOffice\PhpSpreadsheet\Cell\DataType;
use PhpOffice\PhpSpreadsheet\Cell\DefaultValueBinder;
use Maatwebsite\Excel\Concerns\WithCustomValueBinder;


class ClassificationsExport extends DefaultValueBinder implements  ShouldAutoSize , WithCustomValueBinder , WithHeadings , FromCollection
{
    protected $rows;

    public function __construct(array $rows)
    {
        $this->rows = $rows;
    }

    public function bindValue(Cell $cell, $value)
    {
        if (is_numeric($value) && $value > 10000000 ) {
            $cell->setValueExplicit($value, DataType::TYPE_STRING);
            return true;
        }

        // else return default behavior
        return parent::bindValue($cell, $value);
    }


    public function headings(): array
    {
        return [
            'الرقم التسلسلي للفاتورة / Invoice Seq', // - 01
            'رمز بلد المنشأ / Country of Origin Code',  // - 02
            'رمز التعرفة / Tariff Code', // - 03
            'وصف البضائع / Goods Description', // - 04
            'الوزن الإجمالي / Gross Weight', // - 05
            'الوزن الصافي / Net Weight', // - 05
            'عدد الوحدات / No of Unit',// - 06
            'قيمة الصنف / Item Value',// - 07
            'رمز نوع الرسوم / Duty Type Code',// - 08
            'رمز نوع التار يخ / Date Type Code',// - 09
            'تاريخ الإنتاج / Production Date',// - 10
            'تاريخ الإنتهاء / Expiry Date',// - 11
            'حجم الحاوية / Container Size',// - 12
            'بلد الشراء / Buying Country',// - 13
            'وصف المسار / Route Description',// - 14
            'سعر البيع للمستهلك النهائي / Sale Price For Consumer',// - 15
            'رقم الصنف(في نظام هيئة الغذاء و الدواء) / Batch Number',// - 16
            'رقم الطلب(في نظام هيئة الغذاء والدواء) / Pre-Notification Number',// - 17
            'كمية الصنف / Item Quanitity',// - 18
            'وحدة الصنف / Item Unit',// - 19
            'الكمية الدولية / International Quantity',// - 20
            'كمية التعرفة / Tariff Quantity',// - 21
            'وحدة التعرفة / Tariff Unit',// - 22
            'كمية الاحد الأدنى للرسوم / Minimum Duty Quantity',// - 23
            'وحدة الحد الأدنى للرسوم / Minimum Duty Unit',// - 24
            'كمية البندرول / Banderole Quantity',// - 25
            'وحدة الرسوم الإضافية / Additional Fee Unit',// - 26
        ];
    }

    public function collection()
    {
        return collect($this->rows)->map(function ($row) {
            return [
                1,                                                          // - 01
                $row->country_id,                                           // - 02
                string_len($row->customs_item_number , 1 , 16 ,  "_") ,     // - 03
                string_len($row->item_name , 22 , 300 , "  البضاعه الوارده  ") ,      // - 04
                $row->weight,                                               // - 05
                $row->weight,                                               // - 05
                $row->qry,                                                  // - 06
                $row->amount,                                               // - 07
                1,                                                          // - 08
                '',                                                         // - 09
                '',                                                         // - 10
                '',                                                         // - 11
                $row->container_size ,                                      // - 12
                '',                                                         // - 13
                '',                                                         // - 14
                '',                                                         // - 15
                '',                                                         // - 16
                '',                                                         // - 17
                $row->qry,                                                  // - 18
                $row->unit_id,                                              // - 19
                $row->qry,                                                  // - 20
                '',                                                         // - 21
                '',                                                         // - 22
                '',                                                         // - 23
                '',                                                         // - 24
                '',                                                         // - 25
                ''                                                          // - 26
            ];
        });
    }

}
