<?php

namespace App\Http\Controllers\API\V1\Finance;

use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;
use App\Models\Finance\CashReceipt;
use App\Models\Main\ScreenSub;
use App\Models\Main\Bank;
use App\Models\Main\FinanceYear;
use Validator;
use DB;
class CashReceiptController extends BaseController
{
    protected $return = array();
    protected $filter = array();
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index($lang , Request $request)
    {
        $this->checkPermission('02-006','view');

        $this->filter                = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? 20 ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );
        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('02-006' , $lang)->sub_title ?? '';
        $this->return['items']       = CashReceipt::active_cash_receipt( $lang );
        return $this->sendResponse( $this->return );
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create($lang , Request $request)
    {
        $this->checkPermission('02-006','add');

        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('02-006' , $lang);
        $this->return['customers']          = [];
        $this->return['bank']               = Bank::all_banks($lang);
        $this->return['delegates']          = [];
        return $this->sendResponse( $this->return  );
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->checkPermission('02-006','add');

        $validator = validator::make( $request->all() , CashReceipt::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        $seed = array(
            'date' => $request['date'] ,
            'from_type' => $request['from_type'] ,
            'user_id' => $request['user_id'],
            'delegate_id' => $request['delegate_id'],
            'value' => $request['value'] ,
            'description' => $request['description'] ,
            'advance_payment' => $request['advance_payment'] ,
            'cost_center' => $request['cost_center'] ,
            'collection_method' => $request['collection_mehtod']
        );
        $seed['active'] = 1;

        if(isset($request['cost_center_id'])){
            $seed['cost_center_id'] = $request['cost_center_id'] ;
        }

        if(isset($request['bank_id'])){
            $seed['bank_id'] = $request['bank_id'];
        }

        CashReceipt::add_cash_receipt($seed);
        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($lang, $id)
    {
        $this->checkPermission('02-006','view');

        $this->return['screen_name']          = ScreenSub::get_screen_sub_by_screen_code('02-006' , $lang)->sub_title ?? '';
        $this->return['data']                 = CashReceipt::get_cash_receipt($id);

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update($lang , $id, Request $request)
    {
        $this->checkPermission('02-006','edit');


        $update                         = $request->all();
        //dd($id);
        $validator = validator::make( $request->all() , CashReceipt::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        CashReceipt::update_cash_receipt($update ,  $id);

        return $this->sendResponse( $this->return , trans('main.update_success') );

    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($lang, $id)
    {
        $this->checkPermission('02-006','delete');
        CashReceipt::delete_cash_receipt($id);
        return $this->sendResponse( [] , trans('main.delete_success') );
    }

}
