<?php
namespace App\Http\Controllers\API\V1\Finance;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;

use App\Classes\Finance\MaintenancePettyHelper;


use App\Models\Finance\MaintenancePetty;
use App\Models\Finance\ExpenseCode;
use App\Models\Finance\AccountTree;
use App\Models\Finance\Treasury;
use App\Models\Main\DailyType;
use App\Models\Main\ScreenSub;
use App\Models\Sales\Delegate;
use App\Models\Sales\Clearance;
use App\Models\Main\Branch;

use DB;
use PDF;
use App;
use Excel;

class MaintenancePettyController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('02-018','view');


        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'status' => $request['status'] ?? 1 ,
            'rows'   => $request['rows'] ?? 20 ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('02-018', $lang)->sub_title ?? '';
        $this->return['items']       = MaintenancePetty::all_maintenance_pettys($this->filter);

        return $this->sendResponse($this->return);
    }

    public function show($lang, $id)
    {
        $this->checkPermission('02-018','view');

        $this->return['screen_name']   = ScreenSub::get_screen_sub_by_screen_code('02-018' , $lang)->sub_title ?? '';
        $this->return['master']        = MaintenancePetty::get_maintenance_petty_m($id, $lang);
        $this->return['details']       = MaintenancePetty::get_maintenance_petty_d($id, $lang);

        $this->return['daily_types']   = DailyType::all_daily_types(array('active' => '1', 'lang' => $lang));
        $this->return['delegates']     = Delegate::all_delegates(array('active' => '1', 'lang' => $lang));
        $this->return['treasurys']     = Treasury::all_treasury(array('active' => '1', 'lang' => $lang));
        $this->return['account_trees'] = AccountTree::active_account_trees($lang, array('account_type' => 2, 'active' => '1'));

        return $this->sendResponse($this->return);
    }

    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('02-018','edit');

        $update = $request->only(['items']);

        $master = MaintenancePetty::get_maintenance_petty_m($id, $lang);

        DB::beginTransaction();

        if (isset($update['items']) && is_array($update['items'])) {
            foreach ($update['items'] as $item) {

                // if($item['amount'] > $master->remaining_amount){
                //     return $this->sendError(trans("error.Detail Amount Must Be Less Than Or Equal ").$master->remaining_amount);
                // }

                $details_seed = array(
                    'maintenance_petty_m_id' => $id,
                    'clearance_id'        => $item['clearance_id'] ?? 0 ,
                    'treasury_id'         => $item['treasury_id'] ?? 0 ,
                    'account_tree_id'     => $item['account_tree_id'] ?? 0 ,
                    'receipt_number'      => $item['receipt_number'] ?? "",
                    'description_ar'      => $item['arabic_description'] ?? "",
                    'description_en'      => $item['english_description'] ?? "",
                    'amount'              => $item['amount'],
                    'add_user'            => auth()->user()->id,
                );

                $validator = validator()->make($details_seed, MaintenancePetty::$details_rules);

                if($validator->fails()){
                    return $this->sendError(trans('error.add_fails'), $validator->errors());
                }

                MaintenancePetty::add_maintenance_petty_d($details_seed);

                // Calculate Difference
                DB::table('maintenance_petty_m')
                    ->where('maintenance_petty_m_id', $id)
                    ->decrement('remaining_amount', $item['amount']);

                // Close maintenance Petty Where Master Total Equal Details Total
                $details_total = DB::table('maintenance_petty_d')->where('maintenance_petty_m_id', $id)->sum('amount');
                if ($master->total_amount == $details_total){
                    DB::table('maintenance_petty_m')
                        ->where('maintenance_petty_m_id', $id)
                        ->update([
                            'maintenance_petty_status' => 2
                        ]);
                }
            }
        }else{
            return $this->sendError(trans('error.Found 0 Items'));
        }

        DB::commit();
        return $this->sendResponse($id, trans('main.maintenance Petty Added Successfully'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('02-018','delete');

        $master = MaintenancePetty::get_maintenance_petty_m($id, $lang);

        if(!isset($master->maintenance_petty_m_id) || $master->branch_id != $this->data['current_branch'] || !(new MaintenancePettyHelper)->can_edit_or_delete($id)){
            return $this->sendError(trans("error.delete_fail"));
        }

        MaintenancePetty::delete_maintenance_petty_m($id);

        return $this->sendResponse([], trans('main.delete_success'));
    }





    public function export_data($lang , $id , Request $request)
    {
        $this->checkPermission('02-018','print');

        $this->return['lang']           = $lang;

        $this->return['print_title']    = ScreenSub::get_screen_sub_by_screen_code('02-018' , $lang)->sub_title ?? 'Maintenance Petty';
        $this->return['master']         = MaintenancePetty::get_maintenance_petty_m($id, $lang);
        $this->return['details']        = MaintenancePetty::get_maintenance_petty_d($id, $lang);

        $this->return['branch_data']    = Branch::get_branch($this->return['master']->branch_id);
        $this->return['print_right']    = $this->return['branch_data']->print_label_ar ?? '';
        $this->return['print_left']     = $this->return['branch_data']->print_label_en ?? '';
        $this->return['logo']           = get_logo( $this->return['master']->branch_id );


        switch (@$request['type']) {
            case 'excel':
                $this->return['logo']     = public_path().$this->return['logo'];
                $export = new InvoicesExport($this->return , 'finance/maintenance_petty/print');
                return Excel::download($export, 'maintenance_petty.xlsx');
                break;
            case 'pdf':
                $this->return['logo']     = public_path().$this->return['logo'];
                $pdf = PDF::loadView('Finance/maintenance_petty/print', array('data' => $this->return ));
                return $pdf->stream();

                break;
            case 'image':
                $this->return['logo']     = public_path().$this->return['logo'];
                $image = App::make('snappy.image.wrapper');
                $image->loadView('Finance/maintenance_petty/print', array('data' => $this->return ));
                return $image->stream();

                break;
            case 'print':
                return view('Finance/maintenance_petty/print')->with('data' , $this->return);
                break;
        }

    }


    public function toggle_active($lang, $id)
    {
        $item = MaintenancePetty::find($id);

        if($item->maintenance_petty_active){
            $this->checkPermission('02-018','delete');


            if(!(new MaintenancePettyHelper)->can_edit_or_delete($id)){
                return $this->sendError( trans('error.delete_fails'), []);
            }

            MaintenancePetty::delete_maintenance_petty_m($id);
            $this->return['active'] = false;
        }else{
            $this->checkPermission('02-018','edit');


            if(!(new MaintenancePettyHelper)->can_edit_or_delete($id)){
                return $this->sendError( trans('error.update_fails'), []);
            }

            MaintenancePetty::update_maintenance_petty_m(array('maintenance_petty_active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }
}
