<?php

namespace App\Http\Controllers\API\V1\Finance;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;

use App\Classes\General\Upload;
use App\Models\Finance\AccountTree;
use App\Models\Finance\CostCenter;
use App\Models\Finance\MultiVoucher;
use App\Models\Finance\Treasury;
use App\Models\Main\Bank;
use App\Models\Main\ScreenSub;
use App\Models\Sales\Customer;
use App\Models\Finance\Tax;
use App\Models\Sales\Delegate;
use App\Models\Finance\ExpenseCode;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\DB;

class MultiVoucherAttachController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('03-051','view');
        $this->filter = array(

        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-051', $lang)->sub_title ?? '';

        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('03-051','add');

        $this->return['screen_name']       = ScreenSub::get_screen_sub_by_screen_code('03-051', $lang)->sub_title ?? '';

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('03-051','add');

        $item = $request->all();

            $check  = (new Upload)->uploadFile($item['attachment'] , 'multi_voucher' );
            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );

            DB::table('multi_voucher_d')->where('multi_voucher_d_id',$item['d_id'])->update(['attachment' => $check['url']]);

        return $this->sendResponse($this->return, trans('main.add_success'));

    }

    public function show($lang, $id)
    {
        $this->checkPermission('03-051','view');

        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('03-051' , $lang)->sub_title ?? '';
        $this->return['master']             = MultiVoucher::get_multi_voucher_m($id, $lang);
        $this->return['details']            = MultiVoucher::get_multi_voucher_d($id, $lang ,['dis_type'=>8]);
        $this->return['bank']               = Bank::all_banks(array('lang' => $lang, 'active' => 1 ));
        $this->return['purchases_vendors']  = [];
        $this->return['customers']          = Customer::all_customers(array('lang' => $lang, 'active' => 1));
        $this->return['account_trees']      = AccountTree::active_account_trees($lang, array('account_type' => 2, 'active' => 1));
        $this->return['cost_centers']       = CostCenter::active_cost_centers($lang, array('account_type' => 2, 'active' => 1));
        $this->return['treasurys']          = Treasury::all_treasury(['lang' => $lang, 'active' => 1, 'user' => auth()->user()->id]);
        $this->return['taxes']              = Tax::all_taxes(['lang' => $lang, 'active' => 1 ] );
        $this->return['currencys']          = DB::table('currencys')->selectRaw("currency_id as id , name_".$lang." as name")->get();
        $this->return['types']              = [["id" => 1 , "name" => "مقبوضات"],["id" => 2 , "name" => "مدفوعات"]];
        $this->return['attachments']        = MultiVoucher::get_attachments($id);
        $this->return['delegates']          = Delegate::all_delegates(array('active' => '1', 'lang' => $lang));
        $this->return['expense_codes']      = ExpenseCode::all_expense_codes(array('active' => '1', 'lang' => $lang));

        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('03-051','edit');

        $update = $request->all();
        DB::beginTransaction();

        DB::commit();

        return $this->sendResponse([], trans('main.update_success'));

    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('03-051','delete');

        $file = DB::table('multi_voucher_d')
            ->where('multi_voucher_d_id' , $id)
            ->first();

        DB::table('multi_voucher_d')
            ->where('multi_voucher_d_id' , $id)
            ->update(['attachment'=>null]);

        if(Storage::exists($file->attachment)){
            Storage::delete($file->attachment);
        }

        return $this->sendResponse( $this->return , trans('main.delete_success') );
    }


    public function download_attach($lang , $id){

        $this->checkPermission('03-051','view');

        $file = DB::table('multi_voucher_d')
            ->where('multi_voucher_d_id' , $id)
            ->first();

        return response()->file(public_path().$file->attachment);

    }
    public function add_details_attach($lang , $id , Request $request){
        $this->checkPermission('03-051','add');
        $item = $request->all();
            $check  = (new Upload)->uploadFile($item['attachment'] , 'multi_voucher' );
            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );

            DB::table('multi_voucher_d')->where('multi_voucher_d_id',$id)->update(['attachment' => $check['url']]);

        return $this->sendResponse( $this->return );
    }
    public function delete_details_attach($lang , $id , Request $request){
        $this->checkPermission('03-051','delete');
        $file = DB::table('multi_voucher_d')->where('multi_voucher_d_id',$id)->value('attachment');
        DB::table('multi_voucher_d')->where('multi_voucher_d_id',$id)->update(['attachment' => null]);

        if(Storage::exists($file)){
            Storage::delete($file);
        }
            return $this->sendResponse( $this->return , trans('main.delete_success') );
    }

}
