<?php

namespace App\Http\Controllers\API\V1\Finance;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Finance\Tax;
use Illuminate\Http\Request;
use App\Classes\Finance\TaxHelper;
use App\Models\Finance\AccountTree;

use App\Models\Main\ScreenSub;

class TaxController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('02-012','view');



        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'list'   => $request['list'] ?? null ,
            'rows'   => $request['rows'] ?? null ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('02-012', $lang)->sub_title ?? '';
        $this->return['items']       = Tax::all_taxes($this->filter);

        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('02-012','add');


        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('02-012', $lang)->sub_title ?? '';
        $this->return['account_tree']   = AccountTree::active_account_trees($lang, array('account_type' => 2, 'active' => '1'));

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('02-012','add');


        $insert             = $request->only(['name_en', 'name_ar', 'details', 'tax_percentage' , 'account_tree_id']);
        $insert['add_user'] = auth()->user()->id;

        $validator = validator()->make($insert, Tax::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        $this->return['id'] = Tax::add_tax($insert);
        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function show($lang, $id)
    {
        $this->checkPermission('02-012','view');


        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('02-012', $lang)->sub_title ?? '';
        $this->return['account_tree']   = AccountTree::active_account_trees($lang, array('account_type' => 2, 'active' => '1'));
        $this->return['data']           = Tax::get_tax($id, $lang);

        return $this->sendResponse($this->return);
    }

    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('02-012','edit');


        if(!(new TaxHelper)->can_edit_or_delete($id)){
            return $this->sendError(trans("error.update_fail"));
        }

        $update    = $request->only(['name_en', 'name_ar', 'details', 'tax_percentage' , 'account_tree_id']);

        $validator = validator()->make($update, Tax::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        Tax::update_tax($update, $id);
        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('02-012','delete');


        if(!(new TaxHelper)->can_edit_or_delete($id)){
            return $this->sendError(trans("error.delete_fail"));
        }

        Tax::delete_tax($id);
        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function toggle_active($lang, $id)
    {
        $item = Tax::find($id);

        if($item->tax_active){
            $this->checkPermission('02-012','delete');


            if(!(new TaxHelper)->can_edit_or_delete($id)){
                return $this->sendError(trans('error.delete_fails'), []);
            }

            Tax::delete_tax($id);
            $this->return['active'] = false;
        }else{
            $this->checkPermission('02-012','edit');

            if(!(new TaxHelper)->can_edit_or_delete($id)){
                return $this->sendError(trans('error.update_fails'), []);
            }

            Tax::update_tax(array('tax_active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }
}
