<?php

namespace App\Http\Controllers\API\V1\Finance;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;

use App\Classes\Finance\TreasuryHelper;

use App\Models\Finance\Treasury;
use App\Models\Finance\AccountTree;
use App\Models\Main\ScreenSub;
use App\Models\User;



class TreasuryController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('02-006','view');


        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? null ,
            'word'        => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('02-006' , $lang)->sub_title ?? '';
        $this->return['items']       = Treasury::all_treasury( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('02-006','add');
        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('02-006' , $lang);
        $this->return['account_trees']      = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['users']              = User::all_users( array( 'user_active' => 1 , 'lang' => $lang ) );

        return $this->sendResponse( $this->return  );
    }

    public function store(Request $request)
    {
        $this->checkPermission('02-006','add');

        $insert                 = $request->only(['treasury_name_en', 'treasury_name_ar', 'users',
                                                  'account_tree_id', 'first_balance', 'first_balance_date',
                                                  'first_balance_type']);
        $insert['add_user']     = auth()->user()->id;
        $insert['users']        = ( is_array($insert['users'] ?? []) ) ? implode(',', $insert['users'] ?? [] ) : $insert['users'] ;
        $validator              = validator()->make( $insert , Treasury::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        $this->return['id']     = Treasury::add_treasury($insert);
        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id, Request $request){
        $this->checkPermission('02-006','view');
        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('02-006' , $lang)->sub_title ?? '';
        $this->return['item']           = Treasury::get_treasury($id);
        $this->return['users']          = User::all_users( array( 'user_active' => 1 , 'lang' => $lang ) );
        $this->return['account_trees']  = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));

        return $this->sendResponse( $this->return , [] );

    }

    public function update($lang , $id, Request $request)
    {
        $this->checkPermission('02-006','edit');

        $update         = $request->only(['treasury_name_en', 'treasury_name_ar', 'users', 'account_tree_id']);
        $update['users']= ( is_array($update['users'] ?? []) ) ? implode(',', $update['users'] ?? [] ) : $update['users'] ;
        $update_seed    = collect($update)->only('treasury_name_ar' , 'treasury_name_en' , 'users')->all() ;
        $rules          = collect(Treasury::$rules)->only('treasury_name_ar' , 'treasury_name_en')->all() ;
        $validator      = validator()->make( $update  , $rules);
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        Treasury::update_treasury($update ,  $id);
        return $this->sendResponse( $this->return , trans('main.update_success') );

    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('02-006','delete');

        if( !(new TreasuryHelper)->can_edit_account_code($id) ){
            return $this->sendError( trans('error.delete_fails') , [] );
        }

        Treasury::delete_treasury($id);
        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function sort_post($lang , Request $request){
        $this->checkPermission('02-006','edit');

        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            Treasury::update_treasury(array('treasury_order' => $key ),  $value['id']);
        }

        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function toggle_active($lang , $id){

        $item = Treasury::find($id);
        if($item->treasury_active){

            $this->checkPermission('02-006','delete');

            Treasury::delete_treasury($id);
            $this->return['active'] = false;

        }else{
            $this->checkPermission('02-006','edit');

            Treasury::update_treasury(array('treasury_active' => 1 ) , $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }


}
