<?php

namespace App\Http\Controllers\API\V1\Main;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;

use App\Models\Main\Drafts;



class DraftController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {

        $this->filter = array(
            'lang'        => $lang ,
            'screen_code' => $request['screen_code']?? null,
            'branch_id'   => request()->header('branchid') ?? $this->data['current_branch'],
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? null
        );

        $this->return['items']       = Drafts::all_drafts($this->filter);

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {

        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {

        $inputs    = $request->all();

           $insert =  [
            'branch_id'     => request()->header('branchid') ?? $this->data['current_branch'],
            'screen_code'   => $inputs['screen_code'],
            'form_data'     => $inputs['form_data'],
            'add_user'      => auth()->user()->id,
        ];

        $validator = validator()->make( $insert , Drafts::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        $this->return['id'] = Drafts::add_Draft($insert);
        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {

        $this->return['data']               = Drafts::find($id);

        return $this->sendResponse( $this->return , '' );
    }

    public function update($lang , $id , Request $request)
    {
        $inputs    = $request->all();
        $update =  [
            'branch_id'     => request()->header('branchid') ?? $this->data['current_branch'],
            'screen_code'   => $inputs['screen_code'],
            'form_data'     => $inputs['form_data'],
        ];

        $validator  = validator()->make( $update , Drafts::$rules );
            if($validator->fails())
            {
                return $this->sendError( trans('error.add_fails') , $validator->errors() );
            }
        Drafts::update_Draft($update , $id );
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function destroy($lang , $id)
    {

        Drafts::delete_draft($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }


    public function toggle_active($lang , $id){
        $item = Drafts::find($id);
        if($item->Drafts_active){

            Drafts::delete_draft($id);
            $this->return['active'] = false;

        }else{

            Drafts::update_draft(array('Drafts_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

}
