<?php

namespace App\Http\Controllers\API\V1\main;

use App\Http\Controllers\API\V1\BaseController;
use App\Models\Main\NotAllowedTime;
use App\Models\Main\ScreenSub;
use Illuminate\Http\Request;

class NotAllowedTimeController extends BaseController
{
    public function store(Request $request)
    {

//        $this->checkPermission('01-008','add');

        $validator = validator()->make($request->all(), NotAllowedTime::$rules);

        if ($validator->fails()) {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        $times = $request['times'] ?? [];
        $user_id = $request['user_id'];
        foreach ($times as $time) {

            $insert = array(
                "day"       => $time['day'] ?? null,
                "user_id"   => $user_id ?? 0,
                "from_time" => $time['from_time'] ?? null ,
                "to_time"   => $time['to_time'] ?? null
            );
            $insert['add_user'] = auth()->user()->id;


            NotAllowedTime::add_not_allowed_time($insert);

        }

        return $this->sendResponse([], trans('main.add_success'));

    }


    public function show($lang, $user_id)
    {
//        $this->checkPermission('01-008','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('01-008', $lang)->sub_title ?? '';
        $this->return['data']           = NotAllowedTime::get_not_allowed_time($user_id, $lang);

        return $this->sendResponse($this->return);
    }
    public function update($lang, $user_id, Request $request)
    {
//        $this->checkPermission('01-008','edit');
        $validator = validator()->make($request->all(), NotAllowedTime::$rules);

        if ($validator->fails()) {
            return $this->sendError(trans('error.update_fails'), $validator->errors());
        }
        $times = $request['times'] ?? [];
        NotAllowedTime::delete_not_allowed_time($user_id);
        foreach ($times as $time) {

            $insert = array(
                "day"       => $time['day'],
                "user_id"   => $user_id,
                "from_time" => date("H:i:s" , strtotime($time['from_time'])),
                "to_time"   => date("H:i:s" , strtotime($time['to_time']))
            );
            $insert['add_user'] = auth()->user()->id;


            NotAllowedTime::add_not_allowed_time($insert);

        }

        return $this->sendResponse([], trans('main.update_success'));

    }

    public function destroy($lang, $user_id, Request $request)
    {
//        $this->checkPermission('01-008','delete');
        $not_allowed_time_id = $request->not_allowed_time_id ?? null;

        NotAllowedTime::delete_not_allowed_time($user_id,$not_allowed_time_id);
        return $this->sendResponse([], trans('main.delete_success'));
    }
}
