<?php

namespace App\Http\Controllers\API\V1\Main;

use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;

use App\Models\Main\ScreenSub;
use App\Models\Main\SalesAgent;
use App\Models\Main\Branch;
use App\Models\Finance\AccountTree;


class SalesAgentController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('01-013','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('01-013' , $lang)->sub_title ?? '';
        $this->return['items']       = SalesAgent::all_sales_agents( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('01-013','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('01-013' , $lang)->sub_title ?? '';
        $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['branchs']        = Branch::all_branchs( array('lang' => $lang , 'active' => 1 ) );
        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('01-013','add');

        $insert             = $request->only(['name_en', 'name_ar', 'account_tree_id', 'branch_id']);
        $insert['add_user'] = auth()->user()->user_id;
        $validator          = validator()->make( $insert , SalesAgent::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        $this->return['id'] = SalesAgent::add_sales_agent($insert);

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('01-013','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('01-013' , $lang)->sub_title ?? '';
        $this->return['data']           = SalesAgent::find($id);
        $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['branchs']        = Branch::all_branchs( array('lang' => $lang , 'active' => 1 ) );

        return $this->sendResponse( $this->return , '' );
    }

    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('01-013','edit');

        $update     = $request->only(['name_en', 'name_ar', 'account_tree_id', 'branch_id']);
        $validator  = validator()->make( $update , SalesAgent::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        SalesAgent::update_sales_agent($update ,  $id );
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('01-013','delete');

        SalesAgent::delete_sales_agent($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function sort_post($lang , Request $request){
        $this->checkPermission('01-013','edit');

        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            SalesAgent::update_sales_agent(array('sales_agent_order' => $key ),  $value['id']);
        }
        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function toggle_active($lang , $id){
        $item = SalesAgent::find($id);
        if($item->sales_agent_active){

            $this->checkPermission('01-013','delete');

            SalesAgent::delete_sales_agent($id);
            $this->return['active'] = false;

        }else{

            $this->checkPermission('01-013','edit');

            SalesAgent::update_sales_agent(array('sales_agent_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }
}
