<?php

namespace App\Http\Controllers\API\V1\Main;
use App\Http\Controllers\API\V1\BaseController;
use App\Classes\General\Upload;
use Illuminate\Http\Request;

use App\Models\Finance\AccountTree;
use App\Models\Main\Setting;
use App\Models\Main\ScreenSub;
use App\Models\Finance\Tax;


class SettingController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('01-010','view');


        $this->filter = array(
            'lang'        => $lang ,
            // 'active'      => $request['active'] ?? 1 ,
            // 'rows'        => $request['rows'] ?? null ,
            // 'page'        => $request['page'] ?? 1 ,
            'main_id'     => ScreenSub::get_screen_sub($request['sub_id'] ?? null , $lang)->main_id ?? 0 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('01-010' , $lang)->sub_title ?? '';
        $this->return['settings']       = [];
        $items                          = Setting::all_settings( $this->filter )->groupBy('category_id');
        $this->return['loockups']       = [];

        foreach ($items as $key => $value) {
            $this->return['settings'][] = array(
                'id'    => $value[0]->category_id ,
                'title' => $value[0]->category_title ,
                'settings' => $value
            );
        }

        $this->return['loockups']['accounts']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['loockups']['taxes']      = Tax::all_taxes([ 'lang' => $lang , 'account_type' => 2 , 'active' => 1 ]);

        foreach ($this->return['loockups']['accounts'] as $key => $value) {
            $this->return['loockups']['accounts'][$key]->id = (String) $this->return['loockups']['accounts'][$key]->id ;
        }

        foreach ($this->return['loockups']['taxes'] as $key => $value) {
            $this->return['loockups']['taxes'][$key]->id = (String) $this->return['loockups']['taxes'][$key]->id ;
        }

        $this->return['loockups']['list']       = array(
            array(
                'id'    => "1" ,
                'name'  => trans('main.yes')
            ),
            array(
                'id'    => "0" ,
                'name'  => trans('main.no')
            )
        );


        return $this->sendResponse( $this->return );
    }

    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('01-010','edit');
        $update     = [];
        $items      = $request->only('items');
        foreach ($items['items'] as $row) {
            if( $row['old_value'] != $row['new_value'] ){

                $item = Setting::get_setting($row['id'] );
                if( $item->type == 'file'  ){

                    if(isset($row['new_value']) && strlen($row['new_value']) > 100){
                        $upload = new Upload;
                        $check  = $upload->uploadBase64Image($row['new_value'] , 'setting' );
                        if(!$check['check'])
                            return $this->sendError( trans('error.add_fails') , $check['msg'] );
                        else
                            $update['value'] = $check['url'];
                    }

                }else{
                    $update['value'] = $row['new_value'];
                }

                Setting::update_setting($update['value'] , $row['id']);

            }
        }

        $this->return = Setting::get_setting($row['id'] );
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }


}
