<?php

namespace App\Http\Controllers\API\V1\Sales;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Sales\CovenantType;
use App\Models\Main\ScreenSub;
use Illuminate\Http\Request;

class CovenantTypeController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('04-002','view');


        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'rows'   => $request['rows'] ?? null ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('04-002', $lang)->sub_title ?? '';
        $this->return['items']       = CovenantType::all_covenant_types($this->filter);

        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('04-002','add');

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('04-002', $lang)->sub_title ?? '';

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('04-002','add');

        $insert = $request->only(['name_en', 'name_ar']);
        $insert['add_user'] = auth()->user()->id;

        $validator = validator()->make($insert, CovenantType::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        $this->return['id'] = CovenantType::add_covenant_type($insert);
        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function show($lang, $id)
    {
        $this->checkPermission('04-002','view');

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('04-002', $lang)->sub_title ?? '';
        $this->return['data']        = CovenantType::get_covenant_type($id, $lang);

        return $this->sendResponse($this->return);
    }

    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('04-002','edit');

        $update    = $request->only(['name_en', 'name_ar']);

        $validator = validator()->make($update, CovenantType::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        CovenantType::update_covenant_type($update, $id);
        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('04-002','delete');

        CovenantType::delete_covenant_type($id);
        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function toggle_active($lang, $id)
    {
        $item = CovenantType::find($id);

        if($item->covenant_type_active){
            $this->checkPermission('04-002','delete');

            CovenantType::delete_covenant_type($id);
            $this->return['active'] = false;
        }else{
            $this->checkPermission('04-002','edit');

            CovenantType::update_covenant_type(array('covenant_type_active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }
}
