<?php
namespace App\Http\Controllers\API\V1\Sales;

use App\Http\Controllers\API\V1\BaseController;
use App\Classes\Finance\CustomerStatmentReportHelper;
use App\Classes\General\General;
use App\Models\Main\Bank;
use App\Models\Sales\CustomerClaim;
use Illuminate\Http\Request;
use App\Models\Main\ScreenSub;
use App\Models\Sales\Clearance;
use Illuminate\Support\Facades\DB;


class CustomerClaimController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function store($lang, Request $request)
    {
        // $this->checkPermission('03-032','add');
        $clearance_id = $request['clearance_id'] ?? 0;
        if($request['clearance_id'] == 0 ){
            return $this->sendError( 'clearance ID is required', [], 200);
        }

        $get_customer_claim = DB::table('customer_claims_m')->where('clearance_id',$request['clearance_id'])->count();
        if ($get_customer_claim > 0){
            return $this->sendError( trans('error.duplicate_claim') , [] );
        }

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-032', $lang)->sub_title ?? '';
        if(collect($request['deleget_amounts'])->sum('amount') == 0 && collect($request['deleget_amounts'])->count('amount') == 0   ){
            return $this->sendError('No Amounts found', [], 200);
        }

        $year   = array_search( date("Y" , strtotime($request['customer_claim_date'] ??  date("Y-m-d") )) , $this->data['years'] );
        if($year == ''){
            return $this->sendError( trans("error.finance_year_not_aval"));
        }
        if((new General)->check_financial_year($request['customer_claim_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($request['customer_claim_date']));
        }
        DB::beginTransaction();

        $get_clearance    = Clearance::get_clearance($clearance_id , $lang);

        $id = CustomerClaim::add_customer_claim([
            'customer_balance'  => $request['customer_balance'] ?? 0 ,
            'clearance_id'      => $clearance_id  ,
            'finance_year_id'   => $year ,
            'port_id'           => $get_clearance->port_id ,
            'customer_id'       => $get_clearance->customer_id ,
            'customer_claim_date'      => $request['customer_claim_date'] ?? date("Y-m-d") ,
            'total_delegate'    => collect($request['deleget_amounts'])->sum('amount')  ?? 0,
            'total_cost'        => collect($request['extra_amounts'])->sum('amount_with_vat') ?? 0 ,
            'total_cost_without_vat'    => collect($request['extra_amounts'])->sum('amount_without_vat') ?? 0 ,
            'total_amount'      => (collect($request['extra_amounts'])->sum('amount_with_vat') + collect($request['deleget_amounts'])->sum('amount'))  ?? 0 ,
            'vat_per'           => 15 ,
            'add_user'          => auth()->guard('api')->user()->id,
        ]);
        foreach ($request->extra_amounts as $key => $row) {
            $vat_per = 15;
            $amount_with_vat = $row['amount_with_vat']  ?? 0;
            $amount_without_vat =  $row['amount_without_vat'] ?? 0;
            $vat_amount = ($amount_with_vat - $amount_without_vat ) ?? 0 ;

            $insert_revenue = array(
                'customer_claim_m_id' => $id,
                'revenue_id' => $row['revenue_id'] ?? 0 ,
                'amount_with_vat' => $amount_with_vat,
                'vat_per' => $vat_per ,
                'vat_amount' => $vat_amount,
                'amount_without_vat' => $amount_without_vat,
            );
            CustomerClaim::add_customer_claim_d( $insert_revenue );


        }

        foreach ($request['deleget_amounts'] as $key => $row) {
            $insert_expense_code = array(
                'customer_claim_m_id'          => $id ,
                'expense_code_id'       => $row['expense_code_id'] ?? 0,
                'receipt_number'        => $row['receipt_number'] ?? 0 ,
                'amount'                => $row['amount'] ?? 0,
                'description_ar'        => $row['description'] ?? '' ,
                'description_en'        => $row['description'] ?? ''
            );
            DB::table('customer_claims_delegate_d')->insert($insert_expense_code);
        }

        DB::commit();

        return $this->sendResponse($id, trans('main.add_success'));
    }

    public function show($lang, $clearance_id)
    {
       // $this->checkPermission('03-032','view');
        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('03-032', $lang)->sub_title ?? '';
        $this->return['is_customer_claim']  = (DB::table('customer_claims_m')->where('clearance_id',$clearance_id)->count() > 0) ? 1 :0;
        $this->return['clearance']          = Clearance::get_clearance($clearance_id,$lang);
        $this->return['clearance_d']        = Clearance::get_clearance_d($clearance_id , $lang);
        // get goods_type
        $classification_m                   = DB::table('classifications_m')->where('clearances_m_id',$clearance_id)->first();
        $this->return['first_item_of_classification_detail'] = null;
        if (!is_null($classification_m)){
            $this->return['first_item_of_classification_detail']        = DB::table('classifications_d')->where('classifications_m_id',$classification_m->classification_id)->first()->item_name;
        }
        $this->return['bank']               = Bank::all_banks(array('lang' => $lang, 'active' => 1,'show_invoice' => 1 ));

        $this->return['prices']             = [];
        $prices                             = DB::table('customer_prices')
            ->where('customer_id' , $this->return['clearance']->customer_id)
            ->where('customer_price_active' , 1 )
            ->first();
        $transport_prices                   = DB::table('customer_transport_prices')
            ->where('customer_id' , $this->return['clearance']->customer_id)
            ->where('customer_transport_price_active' , 1 )
            ->first();

        $this->return['prices'][]           = [
            'title' => 'اجور التخليص' ,
            'price' => $prices->cont_b_first ?? 0 ,
            'rev_id'=> 1
        ];

        $this->return['prices'][]           = [
            'title' => 'اجور النقل' ,
            'price' => $transport_prices->container_40 ?? 0 ,
            'rev_id'=> 5
        ];

        $this->return['prices'][]           = [
            'title' => 'اجور تحميل وتفريغ' ,
            'price' => $prices->loading_price ?? 0 ,
            'rev_id'=>  2
        ];

        $this->return['prices'][]           = [
            'title' => 'مصاريف متنوعه' ,
            'price' => $prices->other_price ?? 0 ,
            'rev_id'=> 4
        ];



        if ($this->return['is_customer_claim']){
            $this->return['master']             = CustomerClaim::get_customer_claim($clearance_id, $lang);
            $this->return['details']            = CustomerClaim::get_customer_claim_d($this->return['master']->customer_claim_id, $lang);
            $this->return['delegate_details']   = CustomerClaim::get_customer_claim_delegate_d($this->return['master']->customer_claim_id, $lang);
            $this->return['customer_balance']   = $this->return['master']->customer_balance;
        }else{
            $this->return['details']            = DB::table('revenues')
                ->selectRaw('revenue_id , revenue_name_'.$lang.' as revenue_name , 0 as amount_with_vat, 0 as amount_without_vat ')
                ->where('revenues_active',1)
                ->whereNotNull('revenues_order')
                ->orderBy('revenues_order' , 'asc')
                ->get();
            $this->return['delegate_details']   = DB::table('expense_codes')
                ->selectRaw('name_'.$lang.' as expense_code_name , 0 as amount , expense_codes.expense_code_id ')
                ->where('expense_code_active',1)
                ->whereNotNull('expense_codes_order')
                ->orderBy('expense_codes_order', 'asc')
                ->get();

            $this->return['customer_balance']   = CustomerStatmentReportHelper::get_account_customer_statment($this->return['clearance']->customer_id , $lang ,  array('total' => true , 'before' => date("Y-m-d"),'with_draft' => 0) , null)->total ?? 0 ;

        }

        return $this->sendResponse($this->return);
    }

    public function update($lang,$clearance_id,Request $request)
    {
        // $this->checkPermission('03-032','edit');
        $get_customer_claim = CustomerClaim::get_customer_claim($clearance_id,$lang);
        if((new General)->check_financial_year($get_customer_claim->customer_claim_date) != "true"){
            return $this->sendError((new General)->check_financial_year($get_customer_claim->customer_claim_date));
        }
        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-032', $lang)->sub_title ?? '';


        if(collect($request['deleget_amounts'])->sum('amount') == 0 && count($request['extra_amounts']) == 0   ){
            return $this->sendError('No Amounts found', [], 200);
        }

        DB::beginTransaction();

            CustomerClaim::update_customer_claim([
                'customer_balance'      => $request['customer_balance'] ?? 0 ,
                'total_delegate'        => collect($request['deleget_amounts'])->sum('deleget_amounts') ,
                'total_cost'            => collect($request['amounts'])->sum('amount_with_vat') ,
                'total_amount'          => collect($request['amounts'])->sum('amount_with_vat')+collect($request['deleget_amounts'])->sum('deleget_amounts'),
                'total_journals'        => collect($request['journals'])->sum('amount') ,
                'total_cost_without_vat'=> collect($request['amounts'])->sum('amount_without_vat') ,
            ],$get_customer_claim->customer_claim_id);

            CustomerClaim::hard_delete_customer_claim_d($get_customer_claim->customer_claim_id);
            foreach ($request->extra_amounts as $key => $row) {
                $vat_per = 15;
                $amount_with_vat = $row['amount_with_vat']  ?? 0;
                $amount_without_vat =  $row['amount_without_vat'] ?? 0;
                $vat_amount = ($amount_with_vat - $amount_without_vat ) ?? 0 ;

                $update_revenue = array(
                    'customer_claim_m_id' => $get_customer_claim->customer_claim_id,
                    'revenue_id' => $row['revenue_id']  ?? 0,
                    'amount_with_vat' => $amount_with_vat,
                    'vat_per' => $vat_per,
                    'vat_amount' => $vat_amount,
                    'amount_without_vat' => $amount_without_vat ?? 0,
                );
                CustomerClaim::add_customer_claim_d( $update_revenue );
            }

            CustomerClaim::hard_delete_customer_claim_delegate_d($get_customer_claim->customer_claim_id);
            foreach ($request['deleget_amounts'] as $key => $row) {
                DB::table('customer_claims_delegate_d')->insert([
                    'customer_claim_m_id'          => $get_customer_claim->customer_claim_id ,
                    'expense_code_id'       => $row['expense_code_id'] ,
                    'receipt_number'        => $row['receipt_number'] ?? 0 ,
                    'amount'                => $row['amount'] ,
                    'description_ar'        => $row['description'] ?? '' ,
                    'description_en'        => $row['description'] ?? ''
                ]);
            }

        DB::commit();

        return $this->sendResponse([], trans('main.update_success'));
    }

    public function destroy($lang, $customer_claim_id)
    {
        // $this->checkPermission('03-032','delete');
        $item = DB::table('customer_claims_m')->where('customer_claim_id',$customer_claim_id)->first();


        DB::beginTransaction();
        if((new General)->check_financial_year($item->customer_claim_date) != "true"){
            return $this->sendError((new General)->check_financial_year($item->customer_claim_date));
        }
        CustomerClaim::hard_delete_customer_claim_m($customer_claim_id);
        CustomerClaim::hard_delete_customer_claim_d($customer_claim_id);
        CustomerClaim::hard_delete_customer_claim_delegate_d($customer_claim_id);

        DB::commit();

        return $this->sendResponse([], trans('main.delete_success'));
    }


}
