<?php

namespace App\Http\Controllers\API\V1\Sales;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Sales\CustomerFile;
use App\Models\Sales\CustomerInfo;
use Illuminate\Http\Request;
use App\Classes\General\Upload;

use App\Models\Finance\AccountTree;
use App\Models\Main\Delegate;
use App\Models\Sales\Customer;
use App\Models\Main\ScreenSub;
use App\Models\Main\Setting;

use App\Models\Trans\Area;
use App\Models\Trans\Port;


use DB;
use Illuminate\Support\Arr;

class CustomerController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('04-003','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null,
            'is_main'        => $request['is_main'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('04-003' , $lang)->sub_title ?? '';
        $this->return['items']       = Customer::all_customers( $this->filter );

        foreach($this->return['items'] as $key => $row){
            if(strlen($row->customer_token ) > 10 ){
                $this->return['items'][$key]->link = "https://updatecustomer.arabdivs.com?clientid=".request()->header('clientID')."&token=".$row->customer_token;
            }
        }

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('04-003','add');

        $this->return['extra_inputs']   = Customer::get_dynamic_inputs();
        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-003' , $lang)->sub_title ?? '';
        $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['delegates']      = Delegate::all_delegates(array('lang' => $lang  , 'active' => 1 ));
        $this->return['default_account']= intval( Setting::get_main_setting_by_code('default_customer_account')->value ?? 0 );
        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('04-003','add');
        $insert = $request->only(['name_en', 'name_ar', 'mobile', 'address', 'vat',
                                  'vat_register_number', 'location', 'delegate_id',
                                  'collection_delegate_id', 'account_tree_id',
                                  'customer_type', 'id_number', 'email' ,'credit_period'  , 'notes' , 'issuing_invoice' , 'alireda_code','extra_inputs','customer_main_id','is_main',
                                  'add_building_no' ,'add_street' ,'add_sec_no' ,'add_district' ,'add_postcode' ,'add_city' , 'add_country' ]);



        $insert['account_tree_id']  = $insert['account_tree_id'] ?? Setting::get_main_setting_by_code('default_customer_account')->value ?? 0 ;
        $validator = validator()->make( $insert , Customer::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        if( isset($request->commercial_register_file) || isset($request->prices_list_file) || isset($request->iban_file) || isset($request->other_file) ){
            $insert['issuing_invoice'] = 1;
        }

        if(isset($request['commercial_register_file']) && strlen($request['commercial_register_file']) > 100){

            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['commercial_register_file'] , 'customers' );

            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            else
                $insert['commercial_register_file'] = $check['url'];
        }

        if(isset($request['prices_list_file']) && strlen($request['prices_list_file']) > 100){
            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['prices_list_file'] , 'customers' );

            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            else
                $insert['prices_list_file'] = $check['url'];
        }


        if(isset($request['iban_file']) && strlen($request['iban_file']) > 100){
            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['iban_file'] , 'customers' );

            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            else
                $insert['iban_file'] = $check['url'];
        }

        if(isset($request['other_file']) && strlen($request['other_file']) > 100){
            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['other_file'] , 'customers' );

            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            else
                $insert['other_file'] = $check['url'];
        }


        $insert['add_user'] = auth()->user()->id;
        $this->return['id'] = Customer::add_customer($insert);

        if(isset($request->customer_contacts)){
            $customer_contacts =  $request['customer_contacts'];
            foreach ($customer_contacts as $customer_contact){
                $seed['customers_info_name']  = $customer_contact['customers_info_name'] ?? null;
                $seed['customers_info_email']  = $customer_contact['customers_info_email'] ?? null;
                $seed['customers_info_phone']  = $customer_contact['customers_info_phone']  ?? null;
                $seed['country_code']          = $customer_contact['country_code']  ?? null;

                $seed['customer_id']  = $this->return['id'];
                $seed['add_user']  = auth()->user()->id;
                CustomerInfo::add_customer_info($seed);
            }
        }

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function download_file($lang , $id , Request $request){
        $this->checkPermission('04-003','view');
        $file = CustomerFile::get_file($id,$lang);

        if(file_exists(public_path($file->file_path ?? '' ))){
            return response()->download(public_path($file->file_path ?? '' ));
        }

        return $this->sendError( '404 File not found ' , '' );

    }

    public function upload_file($lang,$id,Request $request){
        $request->only(['file']);
        $validator = validator()->make( $request->all() , [
            'file_path' => 'required'
        ]);

        $request['folder'] = 'customer';


        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        if(isset($request['file_path'])){
            $upload = new Upload;
            $check  = $upload->uploadFile($request['file_path'] , $request['folder'], $request['attachment_name'] );

            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            else
            $insert = array('file_name' => $request->file_name,'file_path' => $check['url'],'customer_id' => $id,'add_user'=>auth()->user()->id,'add_date' => date('Y-m-d H:i:s'));
           $check['customer_file_id'] =  CustomerFile::add_customer_file($insert);
            return  $this->sendResponse($check);
        }
    }

    public function delete_file($lang,$id){
        $this->checkPermission('04-003','delete');
        CustomerFile::delete_customer_file($id);

            return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function check_customer_name($lang , Request $request)
    {
        $this->checkPermission('04-003','view');

        if (isset($request->name_ar)){
           $this->return = DB::table('customers')->where('name_ar',$request->name_ar)->get();
        }elseif (isset($request->name_en)){
            $this->return = DB::table('customers')->where('name_en',$request->name_en)->get();
        }

        return $this->sendResponse($this->return ?? []);
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('04-003','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-003' , $lang)->sub_title ?? '';
        $this->return['data']           = Customer::get_customer($id,$lang);
        $this->return['client_attachments'] = CustomerFile::get_customer_file($id,$lang);
        $this->return['customer_contacts'] = CustomerInfo::all_customer_info($id,$lang);
        $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['areas']          = Area::all_areas(['lang' => $lang, 'active' => 1 ]);
        $this->return['delegates']      = Delegate::all_delegates(array('lang' => $lang  , 'active' => 1 ));


        $this->return['prices']         = DB::table('ports')
            ->leftjoin('customer_prices' , function($join) use($id){
                $join->on('ports.port_id' , '=' , 'customer_prices.port_id')
                    ->where('customer_prices.customer_id' , '=' , $id)
                    ->where('customer_prices.customer_price_active' , '=' , 1);
            })
            ->groupBy('ports.port_id')
            ->selectRaw('
                ports.port_id ,
                ports.port_name_'.$lang.' as port_name ,
                ifnull(cont_s_first , 0) as cont_s_first ,
                ifnull(cont_s_sec , 0) as cont_s_sec ,
                ifnull(cont_s_other , 0) as cont_s_other ,
                ifnull(cont_b_first , 0) as cont_b_first ,
                ifnull(cont_b_sec , 0) as cont_b_sec ,
                ifnull(cont_b_other , 0) as cont_b_other ,
                ifnull(loading_type , 0) as loading_type ,
                ifnull(loading_price , 0) as loading_price ,
                ifnull(translate_type , 0) as translate_type ,
                ifnull(translate_price , 0) as translate_price ,
                ifnull(other_type , 0) as other_type ,
                ifnull(other_price , 0) as other_price
            ')
            ->get();


        $this->return['trans_prices']         = DB::table('customer_transport_prices')
            ->leftjoin('ports' , 'customer_transport_prices.port_id' , '=' , 'ports.port_id')
            ->leftjoin('areas as from_areas' , 'customer_transport_prices.from_area_id' , '=' , 'from_areas.area_id')
            ->leftjoin('areas as to_areas' , 'customer_transport_prices.to_area_id' , '=' , 'to_areas.area_id')
            ->where('from_area_id' , '>' , 0 )
            ->where('to_area_id' , '>' , 0 )
            ->where('customer_transport_prices.customer_id' , '=' , $id)
            ->selectRaw('
                ports.port_id ,
                ports.port_name_'.$lang.' as port_name ,

                from_area_id  ,
                from_areas.area_name_'.$lang.' as from_area_name ,
                to_area_id ,
                to_areas.area_name_'.$lang.' as to_area_name ,

                ifnull(price , 0) as price ,
                container_20 ,
                container_40,
                ifnull(price_special , 0) as price_special ,
                container_40_special ,
                container_20_special

            ')
            ->get();


        $this->return['warehouse_prices']         = DB::table('wearhouses')
            ->leftjoin('customer_warehouse_prices' , function($join) use($id){
                $join->on('wearhouses.wearhouse_id' , '=' , 'customer_warehouse_prices.warehouse_id')
                    ->where('customer_warehouse_prices.customer_id' , '=' , $id)
                    ->where('customer_warehouse_prices.customer_warehouse_price_active' , '=' , 1);
            })
            ->groupBy('wearhouses.wearhouse_id')
            ->where('wearhouses.wearhouse_active' ,1 )
            ->selectRaw('
                wearhouses.wearhouse_id ,
                wearhouses.wearhouse_name_'.$lang.' as wearhouse_name ,
                ifnull(cont_40 , 0) as cont_40 ,
                ifnull(cont_20 , 0) as cont_20 ,
                ifnull(cont_empty_40 , 0) as cont_empty_40 ,
                ifnull(cont_empty_20 , 0) as cont_empty_20 ,
                ifnull(loading_unloading , 0) as loading_unloading ,
                ifnull(lcl , 0) as lcl ,
                ifnull(warehouse_temp , 0) as warehouse_temp ,
                ifnull(customer_warehouse_prices.allowed_days , 0) as allowed_days
            ')
            ->get();

        return $this->sendResponse( $this->return , '' );
    }

    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('04-003','edit');

        $update = $request->only(['name_en', 'name_ar', 'mobile', 'address', 'vat',
                                   'vat_register_number', 'location', 'delegate_id',
                                   'collection_delegate_id', 'account_tree_id',
                                   'customer_type', 'id_number', 'email' , 'credit_period' , 'notes' , 'issuing_invoice' ,
            'alireda_code','customer_main_id','is_main' , 'add_building_no' ,'add_street' ,'add_sec_no' ,'add_district' ,'add_postcode' ,'add_city' , 'add_country' ]);

        $rules            = Customer::$rules;
        $rules['name_ar'] = $rules['name_ar'].',name_ar,'.$id.',customer_id';
        $rules['name_en'] = $rules['name_en'].',name_en,'.$id.',customer_id';

        $validator  = validator()->make( $update , $rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        if( isset($request->commercial_register_file) || isset($request->prices_list_file) || isset($request->iban_file) || isset($request->other_file) ){
            $update['issuing_invoice'] = 1;
        }


        if(isset($request['commercial_register_file']) && strlen($request['commercial_register_file']) > 100){
            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['commercial_register_file'] , 'customers' );

            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            else
                $update['commercial_register_file'] = $check['url'];
        }

        if(isset($request['prices_list_file']) && strlen($request['prices_list_file']) > 100){
            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['prices_list_file'] , 'customers' );

            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            else
                $update['prices_list_file'] = $check['url'];
        }


        if(isset($request['iban_file']) && strlen($request['iban_file']) > 100){
            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['iban_file'] , 'customers' );

            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            else
                $update['iban_file'] = $check['url'];
        }

        if(isset($request['other_file']) && strlen($request['other_file']) > 100){
            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['other_file'] , 'customers' );

            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            else
                $update['other_file'] = $check['url'];
        }

        Customer::update_customer($update ,  $id );

        if(isset($request->customer_contacts)){
            $customer_contacts =  $request['customer_contacts'];
            CustomerInfo::delete_customer_info($id);
            foreach ($customer_contacts as $customer_contact){
                $seed['customers_info_name']  = $customer_contact['customers_info_name'] ?? null;
                $seed['customers_info_email']  = $customer_contact['customers_info_email'] ?? null;
                $seed['customers_info_phone']  = $customer_contact['customers_info_phone']  ?? null;
                $seed['country_code']          = $customer_contact['country_code']  ?? null;
                $seed['customer_id']  = $id  ?? null;
                $seed['add_user']  = auth()->user()->id;
                CustomerInfo::add_customer_info($seed);
            }
        }

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }


    public function destroy($lang , $id)
    {
        $this->checkPermission('04-003','delete');

        Customer::delete_customer($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function sort_post($lang , Request $request){
        $this->checkPermission('04-003','edit');

        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            Customer::update_customer(array('customer_order' => $key ),  $value['id']);
        }
        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function toggle_active($lang , $id){
        $item = Customer::find($id);
        if($item->customer_active){

            $this->checkPermission('04-003','delete');

            Customer::delete_customer($id);
            $this->return['active'] = false;

        }else{
            $this->checkPermission('04-003','edit');


            Customer::update_customer(array('customer_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }


    public function toggle_customer_prices($lang , Request $request){
        $this->checkPermission('04-003','view');
        DB::beginTransaction();

            $data = $request->only([ "customer_id","port_id","cont_s_first","cont_s_sec","cont_s_other","cont_b_first","cont_b_sec","cont_b_other","loading_type","loading_price","translate_type","translate_price","other_type","other_price"]);


            $check = DB::table('customer_prices')
                ->where('customer_id' , $data['customer_id'])
                ->where('port_id' , $data['port_id'])
                ->first();

            if( isset($check->customer_price_id) ){
                $id = $check->customer_price_id;
                DB::table('customer_prices')
                    ->where('customer_id' , $data['customer_id'])
                    ->where('port_id' , $data['port_id'])
                    ->update( $data );
            }else{
                $insert['add_user'] = auth()->user()->user_id;
                $id = DB::table('customer_prices')
                    ->where('customer_id' , $data['customer_id'])
                    ->where('port_id' , $data['port_id'])
                    ->insertGetId( $data );
            }

            DB::table('customer_transport_prices')
                ->where('customer_id' , $data['customer_id'])
                ->where('port_id' , $data['port_id'])
                ->delete();

            foreach ($request['prices'] as $key => $row) {
                DB::table('customer_transport_prices')->insertGetId( [
                    'customer_id'   => $data['customer_id'] ,
                    'port_id'       => $data['port_id'] ,
                    'price'         => $row['price'] ,
                    'from_area_id'  => $row['from_area_id'] ,
                    'to_area_id'    => $row['to_area_id'] ,
                    'container_20'  => $row['container_20'] ,
                    'container_40'  => $row['container_40'],
                    'container_20_special'  => $row['container_20_special'] ,
                    'container_40_special'  => $row['container_40_special'],
                    'price_special'         => $row['price_special']
                ] );
            }

        DB::commit();

        $this->return['id'] = $id;
        return $this->sendResponse( $this->return , trans('main.update_success') );

    }


    public function toggle_customer_transport_prices($lang , Request $request){
        $this->checkPermission('04-003','view');

        $data   = $request->only([ "customer_id","port_id","from_area_id","to_area_id","price" , 'container_20' ,'container_40','price_special' , 'container_20_special','container_40_special']);
        $check  = DB::table('customer_transport_prices')
            ->where('customer_id' , $data['customer_id'])
            ->where('port_id' , $data['port_id'])
            ->where('from_area_id' , $data['from_area_id'])
            ->where('to_area_id' , $data['to_area_id'])
            ->first();

        if( isset($check->customer_transport_price_id) ){
            $id = $check->customer_transport_price_id;
            DB::table('customer_transport_prices')
                ->where('customer_id' , $data['customer_id'])
                ->where('port_id' , $data['port_id'])
                ->where('from_area_id' , $data['from_area_id'])
                ->where('to_area_id' , $data['to_area_id'])
                ->update( $data );
        }else{
            $insert['add_user'] = auth()->user()->user_id;
            $id = DB::table('customer_transport_prices')
                ->where('customer_id' , $data['customer_id'])
                ->where('port_id' , $data['port_id'])
                ->where('from_area_id' , $data['from_area_id'])
                ->where('to_area_id' , $data['to_area_id'])
                ->insertGetId( $data );
        }


        $this->return['id'] = $id;
        return $this->sendResponse( $this->return , trans('main.update_success') );




    }


    public function toggle_customer_warehouse_prices($lang , Request $request){
        $this->checkPermission('04-003','view');

        $data   = $request->only([ "customer_id","warehouse_id","cont_40","cont_20","cont_empty_40",'cont_empty_20','loading_unloading','lcl','warehouse_temp','allowed_days']);
        $check  = DB::table('customer_warehouse_prices')
            ->where('customer_id' , $data['customer_id'])
            ->where('warehouse_id' , $data['warehouse_id'])
            ->first();

        if( isset($check->customer_warehouse_price_id) ){
            $id = $check->customer_warehouse_price_id;
            DB::table('customer_warehouse_prices')
                ->where('customer_id' , $data['customer_id'])
                ->where('warehouse_id' , $data['warehouse_id'])
                ->update( $data );
        }else{
            $insert['add_user'] = auth()->user()->user_id;
            $id = DB::table('customer_warehouse_prices')->insertGetId( $data );
        }


        $this->return['id'] = $id;
        return $this->sendResponse( $this->return , trans('main.update_success') );




    }


    public function get_customer_contacts($lang , $id )
    {
        $this->checkPermission('04-012','view');
        $this->return['customer_contacts'] = CustomerInfo::all_customer_info($id,$lang);

        return $this->sendResponse( $this->return , '' );
    }
    
}
