<?php

namespace App\Http\Controllers\API\V1\Sales;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Sales\CustomerFile;
use App\Models\Sales\CustomerInfo;
use Illuminate\Http\Request;

use App\Models\Finance\AccountTree;
use App\Models\Main\Delegate;
use App\Models\Sales\Customer;
use App\Models\Main\ScreenSub;
use App\Models\Main\Setting;

use App\Models\Trans\Area;


use DB;

class CustomerPricesController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-045','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null,
            'is_main'     => $request['is_main'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-045' , $lang)->sub_title ?? '';
        $this->return['items']       = Customer::all_customers( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-045','add');

        $this->return['extra_inputs']   = Customer::get_dynamic_inputs();
        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-045' , $lang)->sub_title ?? '';
        $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['delegates']      = Delegate::all_delegates(array('lang' => $lang  , 'active' => 1 ));
        $this->return['default_account']= intval( Setting::get_main_setting_by_code('default_customer_account')->value ?? 0 );
        return $this->sendResponse( $this->return  );
    }


    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-045','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-045' , $lang)->sub_title ?? '';
        $this->return['data']           = Customer::get_customer($id,$lang);
        $this->return['client_attachments'] = CustomerFile::get_customer_file($id,$lang);
        $this->return['customer_contacts'] = CustomerInfo::all_customer_info($id,$lang);
        $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['areas']          = Area::all_areas(['lang' => $lang, 'active' => 1 ]);
        $this->return['delegates']      = Delegate::all_delegates(array('lang' => $lang  , 'active' => 1 ));


        $this->return['prices']         = DB::table('ports')
            ->leftjoin('customer_prices' , function($join) use($id){
                $join->on('ports.port_id' , '=' , 'customer_prices.port_id')
                    ->where('customer_prices.customer_id' , '=' , $id)
                    ->where('customer_prices.customer_price_active' , '=' , 1);
            })
            ->groupBy('ports.port_id')
            ->selectRaw('
                ports.port_id ,
                ports.port_name_'.$lang.' as port_name ,
                ifnull(cont_s_first , 0) as cont_s_first ,
                ifnull(cont_s_sec , 0) as cont_s_sec ,
                ifnull(cont_s_other , 0) as cont_s_other ,
                ifnull(cont_b_first , 0) as cont_b_first ,
                ifnull(cont_b_sec , 0) as cont_b_sec ,
                ifnull(cont_b_other , 0) as cont_b_other ,
                ifnull(loading_type , 0) as loading_type ,
                ifnull(loading_price , 0) as loading_price ,
                ifnull(lcl_loading_price , 0) as lcl_loading_price ,

                ifnull(translate_type , 0) as translate_type ,
                ifnull(translate_price , 0) as translate_price ,
                ifnull(lcl_translate_price , 0) as lcl_translate_price ,

                ifnull(other_type , 0) as other_type ,
                ifnull(other_price , 0) as other_price ,
                ifnull(lcl_other_price , 0) as lcl_other_price

            ')
            ->get();


        $this->return['trans_prices']         = DB::table('customer_transport_prices')
            ->leftjoin('ports' , 'customer_transport_prices.port_id' , '=' , 'ports.port_id')
            ->leftjoin('areas as from_areas' , 'customer_transport_prices.from_area_id' , '=' , 'from_areas.area_id')
            ->leftjoin('areas as to_areas' , 'customer_transport_prices.to_area_id' , '=' , 'to_areas.area_id')
            ->where('from_area_id' , '>' , 0 )
            ->where('to_area_id' , '>' , 0 )
            ->where('customer_transport_prices.customer_id' , '=' , $id)
            ->selectRaw('
                ports.port_id ,
                ports.port_name_'.$lang.' as port_name ,

                from_area_id  ,
                from_areas.area_name_'.$lang.' as from_area_name ,
                to_area_id ,
                to_areas.area_name_'.$lang.' as to_area_name ,

                ifnull(price , 0) as price ,
                container_20 ,
                container_40,
                ifnull(price_special , 0) as price_special ,
                container_40_special ,
                container_20_special

            ')
            ->get();
        return $this->sendResponse( $this->return , '' );
    }

    public function update_customer_prices($lang , Request $request){
        $this->checkPermission('03-045','view');
        DB::beginTransaction();

            $data = $request->only([ "customer_id","port_id","cont_s_first","cont_s_sec","cont_s_other","cont_b_first","cont_b_sec","cont_b_other","loading_type","loading_price","lcl_loading_price","translate_type","translate_price","lcl_translate_price","other_type","other_price" ,"lcl_other_price"]);


            $check = DB::table('customer_prices')
                ->where('customer_id' , $data['customer_id'])
                ->where('port_id' , $data['port_id'])
                ->first();

            if( isset($check->customer_price_id) ){
                $id = $check->customer_price_id;
                DB::table('customer_prices')
                    ->where('customer_id' , $data['customer_id'])
                    ->where('port_id' , $data['port_id'])
                    ->update( $data );
            }else{
                $insert['add_user'] = auth()->user()->user_id;
                $id = DB::table('customer_prices')
                    ->where('customer_id' , $data['customer_id'])
                    ->where('port_id' , $data['port_id'])
                    ->insertGetId( $data );
            }

            DB::table('customer_transport_prices')
                ->where('customer_id' , $data['customer_id'])
                ->where('port_id' , $data['port_id'])
                ->delete();

            foreach ($request['prices'] as $key => $row) {
                DB::table('customer_transport_prices')->insertGetId( [
                    'customer_id'   => $data['customer_id'] ,
                    'port_id'       => $data['port_id'] ,
                    'price'         => $row['price'] ,
                    'from_area_id'  => $row['from_area_id'] ,
                    'to_area_id'    => $row['to_area_id'] ,
                    'container_20'  => $row['container_20'] ,
                    'container_40'  => $row['container_40'],
                    'container_20_special'  => $row['container_20_special'] ,
                    'container_40_special'  => $row['container_40_special'],
                    'price_special'         => $row['price_special']
                ] );
            }

        DB::commit();

        $this->return['id'] = $id;
        return $this->sendResponse( $this->return , trans('main.update_success') );

    }

}
