<?php

namespace App\Http\Controllers\API\V1\Sales;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Sales\CustomerFile;
use App\Models\Sales\CustomerInfo;
use Illuminate\Http\Request;

use App\Models\Finance\AccountTree;
use App\Models\Main\Delegate;
use App\Models\Sales\Customer;
use App\Models\Main\ScreenSub;
use App\Models\Main\Setting;

use App\Models\Trans\Area;


use DB;

class CustomerWarehousePricesController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-046','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null,
            'is_main'     => $request['is_main'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-046' , $lang)->sub_title ?? '';
        $this->return['items']       = Customer::all_customers( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-046','add');

        $this->return['extra_inputs']   = Customer::get_dynamic_inputs();
        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-046' , $lang)->sub_title ?? '';
        $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['delegates']      = Delegate::all_delegates(array('lang' => $lang  , 'active' => 1 ));
        $this->return['default_account']= intval( Setting::get_main_setting_by_code('default_customer_account')->value ?? 0 );
        return $this->sendResponse( $this->return  );
    }


    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-046','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-046' , $lang)->sub_title ?? '';
        $this->return['data']           = Customer::get_customer($id,$lang);
        $this->return['client_attachments'] = CustomerFile::get_customer_file($id,$lang);
        $this->return['customer_contacts'] = CustomerInfo::all_customer_info($id,$lang);
        $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['areas']          = Area::all_areas(['lang' => $lang, 'active' => 1 ]);
        $this->return['delegates']      = Delegate::all_delegates(array('lang' => $lang  , 'active' => 1 ));

        $this->return['warehouse_prices']         = DB::table('wearhouses')
            ->leftjoin('customer_warehouse_prices' , function($join) use($id){
                $join->on('wearhouses.wearhouse_id' , '=' , 'customer_warehouse_prices.warehouse_id')
                    ->where('customer_warehouse_prices.customer_id' , '=' , $id)
                    ->where('customer_warehouse_prices.customer_warehouse_price_active' , '=' , 1);
            })
            ->groupBy('wearhouses.wearhouse_id')
            ->where('wearhouses.wearhouse_active' ,1 )
            ->selectRaw('
                wearhouses.wearhouse_id ,
                wearhouses.wearhouse_name_'.$lang.' as wearhouse_name ,
                ifnull(cont_40 , 0) as cont_40 ,
                ifnull(cont_20 , 0) as cont_20 ,
                ifnull(cont_empty_40 , 0) as cont_empty_40 ,
                ifnull(cont_empty_20 , 0) as cont_empty_20 ,
                ifnull(loading_unloading , 0) as loading_unloading ,
                ifnull(lcl , 0) as lcl ,
                ifnull(warehouse_temp , 0) as warehouse_temp ,
                ifnull(customer_warehouse_prices.allowed_days , 0) as allowed_days
            ')
            ->get();

        return $this->sendResponse( $this->return , '' );
    }

    public function update_customer_warehouse_prices($lang , Request $request){
        $this->checkPermission('03-046','view');

        $data   = $request->only([ "customer_id","warehouse_id","cont_40","cont_20","cont_empty_40",'cont_empty_20','loading_unloading','lcl','warehouse_temp','allowed_days']);
        $check  = DB::table('customer_warehouse_prices')
            ->where('customer_id' , $data['customer_id'])
            ->where('warehouse_id' , $data['warehouse_id'])
            ->first();

        if( isset($check->customer_warehouse_price_id) ){
            $id = $check->customer_warehouse_price_id;
            DB::table('customer_warehouse_prices')
                ->where('customer_id' , $data['customer_id'])
                ->where('warehouse_id' , $data['warehouse_id'])
                ->update( $data );
        }else{
            $insert['add_user'] = auth()->user()->user_id;
            $id = DB::table('customer_warehouse_prices')->insertGetId( $data );
        }


        $this->return['id'] = $id;
        return $this->sendResponse( $this->return , trans('main.update_success') );




    }


}
