<?php

namespace App\Http\Controllers\API\V1\Sales;
use App\Http\Controllers\API\V1\BaseController;
use App\Classes\Finance\DelegatePettyHelper;
use App\Models\Finance\AccountTree;
use App\Models\Sales\CovenantType;
use App\Models\Sales\Delegate;
use App\Models\Main\ScreenSub;
use Illuminate\Http\Request;

class DelegateController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('04-001','view');

        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'rows'   => $request['rows'] ?? null ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('04-001', $lang)->sub_title ?? '';
        $this->return['items']       = Delegate::all_delegates($this->filter);

        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('04-001','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-001', $lang)->sub_title ?? '';
        $this->return['account_trees']  = AccountTree::active_account_trees($lang, array('account_type' => 2, 'active' => '1'));
        $this->return['covenant_types'] = CovenantType::all_covenant_types(array('active' => 1, 'lang' => $lang));

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('04-001','add');

        $insert = $request->only(['name_en', 'name_ar', 'account_tree_id', 'covenant_type_id','is_delegate_petty']);
        $insert['add_user'] = auth()->user()->id;

        $validator = validator()->make($insert, Delegate::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        $this->return['id'] = Delegate::add_delegate($insert);
        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function show($lang, $id)
    {
        $this->checkPermission('04-001','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-001', $lang)->sub_title ?? '';
        $this->return['data']           = Delegate::get_delegate($id, $lang);
        $this->return['account_trees']  = AccountTree::active_account_trees($lang, array('account_type' => 2, 'active' => '1'));
        $this->return['covenant_types'] = CovenantType::all_covenant_types(array('active' => 1, 'lang' => $lang));

        return $this->sendResponse($this->return);
    }

    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('04-001','edit');

        $update = $request->only(['name_en', 'name_ar', 'account_tree_id', 'covenant_type_id','is_delegate_petty']);

        $validator = validator()->make($update, Delegate::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        (new DelegatePettyHelper)->update_balance($request['delegate_id']);


        Delegate::update_delegate($update, $id);
        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('04-001','delete');

        Delegate::delete_delegate($id);
        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function toggle_active($lang, $id)
    {
        $item = Delegate::find($id);

        if($item->delegate_active){
            $this->checkPermission('04-001','delete');

            Delegate::delete_delegate($id);
            $this->return['active'] = false;
        }else{
            $this->checkPermission('04-001','edit');


            Delegate::update_delegate(array('delegate_active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }
}
