<?php
namespace App\Http\Controllers\API\V1\Sales;
use App\Classes\General\General;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;
use App\Models\Sales\ClearanceInvoice;
use App\Models\Sales\Clearance;
use App\Models\Main\Branch;
use App\Models\Main\Setting;
use Illuminate\Support\Facades\DB;
use Salla\ZATCA\GenerateQrCode;
use Salla\ZATCA\Tags\InvoiceDate;
use Salla\ZATCA\Tags\InvoiceTaxAmount;
use Salla\ZATCA\Tags\InvoiceTotalAmount;
use Salla\ZATCA\Tags\Seller;
use Salla\ZATCA\Tags\TaxNumber;

use PDF;
use DNS2D;

class ExportInvoiceController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function get_invoice_by_uuid(Request $request)
    {
        $uuid = $request['uuid'];
        $lang  = 'ar';

        if($request['type'] == 'clearance'){
            $this->return['master']             = ClearanceInvoice::get_invoice_by_uuid($uuid, $lang);
            $id = $this->return['master']->id;
            $this->return['details']            = ClearanceInvoice::get_invoice_d($id, $lang);
            $this->return['delegate_details']   = ClearanceInvoice::get_invoice_delegate_d($id, $lang);
            $this->return['branch_data']        = Branch::get_branch($this->return['master']->branch_id  , $lang);
            $this->return['logo']               = $this->return['branch_data']->branch_image ?? 0;
            $this->return['goods_type']         = DB::table('classifications_m')
                ->join('classifications_d' , 'classifications_m.classification_id' , '=' , 'classifications_d.classifications_m_id')
                ->where('clearances_m_id' , $this->return['master']->clearance_id)
                ->first()->item_name ?? $this->return['master']->goods_type;
            $this->return['containers']         = implode(' , ' ,  Clearance::get_clearance_d($this->return['master']->clearance_id)->pluck('container_number')->toArray() );
            $this->return['tafqeet']            =  ( new General )->numberToText($lang, round($this->return['master']->total_amount , 2) , 'SAR' );
            if( strlen($this->return['master']->invoice_qrcode) > 10 ){

                $this->return['qrcode'] = "data:image/png;base64,".DNS2D::getBarcodePNG( ($this->return['master']->invoice_qrcode) , 'QRCODE', 10, 10);

            }else{

                $this->return['qrcode']             = GenerateQrCode::fromArray([
                    new Seller($this->return['branch_data']->vat_register_name),
                    new TaxNumber($this->return['branch_data']->vat_number),
                    new InvoiceDate(date('Y-m-d\TH:i:s', strtotime($this->return['master']->add_date))),
                    new InvoiceTotalAmount(($this->return['delegate_details']->sum('amount') + $this->return['details']->sum('amount_with_vat') )), // Invoice Total Amount
                    new InvoiceTaxAmount(round($this->return['details']->sum('vat_amount') , 2)) // Invoice Tax Amount
                ])->render();
            }
            $this->return['logo']     = public_path().$this->return['logo'];

            $temp_name  = $request['print_temp'] ?? Setting::get_main_setting_by_code('invoice_whats')->value ?? 'temp_whats_1' ;
            $print_temp = 'Print.clearance.invoice.'.$temp_name;



            $pdf = PDF::loadView($print_temp , array('data' => $this->return ))->setOption('margin-top', 0)
                ->setOption('margin-left', 0)
                ->setOption('margin-right', 0)
                ->setPaper('a4')
                ->setOption("footer-center", "[page] / [topage]");
                
                
        }elseif($request['type'] == 'draft'){
            $this->return['master']             = ClearanceInvoice::get_invoice_by_uuid($uuid, $lang);
            $id = $this->return['master']->id;

            $this->return['details']            = ClearanceInvoice::get_invoice_d($id, $lang);
            $this->return['delegate_details']   = ClearanceInvoice::get_invoice_delegate_d($id, $lang);
            $this->return['branch_data']        = Branch::get_branch($this->return['master']->branch_id  , $lang);
            $this->return['logo']               = $this->return['branch_data']->branch_image ?? 0;
            $this->return['containers']         = implode(' , ' , Clearance::get_clearance_d($this->return['master']->clearance_id)->pluck('container_number')->toArray() );

            $this->return['qrcode']             = GenerateQrCode::fromArray([
                new Seller($this->return['branch_data']->vat_register_name),
                new TaxNumber($this->return['branch_data']->vat_number),
                new InvoiceDate(date('Y-m-d\TH:i:s', strtotime($this->return['master']->add_date))),
                new InvoiceTotalAmount(($this->return['delegate_details']->sum('amount') + $this->return['details']->sum('amount_with_vat') )), // Invoice Total Amount
                new InvoiceTaxAmount(round($this->return['details']->sum('vat_amount') , 2)) // Invoice Tax Amount
            ])->render();
            $this->return['logo']     = public_path().$this->return['logo'];
            $pdf = PDF::loadView('Print/clearance/draft_invoice', array('data' => $this->return ))->setOption("footer-center", "[page] / [topage]");
        }elseif($request['type'] == 'claim'){
            $this->return['master']             = ClearanceInvoice::get_invoice_by_uuid($uuid, $lang);
            $id = $this->return['master']->id;
            $this->return['details']            = ClearanceInvoice::get_invoice_d($id, $lang);
            $this->return['delegate_details']   = ClearanceInvoice::get_invoice_delegate_d($id, $lang);
            $this->return['branch_data']        = Branch::get_branch($this->return['master']->branch_id  , $lang);
            $this->return['logo']               = $this->return['branch_data']->branch_image ?? 0;
            $this->return['containers']         = implode(' , ' ,  Clearance::get_clearance_d($this->return['master']->clearance_id)->pluck('container_number')->toArray() );
            $this->return['qrcode']             = '';
            $this->return['tafqeet']            =  ( new General )->numberToText($lang, round($this->return['master']->total_amount , 2) , 'SAR' );
            $this->return['goods_type']         = DB::table('classifications_m')
                ->join('classifications_d' , 'classifications_m.classification_id' , '=' , 'classifications_d.classifications_m_id')
                ->where('clearances_m_id' , $this->return['master']->clearance_id)
                ->first()->item_name ?? $this->return['master']->goods_type;
            $this->return['logo']     = public_path().$this->return['logo'];
            $pdf = PDF::loadView('Print/clearance/claim_invoice', array('data' => $this->return ))->setOption("footer-center", "[page] / [topage]");
        }
        return $pdf->stream();
    }

}