<?php
namespace App\Http\Controllers\API\V1\Sales;

use App\Classes\General\General;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;
use App\Classes\Sales\NoticeCreditorHelper;
use App\Models\Main\Branch;
use App\Models\Main\ScreenSub;
use App\Models\Sales\NoticeCreditor;
use Salla\ZATCA\GenerateQrCode;
use Salla\ZATCA\Tags\InvoiceDate;
use Salla\ZATCA\Tags\InvoiceTaxAmount;
use Salla\ZATCA\Tags\InvoiceTotalAmount;
use Salla\ZATCA\Tags\Seller;
use Salla\ZATCA\Tags\TaxNumber;
use Illuminate\Support\Facades\DB;

class NoticeCreditorController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('03-043','view');

        $this->filter = array(
            'lang'   => $lang ,
            'branch' => $this->data['current_branch'] ,
            'active' => $request['active'] ?? 1 ,
            'rows'   => $request['rows'] ?? 20 ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-043', $lang)->sub_title ?? '';
        $this->return['items']       = NoticeCreditor::all_notice_creditors($this->filter);


        // if(request()->header('clientID') == 'amwaj'){
        //     $all = DB::table('notice_creditors_m')
        //         ->get();
        //     foreach ($all as $key => $row) {
        //         DB::beginTransaction();
        //         // $this->helper->calculate_invoice($row->sales_invoice_m_id);
        //         (new NoticeCreditorHelper)->generate_journal($row->notice_creditor_m_id);
        //         DB::commit();
        //     }
        // }



        return $this->sendResponse($this->return);
    }
    public function show($lang, $id)
    {
        $this->checkPermission('03-043','view');

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-043', $lang)->sub_title ?? '';

        $this->return['master']        = NoticeCreditor::get_notice_creditor_m($id, $lang);
        $this->return['details']       = NoticeCreditor::get_notice_creditor_d($id, $lang);
        $this->return['branch_data']   = Branch::get_branch($this->return['master']->branch_id  , $lang);
        $this->return['qrcode']        = GenerateQrCode::fromArray([
            new Seller($this->return['branch_data']->vat_register_name),
            new TaxNumber($this->return['branch_data']->vat_number),
            new InvoiceDate(date('Y-m-d\TH:i:s', strtotime($this->return['master']->add_date))),
            new InvoiceTotalAmount( $this->return['details']->sum('amount_with_vat') ), // Invoice Total Amount
            new InvoiceTaxAmount(round($this->return['details']->sum('vat_amount') , 2)) // Invoice Tax Amount
        ])->render();
        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('03-043','view');

        $this->return['notice_types'] =  DB::table('notice_types')->selectRaw('name_'.$lang.' as name ,id ')->get();
        return $this->sendResponse($this->return);
    }

    public function Store($lang, Request $request)
    {
        $this->checkPermission('03-043','add');

        $insert    = $request->all();
        $branch_id = $this->data['current_branch'];

        $year = array_search(date("Y", strtotime($insert['invoice_date'])), $this->data['years']);

        if($year == ''){
            return $this->sendError(trans("error.finance_year_not_aval"));
        }
        if((new General)->check_financial_year($insert['invoice_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($insert['invoice_date']));
        }
        if(!isset($insert['items']) || count($insert['items']) < 1){
            return $this->sendError(trans("No Items"));
        }

        $master_seed = [
            'invoice_m_id'          => $insert['invoice_m_id'],
            'branch_id'             => $branch_id,
            'finance_year_id'       => $year,
            'daily_journal_id'      => 0,
            'description'           => $insert['description'] ?? null,
            'invoice_date'          => $insert['invoice_date'] ?? date('Y-m-d'),
            'notice_type_id'        => $insert['notice_type_id'] ?? 2,
            'add_user'              => auth()->user()->id
        ];

        DB::beginTransaction();

            $m_id = NoticeCreditor::add_notice_creditor_m($master_seed);

            if (isset($insert['items']) && is_array($insert['items'])) {
                foreach ($insert['items'] as $item) {
                    if($master_seed['notice_type_id'] == 1){
                        if($item['is_trip'] == 1){
                            $detail = DB::table('trips')
                            ->selectRaw('
                                current_trip_price * 0.15 as current_vat ,
                                current_trip_price as current_without_vat ,
                                current_trip_price * 1.15 as current_with_vat
                           ')->where('trip_id', $item['d_id'])->first();
                        }elseif($item['is_transaction'] == 1){
                            $detail = DB::table('wearhouse_transactions')
                            ->selectRaw('
                                current_trip_cost * 0.15 as current_vat ,
                                current_trip_cost as current_without_vat ,
                                current_trip_cost * 1.15 as current_with_vat
                           ')->where('wearhouse_transaction_id', $item['d_id'])->first();
                        }

                    }elseif($master_seed['notice_type_id'] == 2){
                        $detail = DB::table('clearance_invoices_d')
                        ->selectRaw('
                           current_vat ,
                           current_without_vat ,
                           current_with_vat
                       ')->where('clearance_invoices_d_id', $item['d_id'])->first();
                    }

                    $new_total_vat          = $detail->current_vat         - $item['new_total_vat'] ;
                    $new_total_without_vat  = $detail->current_without_vat - $item['new_total_without_vat'];
                    $new_total_with_vat     = $detail->current_with_vat    - $item['new_total_with_vat'];

                    if($new_total_without_vat < 0 || $new_total_vat < 0 || $new_total_with_vat < 0 || $item['new_total_vat'] <= 0 || $item['new_total_without_vat'] <= 0 || $item['new_total_with_vat'] <= 0){
                        return $this->sendError(trans("Error In Price"));
                    }
                    if($master_seed['notice_type_id'] == 1){
                        if($item['is_trip'] == 1){
                            DB::table('trips')
                            ->where('trip_id', $item['d_id'])
                            ->update([
                                'current_trip_price'   => $item['new_total_without_vat'],
                            ]);
                        }elseif($item['is_transaction'] == 1){
                            DB::table('wearhouse_transactions')
                            ->where('wearhouse_transaction_id', $item['d_id'])
                            ->update([
                                'current_total_cost'  => $item['new_total_without_vat'],
                            ]);
                        }

                    }elseif($master_seed['notice_type_id'] == 2){
                        DB::table('clearance_invoices_d')
                        ->where('clearance_invoices_d_id', $item['d_id'])
                        ->update([

                            'current_vat'                  => $new_total_vat ,
                            'current_without_vat'          => $new_total_without_vat ,
                            'current_with_vat'             => $new_total_with_vat ,
                        ]);
                    }


                    $detail_seed = [
                        'notice_creditor_m_id'      => $m_id,
                        'invoice_d_id'              => $item['d_id'],
                        'vat_amount'                => $item['new_total_vat'] ,
                        'amount_without_vat'        => $item['new_total_without_vat'] ,
                        'amount_with_vat'           => $item['new_total_with_vat'] ,
                    ];

                    NoticeCreditor::add_notice_creditor_d($detail_seed);

                }
            }

            (new NoticeCreditorHelper)->calculate_master_values($m_id);
            (new NoticeCreditorHelper)->generate_journal($m_id);

        DB::commit();
        $this->return['id'] = $m_id;
        return $this->sendResponse($this->return, trans('main.add_success'));
    }
    public function update($lang, $id , Request $request)
    {
        $this->checkPermission('03-043','add');

        $insert    = $request->all();
        if(!isset($insert['items']) || count($insert['items']) < 1){
            return $this->sendError(trans("No Items"));
        }
        $master_seed = [
            'description'           => $insert['description'] ?? null,
        ];
       $master =  NoticeCreditor::get_notice_creditor_m($id);
       if((new General)->check_financial_year($master->invoice_date) != "true"){
            return $this->sendError((new General)->check_financial_year($master->invoice_date));
        }
        DB::beginTransaction();

            NoticeCreditor::update_notice_creditor_m($master_seed , $id);

            if (isset($insert['items']) && is_array($insert['items'])) {
                foreach ($insert['items'] as $item) {
                    if($master->notice_type_id == 1){
                        if($item['is_trip'] == 1){
                            $detail = DB::table('trips')
                            ->selectRaw('
                                current_trip_price * 0.15 as current_vat ,
                                current_trip_price as current_without_vat ,
                                current_trip_price * 1.15 as current_with_vat
                           ')->where('trip_id', $item['d_id'])->first();
                        }elseif($item['is_transaction'] == 1){
                            $detail = DB::table('wearhouse_transactions')
                            ->selectRaw('
                                current_trip_cost * 0.15 as current_vat ,
                                current_trip_cost as current_without_vat ,
                                current_trip_cost * 1.15 as current_with_vat
                           ')->where('wearhouse_transaction_id', $item['d_id'])->first();
                        }

                    }elseif($master->notice_type_id == 2){
                        $old_item = DB::table('notice_creditors_d')->where('notice_creditor_d_id' ,$item['d_id'] )->first();
                        $detail = DB::table('clearance_invoices_d')
                        ->selectRaw('
                           current_vat ,
                           current_without_vat ,
                           current_with_vat
                       ')->where('clearance_invoices_d_id', $old_item->invoice_d_id)->first();
                    }

                    $new_total_vat          = ($detail->current_vat + $old_item->vat_amount ) - $item['new_total_vat'] ;
                    $new_total_without_vat  = ($detail->current_without_vat + $old_item->amount_without_vat) - $item['new_total_without_vat'];
                    $new_total_with_vat     = ($detail->current_with_vat + $old_item->amount_with_vat)   - $item['new_total_with_vat'];

                    if($new_total_without_vat < 0 || $new_total_vat < 0 || $new_total_with_vat < 0 || $item['new_total_vat'] <= 0 || $item['new_total_without_vat'] <= 0 || $item['new_total_with_vat'] <= 0){
                        return $this->sendError(trans("Error In Price"));
                    }
                    if($master->notice_type_id == 1){
                        if($item['is_trip'] == 1){
                            DB::table('trips')
                            ->where('trip_id', $item['d_id'])
                            ->update([
                                'current_trip_price'   => $item['new_total_without_vat'],
                            ]);
                        }elseif($item['is_transaction'] == 1){
                            DB::table('wearhouse_transactions')
                            ->where('wearhouse_transaction_id', $item['d_id'])
                            ->update([
                                'current_total_cost'  => $item['new_total_without_vat'],
                            ]);
                        }

                    }elseif($master->notice_type_id == 2){
                        DB::table('clearance_invoices_d')
                        ->where('clearance_invoices_d_id', $old_item->invoice_d_id)
                        ->update([
                            'current_vat'                  => $new_total_vat ,
                            'current_without_vat'          => $new_total_without_vat ,
                            'current_with_vat'             => $new_total_with_vat ,
                        ]);
                    }


                    $detail_seed = [
                        'vat_amount'                => $item['new_total_vat'] ,
                        'amount_without_vat'        => $item['new_total_without_vat'] ,
                        'amount_with_vat'           => $item['new_total_with_vat'] ,
                    ];

                    NoticeCreditor::update_notice_creditor_d($detail_seed , $item['d_id']);

                }
            }

            (new NoticeCreditorHelper)->calculate_master_values($id);
            (new NoticeCreditorHelper)->generate_journal($id);

        DB::commit();
        return $this->sendResponse($this->return, trans('main.update_success'));
    }
    // public function print_invoice($id, Request $request)
    // {
    //     $input = $request->all();

    //     App::setLocale('ar');

    //     $this->data['lang']        = 'ar';
    //     $this->data['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-043', $this->data['lang']);
    //     $this->data['master']      = NoticeCreditor::get_notice_creditor_m($id, $this->data['lang']);
    //     $this->data['details']     = NoticeCreditor::get_notice_creditor_d($id, $this->data['lang']);
    //     $this->data['branch_data'] = Branch::get_branch($this->data['master']->branch_id);

    //     $temp_name                 = Setting::get_main_setting_by_code('notice_creditor_print')->value ?? 'temp_1' ;
    //     $print_temp                = 'Print.notice_creditor.'.$temp_name;

    //     $this->data['qrcode']      = GenerateQrCode::fromArray([
    //         new Seller($this->data['branch_data']->vat_register_name), // Seller Name
    //         new TaxNumber($this->data['branch_data']->vat_number), // Seller Tax Number
    //         new InvoiceDate(date('Y-m-d\TH:i:s', strtotime($this->data['master']->add_date))),
    //         new InvoiceTotalAmount($this->data['details']->sum('new_total_price')), // Invoice Total Amount
    //         new InvoiceTaxAmount(round($this->data['details']->sum('new_total_vat') ?? 0,2)) // Invoice Tax Amount
    //     ])->render();

    //     switch (@$input['type']) {
    //         case 'excel':
    //             $this->data['logo'] = public_path().$this->data['branch_data']->branch_image;
    //             $export = new InvoicesExport($this->data, $print_temp);
    //             return Excel::download($export, 'notice_debtor.xlsx');
    //             break;
    //         case 'pdf':
    //             $this->data['logo'] = public_path().$this->data['branch_data']->branch_image;
    //             $pdf = PDF::loadView($print_temp, array('data' => $this->data));
    //             return $pdf->stream();
    //             break;
    //         case 'image':
    //             $this->data['logo'] = public_path().$this->data['branch_data']->branch_image;
    //             $image = App::make('snappy.image.wrapper');
    //             $image->loadView($print_temp, array('data' => $this->data));
    //             return $image->stream();
    //             break;
    //         case 'print':
    //             $this->data['logo'] = $this->data['branch_data']->branch_image;
    //             return view($print_temp)->with('data', $this->data);
    //             break;
    //     }
    // }
}
