<?php
namespace App\Http\Controllers\API\V1\Sales;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Sales\Classification;
use App\Models\Sales\Clearance;
use App\Models\Sales\ClearanceInvoice;
use Illuminate\Http\Request;


use App\Classes\Excel\InvoicesExport;
use App\Classes\General\Upload;
use App\Classes\General\General;


use App\Models\Main\ScreenSub;
use App\Models\Sales\Customer;
use App\Models\Sales\Delegate;
use App\Models\Main\Branch;
use App\Models\Finance\ExpenseCode;
use App\Models\Trans\Agent;
use App\Models\Trans\Port;
use App\Models\Sales\Insurance;
use App\Models\Sales\CustomsItem;


use Illuminate\Support\Facades\DB;
use PDF;
use App;
use App\Classes\Sales\ClearanceHelper;
use App\Http\Resources\ClearanceResource;
use Excel;
use DateTime;
use HanifHefaz\Dcter\Dcter;
use Illuminate\Support\Carbon;

class ReportController extends BaseController
{

    protected $return = array();
    protected $filter = array();

    protected function get_print_data( $branch_id ){

        $branch_data    = Branch::get_branch($branch_id);

        $this->return['print']['print_margin']   = $branch_data->print_margin ?? '';
        $this->return['print']['print_right']    = $branch_data->print_label_ar ?? '';
        $this->return['print']['print_left']     = $branch_data->print_label_en ?? '';
        $this->return['print']['footer']         = $branch_data->print_footer ?? '';
        $this->return['print']['logo']           = get_logo( $branch_id );


    }

    public function custom_codes_report_get($lang , Request $request){

        $this->checkPermission('03-061','view');
        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-061' , $lang)->sub_title ?? '';
        $this->return['customs_items']  = CustomsItem::all_customs_items(array('lang' => $lang , 'active' => 1 ));
        $this->return['customers']      = customer::all_customers(array('lang' => $lang , 'active' => 1 ));

        return $this->sendResponse( $this->return );

    }

    public function custom_codes_report_post($lang , Request $request){

        $filter = array(
            'customs_item_id'   => $request['customs_item'] ?? null ,
            'customer_id'       => $request['customer_id'] ?? null ,
            'from_date'         => $request['from_date'] ?? null ,
            'to_date'           => $request['to_date'] ?? null
        );


        $query = DB::table('classifications_d')
            ->leftjoin('customs_items' , 'classifications_d.customs_item_id' , '=' , 'customs_items.customs_item_id')
            ->leftjoin('classifications_m','classifications_d.classifications_m_id','=','classifications_m.classification_id')
            ->leftjoin('clearances_m' , 'classifications_m.clearances_m_id' , '=' , 'clearances_m.clearance_id')
            ->leftjoin('customers' , 'clearances_m.customer_id' , '=' , 'customers.customer_id')
            ->selectRaw('
                classifications_d.qry as qty ,
                customs_items.item_name_'.$lang.' as customs_item ,
                customers.name_'.$lang.' customer_name ,
                classifications_m.clearances_m_id as clearance_number ,
                clearances_m.action_date as clearance_date
            ');

        if($filter['customs_item_id'] > 0 ){
            $query->where('classifications_d.customs_item_id' , $filter['customs_item_id']);
        }

        if($filter['customer_id'] > 0 ){
            $query->where('clearances_m.customer_id' , $filter['customer_id']);
        }

        if($filter['from_date'] != null ){
            $query->where('clearances_m.action_date' , '>=', $filter['from_date']);
        }

        if($filter['to_date'] != null  ){
            $query->where('clearances_m.action_date' ,'<=',  $filter['to_date']);
        }


        $this->return['items'] = $query ->get();

        return $this->sendResponse( $this->return );

    }

    public function container_status_report_get($lang , Request $request){
        $this->checkPermission('03-035','view');
        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-035' , $lang)->sub_title ?? '';

        return $this->sendResponse( $this->return );

    }
    public function container_status_report_post($lang , Request $request){

        $this->filter = array(
            'lang'              => $lang ,
            'active'            => $request['active'] ?? 1 ,
            'word'              => $request['word'] ?? null ,
            'clearance_id'      => $request['clearance_id'] ?? null
        );

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-035', $lang)->sub_title ?? '';
        $this->return['master']         = Clearance::get_clearance($this->filter['clearance_id'] , $lang);
        $this->return['details']        = DB::table($this->data['custom_db'].'.clearances_d')
            ->leftjoin($this->data['trans_db'].'.trips' , $this->data['custom_db'].'.clearances_d.clearances_d_id' , '=' , $this->data['trans_db'].'.trips.clearance_d_id')
            ->where('clearances_d.clearances_m_id' , $this->filter['clearance_id'])
            ->groupBy('clearances_d.clearances_d_id')
            ->selectRaw('clearances_d.clearances_d_id as d_id , clearances_d.recommended_return_date , clearances_d.container_size , clearances_d.container_number , clearances_d.return_date, clearances_d.archive_id , trips.trip_id ')
            ->get();

        return $this->sendResponse( $this->return );

    }




    public function completed_insurance_get($lang , Request $request){
        $this->checkPermission('03-029','view');


        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-029' , $lang)->sub_title ?? '';
        $this->return['agents']         = Agent::all_agents(array('active' => 1, 'lang' => $lang));

        return $this->sendResponse( $this->return );

    }
    public function completed_insurance_post($lang , Request $request){

        $this->filter = array(
            'lang'              => $lang ,
            'active'            => $request['active'] ?? 1 ,
            'word'              => $request['word'] ?? null ,
            'agent_id'          => $request['agent_id'] ?? null ,
            'clearance_id'      => $request['clearance_id'] ?? null ,
            'from_date'         => $request['from_date'] ?? null ,
            'to_date'           => $request['to_date'] ?? null ,
            'status'            => 1 ,
            'complete'          => 1 ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-029', $lang)->sub_title ?? '';
        $agents = collect(Insurance::all_insurances($this->filter) )->groupBy('agent_id')->all();
        $this->return['items'] = [];

        foreach($agents as $agent){
            $this->return['items'][] = [
                'agent_id' => $agent[0]->agent_id ,
                'agent_name' => $agent[0]->agent_name ,
                'insurances' => $agent
            ];
        }

        return $this->sendResponse( $this->return );

    }


    public function notcompleted_insurance_get($lang , Request $request){
        $this->checkPermission('03-029','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-030' , $lang)->sub_title ?? '';
        $this->return['agents']         = Agent::all_agents(array('active' => 1, 'lang' => $lang));

        return $this->sendResponse( $this->return );

    }
    public function notcompleted_insurance_post($lang , Request $request){

        $this->filter = array(
            'lang'              => $lang ,
            'active'            => $request['active'] ?? 1 ,
            'rows'              => $request['rows'] ?? null ,
            'page'              => $request['page'] ?? 1 ,
            'word'              => $request['word'] ?? null ,
            'agent_id'          => $request['agent_id'] ?? null ,
            'clearance_id'      => $request['clearance_id'] ?? null ,
            'from_date'         => $request['from_date'] ?? null ,
            'to_date'           => $request['to_date'] ?? null ,
            'status'            => 1 ,
            'complete'          => 0 ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-030', $lang)->sub_title ?? '';
        $this->return['items']       = Insurance::all_insurances($this->filter);

        return $this->sendResponse( $this->return );

    }


    public function reimburses_expenses_report_get($lang , Request $request){

//        $this->checkPermission('03-029','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-021' , $lang)->sub_title ?? '';
        $this->return['customers']      = customer::all_customers(array('lang' => $lang , 'active' => 1 ));
        $this->return['delegates']      = Delegate::all_delegates(array('lang' => $lang , 'active' => 1  ));
        $this->return['expense_codes']  = ExpenseCode::all_expense_codes( array('lang' => $lang , 'active' => 1 ) );
        $this->return['ports']          = Port::all_ports(array('lang' => $lang , 'active' => 1 ));

        return $this->sendResponse( $this->return );

    }

    public function reimburses_expenses_report_post($lang , Request $request){
        //$this->checkPermission('03-029','view');

        $this->filter['lang']               = $lang ;
        $this->filter['from']               = $request['from'] ?? date("Y-m-01");
        $this->filter['to']                 = $request['to'] ?? date("Y-m-d");
        $this->filter['customer_id']        = $request['customer_id'] ?? 0 ;
        $this->filter['delegate_id']        = $request['delegate_id'] ?? 0 ;
        $this->filter['clearance_id']       = $request['clearance_id'] ?? 0 ;
        $this->filter['expense_code_id']    = $request['expense_code_id'] ?? null ;
        $this->filter['shipping_policy_no'] = $request['shipping_policy_no'] ?? null ;
        $this->filter['container_no']       = $request['container_no'] ?? null ;
        $this->filter['no_invoice']         = $request['no_invoice'] ?? null ;
        $this->filter['source']             = $request['source'] ?? null ;
        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('04-021' , $lang)->sub_title ?? '';
        $this->filter['port_id']            = $request['port_id'] ?? null ;

        $delegate_petty = DB::table('delegate_petty_d')
            ->leftjoin('delegate_petty_m' , 'delegate_petty_d.delegate_petty_m_id' , '=' , 'delegate_petty_m.delegate_petty_m_id')
            ->leftjoin('delegates' , 'delegate_petty_m.delegate_id' , '=' , 'delegates.delegate_id')
            ->leftjoin('expense_codes' , 'delegate_petty_d.expense_code_id' , '=' , 'expense_codes.expense_code_id')
            ->leftjoin('clearances_m' , 'delegate_petty_d.clearance_id' , '=' , 'clearances_m.clearance_id')
            ->leftjoin('customers' , 'clearances_m.customer_id' , '=' , 'customers.customer_id')
            ->leftjoin('ports' , 'clearances_m.port_id' , '=' , 'ports.port_id')
            ->where('delegate_petty_m.action_date' , '>=' , $this->filter['from'] )
            ->where('delegate_petty_m.action_date' , '<=' , $this->filter['to'] )
            ->selectRaw('
                delegate_petty_d.delegate_petty_m_id as m_id ,
                delegate_petty_d.delegate_petty_d_id as d_id ,
                clearances_m.clearance_id ,
                clearances_m.shipping_policy_no ,
                clearances_m.first_container ,
                expense_codes.expense_code_id ,
                expense_codes.name_'.$lang.' as expense_code_name ,
                ports.port_name_'.$lang.' as port_name ,
                customers.name_'.$lang.' as customer_name ,
                delegates.name_'.$lang.' as delegate_name ,
                clearances_m.action_date as clearance_date ,
                delegate_petty_d.receipt_number ,
                delegate_petty_d.amount ,
                delegate_petty_d.description_'.$lang.' as description ,
                delegate_petty_d.add_date ,
                delegate_petty_d.invoice_id,
                "petty" as source
            ');

        if( $this->filter['customer_id'] > 0 ){
            $delegate_petty->where('clearances_m.customer_id' , $this->filter['customer_id'] );
        }

        if( $this->filter['delegate_id'] > 0 ){
            $delegate_petty->where('delegate_petty_m.delegate_id' , $this->filter['delegate_id'] );
        }

        if( isset($this->filter['expense_code_id']) &&  is_array($this->filter['expense_code_id']) ){
            $delegate_petty->whereIn('delegate_petty_d.expense_code_id' , $this->filter['expense_code_id'] );
        }

        if( $this->filter['clearance_id'] > 0 ){
            $delegate_petty->where('delegate_petty_d.clearance_id' , $this->filter['clearance_id'] );
        }

        if( $this->filter['shipping_policy_no'] != null ){
            $delegate_petty->where('clearances_m.shipping_policy_no' , $this->filter['shipping_policy_no'] );
        }

        if ($this->filter['port_id'] != null) {
            $delegate_petty->where('clearances_m.port_id', $this->filter['port_id']);
        }

        if ($this->filter['no_invoice'] != null) {
            $delegate_petty->where('delegate_petty_d.invoice_id', 0 );
        }

        if( $this->filter['container_no'] != null ){
            $ids = DB::table('clearances_d')
                ->where('container_number' , 'like' , '%'.$this->filter['container_no'].'%')
                ->pluck('clearances_m_id');

            $delegate_petty->whereIn('delegate_petty_d.clearance_id' , $ids );
        }


        $report = DB::table('multi_voucher_d')
            ->join('multi_voucher_m' , 'multi_voucher_d.multi_voucher_m_id' , '=' , 'multi_voucher_m.multi_voucher_m_id')
            ->where('multi_voucher_d.invoice_id' , 0 )
            ->where('multi_voucher_m.multi_voucher_active' , 1 )
            ->where('multi_voucher_d.multi_voucher_d_active' , 1 )
            ->where('multi_voucher_d.clearance_id' , '>',  0 )
            // ->where('voucher_type' , 1 )
            ->leftjoin('delegates' , 'multi_voucher_d.dis_delegate_id' , '=' , 'delegates.delegate_id')
            ->leftjoin('expense_codes' , 'multi_voucher_d.dis_expense_code_id' , '=' , 'expense_codes.expense_code_id')
            ->leftjoin('clearances_m' , 'multi_voucher_d.clearance_id' , '=' , 'clearances_m.clearance_id')
            ->leftjoin('customers' , 'clearances_m.customer_id' , '=' , 'customers.customer_id')
            ->leftjoin('ports' , 'clearances_m.port_id' , '=' , 'ports.port_id')
            ->where('multi_voucher_m.action_date' , '>=' , $this->filter['from'] )
            ->where('multi_voucher_m.action_date' , '<=' , $this->filter['to'] )
            ->groupBy('clearances_m.clearance_id')
            ->having('amount' , '>' , 0 )
            ->selectRaw('
                multi_voucher_d.multi_voucher_m_id as m_id ,
                multi_voucher_d.multi_voucher_d_id  as d_id ,
                clearances_m.clearance_id ,
                clearances_m.shipping_policy_no ,
                clearances_m.first_container ,
                expense_codes.expense_code_id ,
                expense_codes.name_'.$lang.' as expense_code_name ,
                ports.port_name_'.$lang.' as port_name ,
                customers.name_'.$lang.' as customer_name ,
                delegates.name_'.$lang.' as delegate_name ,
                clearances_m.action_date as clearance_date ,
                multi_voucher_d.receipt_number ,

                sum( if(voucher_type = 1 , multi_voucher_d.amount , (-1 * multi_voucher_d.amount) ) ) as amount ,

                multi_voucher_d.description as description ,
                multi_voucher_m.add_date ,
                multi_voucher_d.invoice_id,
                "multi" as source
            ');

        if( $this->filter['customer_id'] > 0 ){
            $report->where('clearances_m.customer_id' , $this->filter['customer_id'] );
        }

        if( $this->filter['delegate_id'] > 0 ){
            $report->where('multi_voucher_m.dis_delegate_id' , $this->filter['delegate_id'] );
        }

        if( $this->filter['expense_code_id'] > 0 ){
            $report->whereIn('multi_voucher_d.dis_expense_code_id' , $this->filter['expense_code_id'] );
        }

        if( $this->filter['clearance_id'] > 0 ){
            $report->where('multi_voucher_d.clearance_id' , $this->filter['clearance_id'] );
        }

        if( $this->filter['shipping_policy_no'] != null ){
            $report->where('clearances_m.shipping_policy_no' , $this->filter['shipping_policy_no'] );
        }

        if ($this->filter['port_id'] != null) {
            $report->where('clearances_m.port_id', $this->filter['port_id']);
        }

        if ($this->filter['no_invoice'] != null) {
            $report->where('multi_voucher_d.invoice_id', 0 );
        }

        if( $this->filter['container_no'] != null ){
            $ids = DB::table('clearances_d')
                ->where('container_number' , 'like' , '%'.$this->filter['container_no'].'%')
                ->pluck('clearances_m_id');

            $report->whereIn('multi_voucher_d.clearance_id' , $ids );
        }
        if($this->filter['source'] == 'petty'){

            $this->return['report']         = $delegate_petty->orderBy('add_date', 'asc')->get();

        }elseif($this->filter['source'] == 'multi'){

            $this->return['report']      = $report->orderBy('add_date', 'asc')->get();

        }else{

            $report->union($delegate_petty);
            $this->return['report']     = $report->orderBy('add_date', 'asc')->get();

        }


        $this->return['ids']            = (collect($this->return['report'])->pluck('clearance_id')->all());
        $this->return['ids']            = array_values( array_unique( array_filter( (array) $this->return['ids']) ));

        $this->get_print_data(1);

        return $this->sendResponse( $this->return );
    }


    public function clearance_report_get($lang , Request $request){

        //$this->checkPermission('03-029','view');


        $this->return['screen_name']            = ScreenSub::get_screen_sub_by_screen_code('03-030' , $lang)->sub_title ?? '';
        $this->return['delegates']              = Delegate::all_delegates(array('active' => 1, 'lang' => $lang ));
        $this->return['clearance_status']       = DB::table('clearance_status')
            ->selectRaw('clearance_status_id as id ,name_'.$lang .' as status_name ,name_ar , name_en ')
            ->get();

        return $this->sendResponse( $this->return );

    }

    public function clearance_report_post($lang , Request $request){

        $this->filter = array(
            'lang'                  => $lang ,
            'active'                => 1 ,
            'full_customer_id'      => $request['customer_id'] ?? null,
            'delegate_id'           => $request['delegate_id'] ?? null,
            'delegate_type'         => $request['delegate_type'] ?? 1,
            'count'                 => $request['no_clearance'] ?? 5
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-030', $lang)->sub_title ?? '';

        $query_status = DB::table('clearance_status')->selectRaw('clearance_status_id as id ,name_'.$lang .' as status_name ,name_ar , name_en ');

        if(isset($request['status']) && is_array($request['status'])){
            $query_status->whereIN('clearance_status_id' , $request['status']);
        }

        $this->return['items']  = $query_status->get();
        foreach ($this->return['items'] as $key => $status){

            $this->filter['status'] = $status->id ;
            $this->return['items'][$key]->clearances = Clearance::all_clearances( $this->filter );
            foreach ($this->return['items'][$key]->clearances as $row){

                $classification_m  = DB::table('classifications_m')->where('clearances_m_id',$row->id)->first();
                $first_item_of_classification_detail = null;
                if (!is_null($classification_m)){
                    $first_item_of_classification_detail = optional(DB::table('classifications_d')->where('classifications_m_id',$classification_m->classification_id)->first())->item_name;
                }
                $row->first_item_of_classification_detail = $first_item_of_classification_detail;
            }
            if(count($this->return['items'][$key]->clearances) < 1 ){
                unset($this->return['items'][$key]);
            }

        }

        $this->return['items'] = collect($this->return['items'])->values();
        return $this->sendResponse( $this->return );

    }
    public function returned_clearances_report_get($lang , Request $request){

            $this->checkPermission('03-040','view');

            $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-040' , $lang)->sub_title ?? '';
            $this->return['agents']         = Agent::all_agents(array('active' => 1, 'lang' => $lang));
            $this->return['ports']          = Port::all_ports(array('active' => 1, 'lang' => $lang));

            return $this->sendResponse( $this->return );

        }

    public function returned_clearances_report_post($lang , Request $request){
        $this->checkPermission('03-040','view');

        $this->filter = array(
            'lang'          => $lang ,
            'active'        => 1 ,
            'agent_id'      => $request['agent_id'] ?? null,
            'returned'      => $request['returned'] ?? 0,
            'port_id'       => $request['port_id'] ?? null,
            'customer_id'   => $request['customer_id'] ?? null,
            'from_date'     => $request['from_date'] ?? date('Y-m-01'),
            'to_date'       => $request['to_date'] ?? date('Y-m-d')
        );

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-040', $lang)->sub_title ?? '';
        $this->return['report']         = (new ClearanceHelper)->returned_clearance_report($this->filter);
        return $this->sendResponse( $this->return );

    }
    public function clearances_delegate_report_get($lang , Request $request){

        $this->checkPermission('03-039','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-039' , $lang)->sub_title ?? '';
        $this->return['delegates']      = Delegate::all_delegates(array('active' => 1, 'lang' => $lang ));

        return $this->sendResponse( $this->return );

    }

    public function clearances_delegate_report_post($lang , Request $request){
        $this->checkPermission('03-039','view');

        $this->filter = array(
            'lang'              => $lang ,
            'active'            => 1 ,
            'delegate_id'       => $request['delegate_id'] ?? null,
            'from'              => $request['from_date'] ?? date('Y-m-01'),
            'to'                => $request['to_date'] ?? date('Y-m-d'),
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-039', $lang)->sub_title ?? '';
        $this->return['report'] = Clearance::all_clearances( $this->filter );
        return $this->sendResponse( $this->return );

    }
    public function invoices_report_get($lang , Request $request){
        $this->checkPermission('02-023','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('02-023' , $lang)->sub_title ?? '';
        $this->return['customers']      = Customer::all_customers(array('active' => 1, 'lang' => $lang));
        $this->return['delegates']      = Delegate::all_delegates(array('active' => 1, 'lang' => $lang ));

        return $this->sendResponse( $this->return );

    }
    public function invoices_report_post($lang , Request $request){

        $this->filter = array(
            'lang'                  => $lang ,
            'active'                => $request['active'] ?? 1 ,
            'rows'                  => $request['rows'] ?? null ,
            'page'                  => $request['page'] ?? 1 ,
            'word'                  => $request['word'] ?? null ,
            'from_date'             => $request['from_date'] ?? null ,
            'to_date'               => $request['to_date'] ?? null ,
            'customer_id'           => $request['customer_id'] ?? null ,
            'delegate_id'           => $request['delegate_id'] ?? null ,
            'is_customs_clearance'  => 1 ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('02-023', $lang)->sub_title ?? '';
        $this->return['invoices']    = ClearanceInvoice::all_invoices($this->filter);

        return $this->sendResponse( $this->return );

    }
    public function clearance_transaction_report($lang , Request $request){
        // $this->checkPermission('03-052','view');
        //update all hijri dates  run this script just first time
        // $comments = DB::table('clearances_comments')->get();
        // foreach($comments as $comment){
        //     $dateOnly = Carbon::parse($comment->add_date)->format('Y-m-d');
        //     $hijri = Dcter::GregorianToHijri($dateOnly);
        //     DB::table('clearances_comments')->where('clearances_comment_id' ,$comment->clearances_comment_id)->update([
        //         'hijri_date' => $hijri
        //     ]);
        // }
        //---------------------------------------------------------------
        $this->filter = array(
            'lang'              => $lang ,
            'clearance_id'      => $request['clearance_id'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-052', $lang)->sub_title ?? '';
        $this->return['clearance']   = Clearance::get_clearance($request['clearance_id'] , $lang);
        $this->return['branch_data'] = Branch::get_branch($this->return['clearance']->branch_id  , $lang);

        $this->return['clearance']->delivery_permission_no = 0;
        $this->return['clearance']->Withdraw_permission_date = 0;
        $this->return['clearance']->payment_date = 0;
        $this->return['clearance']->owner_id = 0;

        $this->return['actions'] = DB::table('clearances_comments')
        ->leftjoin('clearance_status','clearance_status.clearance_status_id','clearances_comments.status_id')
        ->leftjoin('users','users.id','clearances_comments.add_user')
        ->where('clearances_m_id', $request['clearance_id'])
        ->groupBy('clearances_comments.status_id')
                                    ->selectRaw('
                                        clearances_m_id ,
                                        clearances_comment_id as action_id,
                                        has_problem ,
                                        note ,
                                         " " as insert_time,
                                         " " as insert_date,
                                        clearances_comments.add_date ,
                                        clearances_comments.hijri_date ,
                                        clearances_comments.add_user ,
                                        users.user_full_name as added_by,
                                        clearance_status.clearance_status_id ,
                                        clearance_status.name_'.$lang.' as status_name
                                    ')->get();
        if(isset($request['print'])){
            return view('Print.reports.clearance_transactions')->with('data' , $this->return);
        }else{
            return $this->sendResponse( $this->return);
        }

    }

    public function finished_clearances_report_get($lang , Request $request){
        $this->checkPermission('03-062','view');
        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-062' , $lang)->sub_title ?? '';

        return $this->sendResponse( $this->return );

    }
    public function finished_clearances_report_post($lang , Request $request){
        $this->checkPermission('03-062','view');

        $this->filter = array(
            'lang'                  => $lang ,
            'active'                => $request['active'] ?? 1 ,
            'rows'                  => $request['rows'] ?? null ,
            'page'                  => $request['page'] ?? null ,
            'status'                => $request['status'] ?? 6 ,
            'customer_id'           => $request['customer_id'] ?? null ,
            'port_id'               => $request['port_id'] ?? null ,
            'from'                  => $request['from_date'] ?? null ,
            'to'                    => $request['to_date'] ?? null ,
            'container_number'      => $request['container_number'] ?? null ,
            'port_withdraw'         => $request['port_withdraw'] ?? null ,
            'withdrawal_note'       => $request['withdrawal_note'] ?? null ,
            'clearance_id'          => $request['clearance_id'] ?? null ,
            'is_classification'     => $request['is_classification'] ?? null,
            'customer_main_id'      => $request['customer_main_id'] ?? null,
            'agent_id'              => $request['agent_id'] ?? null,
            'delegate_id'           => $request['delegate_id'] ?? null,
            'has_invoice'           => $request['has_invoice'] ?? null,
        );
        $items          = Clearance::all_clearances( $this->filter );

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-062', $lang)->sub_title ?? '';
        $this->return['report']         = ClearanceResource::collection($items);


        return $this->sendResponse( $this->return );

    }

}
