<?php
namespace App\Http\Controllers\API\V1\Sales;

use App\Http\Controllers\API\V1\BaseController;
use App\Models\Main\ScreenSub;
use App\Models\Sales\Clearance;
use App\Models\Sales\ClearanceInvoice;
use App\Models\Sales\ClearanceInvoiceClaim;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class UpdateInvoiceController extends BaseController
{
    protected $return = array();
    protected $filter = array();
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index($lang, Request $request)
    {
        $this->checkPermission('03-058','view');
        $this->filter = array(
            'lang'          => $lang ,
            'active'        => $request['active'] ?? 1 ,
            'rows'          => $request['rows'] ?? null ,
            'page'          => $request['page'] ?? 1 ,
            'word'          => $request['word'] ?? null ,
            'need_update'   =>  1 ,
        );

        $this->return['screen_name']         = ScreenSub::get_screen_sub_by_screen_code('03-058', $lang)->sub_title ?? '';
        if($request->type == 'claim'){
            $this->return['items']      = ClearanceInvoiceClaim::all_invoices($this->filter);

        }elseif($request->type== 'clearance'){
            $this->return['items']  = ClearanceInvoice::all_invoices($this->filter);
        }
        return $this->sendResponse($this->return);
    }
    public function show($lang, $id , Request $request)
    {
        $this->checkPermission('03-058','view');

        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('03-058', $lang)->sub_title ?? '';
        if($request->type == 'claim'){

            $this->return['master']             = ClearanceInvoiceClaim::get_invoice($id, $lang);
            $this->return['details']            = ClearanceInvoiceClaim::get_invoice_d($id, $lang);
            $this->return['new_amounts']        =  $this->get_invoice_amounts($this->return['master']->clearance_id , $lang);
            $this->return['delegate_details']   = ClearanceInvoiceClaim::get_invoice_delegate_d($id, $lang);
            foreach ($this->return['delegate_details'] as $detail){
                $new = collect($this->return['new_amounts'])->where('expense_code_id' , $detail->expense_code_id)->first();
                if(isset($new)){
                    $detail->new_amount     = $new->amount;
                    $detail->from_type      = $new->from_type;
                    $detail->petty_ids      = $new->petty_ids;
                    $detail->multi_ids      = $new->multi_ids;
                }
            }
        }elseif($request->type == 'clearance'){
            $this->return['master']             = ClearanceInvoice::get_invoice($id, $lang);
            $this->return['details']            = ClearanceInvoice::get_invoice_d($id, $lang);
            $this->return['new_amounts']        =  $this->get_invoice_amounts($this->return['master']->clearance_id , $lang);
            $this->return['delegate_details']   = ClearanceInvoice::get_invoice_delegate_d($id, $lang);
            foreach ($this->return['delegate_details'] as $detail){
                $new = collect($this->return['new_amounts'])->where('expense_code_id' , $detail->expense_code_id)->first();
                if(isset($new)){
                    $detail->new_amount     = $new->amount;
                    $detail->from_type      = $new->from_type;
                    $detail->petty_ids      = $new->petty_ids;
                    $detail->multi_ids      = $new->multi_ids;
                }
            }
        }
        return $this->sendResponse($this->return);
    }
    public function update($lang, $id , Request $request)
    {
        $this->checkPermission('03-058','edit');
        $inputs = $request->all();
        DB::beginTransaction();
        if($inputs['type'] == "claim"){
            foreach($inputs['details'] as $detail){
                $detail_seed = [
                    "from_type"                    => $detail['from_type'],
                    "petty_ids"                    => $detail['petty_ids'],
                    "multi_ids"                    => $detail['multi_ids'],
                    "amount"                       => $detail['new_amount'],
                    "description_ar"               => $detail['description'],
                    "description_en"               => $detail['description'],

                ];
                DB::table('clearance_invoices_claim_delegate_d')->where('clearance_invoices_delegate_d_id' , $detail['d_id'])->update($detail_seed);
                DB::table('delegate_petty_d')
                ->whereIn('delegate_petty_d_id' , explode(',', $detail['petty_ids']) )
                ->update( ['invoice_id' => $id , 'invoice_type' => "claim" ] );

                DB::table('multi_voucher_d')
                    ->whereIn('multi_voucher_d_id' , explode(',', $detail['multi_ids']) )
                    ->update( ['invoice_id' => $id , 'invoice_type' => "claim" ] );
            }

            DB::table('clearance_invoices_claim_m')->where('invoice_id' , $id)->update(['need_update' => 0]);



        }elseif($inputs['type'] == "clearance"){
            foreach($inputs['details'] as $detail){
                $detail_seed = [
                    "from_type"                    => $detail['from_type'],
                    "petty_ids"                    => $detail['petty_ids'],
                    "multi_ids"                    => $detail['multi_ids'],
                    "amount"                       => $detail['new_amount'],
                    "description_ar"               => $detail['description'],
                    "description_en"               => $detail['description'],
                ];
                DB::table('clearance_invoices_delegate_d')->where('clearance_invoices_delegate_d_id' , $detail['d_id'])->update($detail_seed);
                DB::table('delegate_petty_d')
                ->whereIn('delegate_petty_d_id' , explode(',', $detail['petty_ids']) )
                ->update( ['invoice_id' => $id , 'invoice_type' => "invoice" ] );

                DB::table('multi_voucher_d')
                    ->whereIn('multi_voucher_d_id' , explode(',', $detail['multi_ids']) )
                    ->update( ['invoice_id' => $id , 'invoice_type' => "invoice" ] );
            }
            DB::table('clearance_invoices_m')->where('invoice_id' , $id)->update(['need_update' => 0]);

        }

        DB::commit();
        return $this->sendResponse($this->return);
    }
    public function get_invoice_amounts($clearance_id ,$lang){

        $clearance_m    = Clearance::get_clearance($clearance_id , $lang);
        if(!$clearance_m){
            $this->return = 'Clearance Not found';
            return false;
        }

        $petty = DB::table('delegate_petty_d')
            ->join('delegate_petty_m' , 'delegate_petty_d.delegate_petty_m_id' , '=' , 'delegate_petty_m.delegate_petty_m_id')
            // ->where('invoice_id' , 0 )
            ->where('delegate_petty_m.delegate_petty_active' , 1 )
            ->where('delegate_petty_d.clearance_id' , '>',  0 )
            ->where('delegate_petty_d.clearance_id' , $clearance_id )
            ->selectRaw('
                "delegate_petty" as from_type ,
                delegate_petty_d.delegate_petty_m_id as m_id ,
                delegate_petty_d.delegate_petty_d_id as d_id ,
                delegate_petty_d.clearance_id ,
                delegate_petty_d.expense_code_id ,
                delegate_petty_d.receipt_number ,
                delegate_petty_d.amount_without_vat as amount ,
                delegate_petty_d.description_'.$lang.' as description ,
                delegate_petty_d.add_date
            ');

        $all = DB::table('multi_voucher_d')
            ->join('multi_voucher_m' , 'multi_voucher_d.multi_voucher_m_id' , '=' , 'multi_voucher_m.multi_voucher_m_id')
            // ->where('multi_voucher_d.invoice_id' , 0 )
            ->where('multi_voucher_m.multi_voucher_active' , 1 )
            ->where('multi_voucher_d.multi_voucher_d_active' , 1 )
            ->where('multi_voucher_d.clearance_id' , '>',  0 )
            ->where('multi_voucher_d.clearance_id' , $clearance_id )
            ->where('voucher_type' , 1 )
            ->selectRaw('
                "multi_voucher" as from_type ,
                multi_voucher_d.multi_voucher_m_id as m_id ,
                multi_voucher_d.multi_voucher_d_id as d_id ,
                multi_voucher_d.clearance_id ,
                multi_voucher_d.dis_expense_code_id as expense_code_id ,
                multi_voucher_d.receipt_number ,
                multi_voucher_d.amount_without_vat as amount ,
                multi_voucher_d.description as description ,
                "" as add_date
            ')
            ->union($petty);


        $deleget_amounts    = DB::table('expense_codes')
            ->leftjoinSub($all, 'all', function ($join) {
                $join->on('all.expense_code_id' , '=' , 'expense_codes.expense_code_id');
            })
            ->where('expense_codes.expense_code_id' , '<>' , 8)
            ->groupBy('expense_codes.expense_code_id')
            ->selectRaw('
                m_id ,
                d_id ,
                group_concat( case when from_type = "delegate_petty" then d_id end) as petty_ids ,
                group_concat( case when from_type = "multi_voucher" then d_id end) as multi_ids ,
                clearance_id ,
                expense_codes.expense_code_id ,
                expense_codes.name_'.$lang.' as expense_code_name ,
                group_concat(ifnull( receipt_number , "-" )) as receipt_number ,
                ifnull( sum(amount) , 0 ) as amount ,
                description ,
                all.from_type ,
                all.add_date
            ')->get();
            //check if invoice paid by customer
            if($clearance_m->paid_by == 1 && in_array($clearance_m->clearance_status_id , [5,6]) ){
                foreach($deleget_amounts as $deleget_amount){
                    if($deleget_amount->expense_code_id == 1){
                        $deleget_amount->description = $deleget_amount->description . "سداد عميل";
                    }
                }
            }

        return $deleget_amounts;

    }

}
