<?php
namespace App\Http\Controllers\API\V1\Trans;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;


use App\Models\Finance\AccountTree;
use App\Models\Main\Delegate;
use App\Models\Trans\ContainerDamage;
use App\Models\Main\ScreenSub;


class ContainerDamageController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-010','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-010' , $lang)->sub_title ?? '';
        $this->return['items']       = ContainerDamage::all_containers( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-010','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-010' , $lang)->sub_title ?? '';
        $this->return['data']           = ContainerDamage::get_container($id);
        $this->return['status']         = ContainerDamage::get_status($lang);

        return $this->sendResponse( $this->return , '' );
    }

    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('03-010','edit');

        $update = $request->only(['status' , 'notes' , 'customer_pay' , 'damage_cost' ]);
        $re     = ContainerDamage::get_container($id);

        if($re->container_status_id >= $update['status'] ){

            return $this->sendError( 'cant update this status' , [] , 200 );

        }

        if($update['status'] == 5){

            $now                    = \Carbon\Carbon::now();
            $lengthOfAd             = $now->diffInDays($re->in_date);

            $update['days']         = $lengthOfAd + 1 ;
            $update['customer_pay'] = $update['customer_pay'] ?? 0;
            $update['damage_cost']  = $update['damage_cost'] ?? 0;
            $update['out_notes']    = $update['notes'] ?? '';
            $update['out_date']     = date("Y-m-d H:i:s");
            $update['out_user']     = auth()->user()->id;
            unset($update['status'] , $update['notes'] );

        }

        unset($update['notes'] , $update['status']);


        ContainerDamage::update_container($update ,  $id );

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

        public function destroy($lang , $id)
    {
        $this->checkPermission('03-010','delete');

        ContainerDamage::delete_container($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }


    public function toggle_active($lang , $id){
        $item = ContainerDamage::get_container($id);
        if($item->container_active){

            $this->checkPermission('03-010','delete');

            ContainerDamage::delete_container($id);
            $this->return['active'] = false;

        }else{

            $this->checkPermission('03-010','edit');

            ContainerDamage::update_container(array('container_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }





}
