<?php

namespace App\Http\Controllers\API\V1\Trans;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;


use App\Models\Finance\AccountTree;
use App\Models\Main\Delegate;
use App\Models\Sales\Customer;
use App\Models\Main\ScreenSub;


class CustomerController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-001','view');


        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-001' , $lang)->sub_title ?? '';
        $this->return['items']       = Customer::all_customers( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-001','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-001' , $lang)->sub_title ?? '';
        $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['delegates']      = Delegate::all_delegates(array('lang' => $lang  , 'active' => 1 ));
        $this->return['default_account']= intval( Setting::get_main_setting_by_code('default_customer_account')->value ?? 0 );

        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('03-001','add');

        $insert = $request->only(['name_en', 'name_ar', 'mobile', 'address', 'vat',
                                  'vat_register_number', 'location', 'delegate_id',
                                  'collection_delegate_id', 'account_tree_id',
                                  'customer_type', 'id_number', 'email' , 'issuing_invoice']);
        $validator = validator()->make( $insert , Customer::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        $insert['add_user'] = auth()->user()->id;
        $this->return['id'] = Customer::add_customer($insert);

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-001','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-001' , $lang)->sub_title ?? '';
        $this->return['data']           = Customer::find($id);
        $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));

        return $this->sendResponse( $this->return , '' );
    }

    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('03-001','edit');

        $update = $request->only(['name_en', 'name_ar', 'mobile', 'address', 'vat',
                                   'vat_register_number', 'location', 'delegate_id',
                                   'collection_delegate_id', 'account_tree_id',
                                   'customer_type', 'id_number', 'email' , 'issuing_invoice']);

        $rules            = Customer::$rules;
        $rules['name_ar'] = $rules['name_ar'].',name_ar,'.$id.',customer_id';
        $rules['name_en'] = $rules['name_en'].',name_en,'.$id.',customer_id';

        $validator  = validator()->make( $update , $rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        unset($update['customer_id'] , $update['customer_active'] , $update['_method'] );
        Customer::update_customer($update ,  $id );

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('03-001','delete');

        Customer::delete_customer($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function sort_post($lang , Request $request){
        $this->checkPermission('03-001','edit');

        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            Customer::update_customer(array('customer_order' => $key ),  $value['id']);
        }
        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function toggle_active($lang , $id){
        $item = Customer::find($id);
        if($item->customer_active){

            $this->checkPermission('03-001','delete');

            Customer::delete_customer($id);
            $this->return['active'] = false;

        }else{
            $this->checkPermission('03-001','edit');


            Customer::update_customer(array('customer_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

}
