<?php

namespace App\Http\Controllers\API\V1\Trans;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;
use App\Classes\General\Upload;


use App\Models\Finance\AccountTree;
use App\Models\Main\Delegate;
use App\Models\Trans\Driver;
use App\Models\Main\ScreenSub;
use App\Models\Main\Branch;
use App\Models\Trans\Port;

class DriverController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-002','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-002' , $lang)->sub_title ?? '';
        $this->return['items']       = Driver::all_drivers( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-002','add');


        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-002' , $lang)->sub_title ?? '';
        $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['ports']          = Port::all_ports(['lang'=>$lang , 'active'=>1]);

        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('03-002','add');
        $insert = $request->only(['driver_name_en', 'driver_name_ar', 'mobile','bank_number','iban', 'licence_expiry_date' , 'residency_number' , 'permit_expiry_date' , 'card_expiry_image' , 'card_expiry_date' , 'port_id']);
        $validator = validator()->make( $insert , Driver::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }


        if(isset($request['card_expiry_image']) && strlen($request['card_expiry_image']) > 10 ){

            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['card_expiry_image'] , 'driver_files' );

            if(!$check['check']){
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            }
        }



        $insert['add_user'] = auth()->user()->id;
        $this->return['id'] = Driver::add_driver($insert);

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-002','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-002' , $lang)->sub_title ?? '';
        $this->return['data']           = Driver::find($id);
        $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['ports']          = Port::all_ports(['lang'=>$lang , 'active'=>1]);

        return $this->sendResponse( $this->return , '' );
    }

    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('03-002','edit');

        $update = $request->only(['driver_name_en', 'driver_name_ar', 'mobile','bank_number','iban', 'licence_expiry_date' , 'residency_number' , 'permit_expiry_date' , 'card_expiry_image' , 'card_expiry_date', 'port_id']);
        $validator  = validator()->make( $update , Driver::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        unset($update['driver_id'] , $update['driver_active'] , $update['_method'] );

        if(isset($request['card_expiry_image']) && strlen($request['card_expiry_image']) > 10 ){

            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['card_expiry_image'] , 'driver_files' );

            if(!$check['check']){
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            }
        }


        Driver::update_driver($update ,  $id );

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('03-002','delete');

        Driver::delete_driver($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function sort_post($lang , Request $request){
        $this->checkPermission('03-002','edit');

        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            Driver::update_driver(array('driver_order' => $key ),  $value['id']);
        }
        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function toggle_active($lang , $id){
        $item = Driver::find($id);
        if($item->driver_active){

            $this->checkPermission('03-002','delete');

            Driver::delete_driver($id);
            $this->return['active'] = false;

        }else{

            $this->checkPermission('03-002','edit');

            Driver::update_driver(array('driver_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

}
