<?php
namespace App\Http\Controllers\API\V1\Trans;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Main\Branch;
use Illuminate\Http\Request;


use App\Models\Finance\AccountTree;
use App\Models\Trans\Area;
use App\Models\Trans\Port;
use App\Models\Main\ScreenSub;


class PortController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-004','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-004' , $lang)->sub_title ?? '';
        $this->return['items']       = Port::all_ports( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-004','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-004' , $lang)->sub_title ?? '';
        $this->return['areas']          = Area::all_areas( ['active' => 1 , 'lang' => $lang] );
        $this->return['branches']          = Branch::all_branchs( ['active' => 1 , 'lang' => $lang] );


        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('03-004','add');
        $insert = $request->only(['port_name_en', 'port_name_ar' , 'area_id','branch_id']);
        $validator = validator()->make( $insert , Port::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        $insert['add_user'] = auth()->user()->id;
        $this->return['id'] = Port::add_port($insert);

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-004','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-004' , $lang)->sub_title ?? '';
        $this->return['data']           = Port::find($id);
        $this->return['areas']          = Area::all_areas( ['active' => 1 , 'lang' => $lang] );
        $this->return['branches']          = Branch::all_branchs( ['active' => 1 , 'lang' => $lang] );


        return $this->sendResponse( $this->return , '' );
    }

    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('03-004','edit');

        $update = $request->only(['port_name_en', 'port_name_ar' , 'area_id','branch_id']);
        $validator  = validator()->make( $update , Port::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        unset($update['port_id'] , $update['port_active'] , $update['_method'] );
        Port::update_port($update ,  $id );

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('03-004','delete');

        Port::delete_port($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function sort_post($lang , Request $request){
        $this->checkPermission('03-004','edit');

        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            Port::update_port(array('port_order' => $key ),  $value['id']);
        }
        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function toggle_active($lang , $id){
        $item = Port::find($id);
        if($item->port_active){

            $this->checkPermission('03-004','delete');

            Port::delete_port($id);
            $this->return['active'] = false;

        }else{

            $this->checkPermission('03-004','edit');

            Port::update_port(array('port_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

}
