<?php
namespace App\Http\Controllers\API\V1\Trans;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;

use App\Classes\General\General;

use App\Models\Trans\Invoice;
use App\Models\Trans\Trip;
use App\Models\Trans\WearhouseTransaction;
use App\Models\Main\ScreenSub;
use App\Models\Main\Branch;

use Salla\ZATCA\GenerateQrCode;
use Salla\ZATCA\Tags\InvoiceDate;
use Salla\ZATCA\Tags\InvoiceTaxAmount;
use Salla\ZATCA\Tags\InvoiceTotalAmount;
use Salla\ZATCA\Tags\Seller;
use Salla\ZATCA\Tags\TaxNumber;
use DB;
use PDF;
use Excel;
use App;
use App\Classes\Trans\InvoiceTransHelper;
use App\Models\Main\Setting;

class TransInvoiceController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-047','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'customer'    => $request['customer'] ?? null ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'type'        => 2,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-047' , $lang)->sub_title ?? '';
        $this->return['items']       = Invoice::all_invoices( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-047','add');
        $this->filter = [
            'from'          => $request['from_date'] ?? date('Y-m-01')  ,
            'to'            => $request['to_date'] ?? date('Y-m-d')  ,
        ];
        $this->return['discount_percentage'] = intval(Setting::get_main_setting_by_code('trip_discount_percentage')->value ?? 0 );

        $this->return['discount_percentage'] = 1 - $this->return['discount_percentage'] / 100  ;
        $this->return['totals'] = DB::connection('other_mysql')->table('clearance_invoices_d')
            ->join('clearance_invoices_m', 'clearance_invoices_m.invoice_id' , 'clearance_invoices_d.invoice_m_id')
            ->whereBetween('invoice_date' , [$this->filter['from'], $this->filter['to']])
            ->where('related_trans_invoice_id' , 0)
            ->where('clearance_invoices_d.revenue_id' , 5)
            ->selectRaw('
                SUM(vat_amount) as total_vat ,
                SUM(amount_with_vat) as total_with_vat,
                SUM(amount_without_vat) as total_without_vat
            ')->first();


        $this->return['totals']->total_with_vat     = $this->return['totals']->total_with_vat * $this->return['discount_percentage'] ;
        $this->return['totals']->total_without_vat  = $this->return['totals']->total_with_vat / 1.15;
        $this->return['totals']->total_vat          = $this->return['totals']->total_with_vat - $this->return['totals']->total_without_vat;

        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('03-047','add');
        $insert                 = $request->only(['invoice_date', 'description', 'from_date', 'to_date']);

        $year   = array_search( date("Y" , strtotime($request['invoice_date'])) , $this->data['years'] );
        if($year == ''){
            return $this->sendError( trans("error.finance_year_not_aval"));
        }
        if((new General)->check_financial_year($insert['invoice_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($insert['invoice_date']));
        }
        $discount_percentage = intval(Setting::get_main_setting_by_code('trip_discount_percentage')->value ?? 0 );

        $discount_percentage = 1 - ($discount_percentage  / 100);
        $total = DB::connection('other_mysql')->table('clearance_invoices_d')
            ->join('clearance_invoices_m', 'clearance_invoices_m.invoice_id' , 'clearance_invoices_d.invoice_m_id')
            ->whereBetween('invoice_date' , [$insert['from_date'], $insert['to_date']])
            ->where('related_trans_invoice_id' , 0)
            ->where('clearance_invoices_d.revenue_id' , 5)
            ->selectRaw('
                SUM(vat_amount) as total_vat ,
                SUM(amount_with_vat) as total_with_vat,
                SUM(amount_without_vat) as total_without_vat
            ')->first();

        $total->total_with_vat      = $total->total_with_vat * $discount_percentage;
        $total->total_without_vat   = $total->total_with_vat / 1.15;
        $total->total_vat           = $total->total_with_vat - $total->total_without_vat;

        if($total->total_with_vat  < 1 ){
            return $this->sendError(trans('error.invalid_invoice_amount'));
        }
        $seed = [
            'type'                  => 2,
            'customer_id'           => 1 ,
            'from_date'             => $insert['from_date'] ,
            'to_date'               => $insert['to_date'] ,
            'description'           => $insert['description'] ,
            'invoice_date'          => $insert['invoice_date'] ,
            'branch_id'             => $this->data['current_branch'] ,
            'finance_year_id'       => $year ,
            'trips'                 => '' ,
            'wearhouse_transactions'=> '' ,
            'trips_total'           => 0 ,
            'transactions_total'    => 0 ,
            'total'                 => $total->total_without_vat  ,
            'discount'              => 0 ,
            'total_without_vat'     => $total->total_without_vat  ,
            'vat_per'               => 15.00 ,
            'total_vat'             => $total->total_vat  ,
            'total_with_vat'        => $total->total_with_vat  ,
            'add_user'              => auth()->user()->id
        ];


        DB::beginTransaction();

        DB::table('invoices')->lockForUpdate()->get();
            $seed['invoice_id'] = intval( DB::table('invoices')->max("invoice_id") ) + 1;
            $this->return['id'] = Invoice::add_invoice($seed);

            DB::connection('other_mysql')->table('clearance_invoices_d')
                ->join('clearance_invoices_m', 'clearance_invoices_m.invoice_id' , 'clearance_invoices_d.invoice_m_id')
                ->whereBetween('invoice_date' , [$insert['from_date'], $insert['to_date']])
                ->where('related_trans_invoice_id' , 0)
                ->where('clearance_invoices_d.revenue_id' , 5)
                ->update(['related_trans_invoice_id' => $this->return['id']]);

            $journal = (new InvoiceTransHelper)->generate_journal($this->return['id']);
            if(!$journal[0])
                return $this->sendError($journal[1], $journal ?? [] );

        DB::commit();

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-047','view');

        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('03-047' , $lang)->sub_title ?? '';
        $this->return['data']               = Invoice::get_invoice($id , $lang);
        $lang                               = 'ar';
        $this->return['lang']               = $lang;
        $this->return['branch_data']        = Branch::get_branch($this->return['data']->branch_id ?? 0  , $lang);
        $this->return['logo']               = $this->return['branch_data']->branch_image ?? 0;
        $this->return['tafqeet']            =  ( new General )->numberToText($this->return['lang'], round($this->return['data']->total_with_vat , 2) , 'SAR' );


        $this->return['qrcode']             = GenerateQrCode::fromArray([
            new Seller($this->return['branch_data']->vat_register_name),
            new TaxNumber($this->return['branch_data']->vat_number),
            new InvoiceDate(date('Y-m-d\TH:i:s', strtotime($this->return['data']->add_date))),
            new InvoiceTotalAmount(round($this->return['data']->total_with_vat , 2)), // Invoice Total Amount
            new InvoiceTaxAmount(round($this->return['data']->total_vat , 2)) // Invoice Tax Amount
        ])->render();

        return $this->sendResponse( $this->return , '' );
    }


    public function print( $id , Request $request)
    {
        $this->checkPermission('03-047','print');
        $lang                               = 'ar';
        $this->return['lang']               = $lang;
        $this->return['master']             = Invoice::get_invoice($id , $lang);
        $this->return['trips']              = Trip::all_trips( ['active' => 1 , 'lang' => $lang ,'invoice_id' => $id ,'customer' => $request['customer'] ] );
        $this->return['transactions']       = WearhouseTransaction::all_wearhouse_transactions( ['active' => 1 , 'lang' => $lang , 'invoice_id' => $id  , 'customer' => $request['customer'] ]);
        $this->return['branch_data']        = Branch::get_branch($this->return['master']->branch_id ?? 0  , $lang);
        $this->return['logo']               = $this->return['branch_data']->branch_image ?? 0;
        $this->return['tafqeet']            =  ( new General )->numberToText($this->return['lang'], round($this->return['master']->total_with_vat , 2) , 'SAR' );


        $this->return['qrcode']             = GenerateQrCode::fromArray([
            new Seller($this->return['branch_data']->vat_register_name),
            new TaxNumber($this->return['branch_data']->vat_number),
            new InvoiceDate(date('Y-m-d\TH:i:s', strtotime($this->return['master']->add_date))),
            new InvoiceTotalAmount(round($this->return['master']->total_with_vat , 2)), // Invoice Total Amount
            new InvoiceTaxAmount(round($this->return['master']->total_vat , 2)) // Invoice Tax Amount
        ])->render();
        // $this->return['qrcode']             =

        switch (@$request['type']) {
            case 'excel':
                $this->return['logo']     = public_path().$this->return['logo'];
                $export = new InvoicesExport($this->return , 'Print/clearance/invoice');
                return Excel::download($export, 'balances.xlsx');
                break;
            case 'pdf':
                $this->return['logo']     = public_path().$this->return['logo'];
                $pdf = PDF::loadView('Print/trans/invoice', array('data' => $this->return ))->setOption("footer-center", "[page] / [topage]");
                return $pdf->stream();
                break;
            case 'image':
                $image = App::make('snappy.image.wrapper');
                $image->loadView('Print/trans/invoice', array('data' => $this->data));
                return $image->stream();
                break;
            case 'print':
                return view('Print.trans.invoice')->with('data' , $this->return);
                break;
            case 'json':
                return view('Print.trans.invoice')->with('data' , $this->return);
                break;
        }


        return $this->sendResponse($this->return);
    }



}
