<?php

namespace App\Http\Controllers\API\V1\Trans;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;


use App\Models\Finance\AccountTree;
use App\Models\Trans\Wearhouse;
use App\Models\Trans\Area;
use App\Models\Main\ScreenSub;


class WearhouseController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-005','view');


        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-005' , $lang)->sub_title ?? '';
        $this->return['items']       = Wearhouse::all_wearhouses( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-005','add');


        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-005' , $lang)->sub_title ?? '';
        $this->return['areas']          = Area::all_areas( ['active' => 1 , 'lang' => $lang] );

        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('03-005','add');
        $insert = $request->only(['wearhouse_name_en', 'wearhouse_name_ar' , 'area_id' , 'price' ,'empty_20' , 'filled_20' , 'empty_40' , 'filled_40' , 'package_price' , 'loading_price' , 'switching_price' , 'allowed_days']);
        $validator = validator()->make( $insert , Wearhouse::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        $insert['add_user'] = auth()->user()->id;
        $this->return['id'] = Wearhouse::add_wearhouse($insert);

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-005','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-005' , $lang)->sub_title ?? '';
        $this->return['data']           = Wearhouse::find($id);
        $this->return['areas']          = Area::all_areas( ['active' => 1 , 'lang' => $lang] );

        return $this->sendResponse( $this->return , '' );
    }

    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('03-005','edit');

        $update = $request->only( ['wearhouse_name_en', 'wearhouse_name_ar' , 'area_id' , 'price' ,'empty_20' , 'filled_20' , 'empty_40' , 'filled_40' , 'package_price' , 'loading_price' , 'switching_price' , 'allowed_days'] );
        $validator  = validator()->make( $update , Wearhouse::$rules );
        if($validator->fails()){
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        unset($update['wearhouse_id'] , $update['wearhouse_active'] , $update['_method'] );
        Wearhouse::update_wearhouse($update ,  $id );

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('03-005','delete');

        Wearhouse::delete_wearhouse($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function sort_post($lang , Request $request){
        $this->checkPermission('03-005','edit');

        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            Wearhouse::update_wearhouse(array('wearhouse_order' => $key ),  $value['id']);
        }
        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function toggle_active($lang , $id){
        $item = Wearhouse::find($id);
        if($item->wearhouse_active){

            $this->checkPermission('03-005','delete');

            Wearhouse::delete_wearhouse($id);
            $this->return['active'] = false;

        }else{

            $this->checkPermission('03-005','edit');

            Wearhouse::update_wearhouse(array('wearhouse_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

}
