<?php

namespace App\Models\Finance;

use Illuminate\Database\Eloquent\Model;
use DB;

class CostCenter extends Model
{

    protected $primaryKey = 'cost_center_id';
    public static $main_table = 'cost_center';
    public $table = 'cost_center';

    public static $rules = array(
        'name_en' => 'required|string|min:3',
        'name_ar' => 'required|string|min:3',
        'cost_center_code' => 'required|string|unique:cost_center',

    );

    public static function active_cost_centers($lang = null , $params = null)
    {
        $query =  DB::table(self::$main_table);

        if($lang != null){
            $query->selectRaw('
            cost_center_id as id  ,
            parent_id ,
            cost_center_code ,
            name_'.$lang.' as name ,
            name_ar ,
            name_en ,
            account_type ,
            if(account_type = "1" , "folder" , "item") as type ,
            if(account_type = "1" , true , false ) as hasChild ,
            protection ,
            active ,
            main_account ,
            level_chars
            ');
        }

        if(isset($params['account_type']))
             $query->where('account_type' , $params['account_type']);

        if(isset($params['parent']))
            $query->where('parent_id' , $params['parent']);


        if(isset($params['active']))
            $query->where('active' , $params['active']);

        if(isset($params['ids']) && is_array($params['ids']) )
            $query->whereIn('cost_center_id' , $params['ids']);

        if(isset($params['start_with']))
            $query->where('cost_center_code' , 'like' , $params['start_with']."%");

        if(isset($params['word']))
            $query->where(self::$main_table.'.name_ar' , 'like' , $params['word']."%")
            ->orWhere(self::$main_table.'.name_en' , 'like' , $params['word']."%")
            ->orWhere(self::$main_table.'.name_en' , 'like' , $params['word']."%");



        $data = $query->get();
    return $data;
    }

    public static function get_cost_center($id , $lang = null)
    {
        $query =  DB::table(self::$main_table)
            ->where('cost_center_id' , $id);

        if($lang != null){
            $query->selectRaw('
                cost_center_id as id  ,
                parent_id ,
                cost_center_code ,
                name_'.$lang.' as name ,
                name_ar ,
                name_en ,
                account_type ,
                if(account_type = "1" , "folder" , "item") as type ,
                if(account_type = "1" , true , false ) as hasChild ,
                protection ,
                active ,
                main_account ,
                level_chars
            ');
        }

        $data = $query->first();
        return $data;

    }

    public static function get_cost_center_balance($id , $lang , $params = null)
    {
        $query = DB::table('daily_journal_d')
            ->join('cost_center', function($join){
                $join->on(DB::raw("FIND_IN_SET(cost_center.cost_center_id, daily_journal_d.cost_centers)"), '>', DB::raw('0'));})
            ->leftjoin('daily_journal_m' , 'daily_journal_d.daily_journal_m_id' , '=' , 'daily_journal_m.daily_journal_m_id')
            ->leftjoin('branchs' , 'daily_journal_m.branch_id' , '=' , 'branchs.branch_id')
            ->leftjoin('finance_years' , 'daily_journal_m.finance_year_id' , '=' , 'finance_years.finance_year_id')
            ->leftjoin('daily_types' , 'daily_journal_m.daily_type_id' , '=' , 'daily_types.daily_type_id')
            ->orderBy('journal_date' )
            ->where('journal_active' , 1)
            ->where('confirmed' , 1)
            ->whereRaw("FIND_IN_SET(cost_center.cost_center_id, daily_journal_d.cost_centers) > 0");

        if(isset($params['before'])){
            $query->where('journal_date' , '<' , $params['before'] );
        }elseif(isset($params['from']) && isset($params['to']) ){
            $query->where('journal_date' , '>=' , $params['from'] )
                  ->where('journal_date' , '<=' , $params['to'] );
        }

        if(is_array($id)){
            $query->whereIn('cost_center.cost_center_id' , $id);
        }else{
            $query->where('cost_center.cost_center_id' , $id);
        }

        if(isset($params['total'])){

            $data = $query->selectRaw('sum(debit) as debit , sum(credit) as credit , (sum(debit) - sum(credit)) as total')
                ->first();

        }else{

            $data = $query->selectRaw('
                year_ser as journal_code ,
                branch_ser ,
                month_ser  ,
                journal_date ,
                debit ,
                credit ,
                daily_types.name_'.$lang.' as daily_type ,
                daily_journal_d.description as description ,
                document_id ,
                daily_journal_d.daily_journal_m_id as m_id ')
                ->get();

        }

        return $data;

    }

    public static function add_cost_center($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_cost_center($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('cost_center_id' , $id )->update($seed);
    }

    public static function delete_cost_center($id)
    {
        return DB::table(self::$main_table)->where('cost_center_id' , $id )->update( array('active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id )   );
    }

}
