<?php

namespace App\Models\Finance;

use Illuminate\Database\Eloquent\Model;
use DB;

class Currency extends Model
{
    protected $primaryKey     = 'currency_id';
    public static $main_table = 'currencys';
    public $table             = 'currencys';

    public static $rules = array(
        'name_en'          => 'required|string',
        'name_ar'          => 'required|string',
        'equivalent_value' => 'required|numeric',
    );

    public static function all_currencys($params = null)
    {
        $query = DB::table(self::$main_table)
            ->orderBy(self::$main_table.'.currency_id', 'desc');

        if(isset($params['lang']))
            $query->selectRaw('
                currency_id as id ,'.
                self::$main_table.'.name_'.$params['lang'].' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.equivalent_value ,'.
                self::$main_table.'.currency_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        if(isset($params['active']))
            $query->where('currency_active', $params['active']);

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->where(function ($stmt) use($params) {
                $stmt->where('name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('currency_id' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_currency($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('currency_id', $id);

        if($lang != null)
            $query->selectRaw('
                currency_id as id ,'.
                self::$main_table.'.name_'.$lang.' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.equivalent_value ,'.
                self::$main_table.'.currency_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        $data = $query->first();
        return $data;
    }

    public static function add_currency($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_currency($seed, $id)
    {
        return DB::table(self::$main_table)->where('currency_id', $id)->update($seed);
    }

    public static function delete_currency($id)
    {
        return DB::table(self::$main_table)->where('currency_id', $id)->update(array('currency_active' => 0, 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }
}
