<?php

namespace App\Models\Finance;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class DelegatePetty extends Model
{
    protected $primaryKey     = 'delegate_petty_m_id';
    public static $main_table = 'delegate_petty_m';
    public static $detail_table = 'delegate_petty_d';
    public $table             = 'delegate_petty_m';

    public static $details_rules = array(
        // 'clearance_id'    => 'required|numeric',
        'expense_code_id' => 'nullable|numeric',
        'amount'          => 'required|numeric',
    );

    public static $upload_rules = array(
        'file_path' => 'required',
        'id' => 'required|exists:delegate_petty_d,delegate_petty_d_id',
    );

    public static function all_delegate_pettys($params = null)
    {

        $query = DB::table(self::$main_table)
            ->leftjoin('branchs', self::$main_table.'.branch_id', '=', 'branchs.branch_id')
            ->leftjoin('finance_years', self::$main_table.'.finance_year_id', '=', 'finance_years.finance_year_id')
            ->leftjoin('delegates', self::$main_table.'.delegate_id', '=', 'delegates.delegate_id')
            ->leftjoin('users', self::$main_table.'.add_user', '=', 'users.id')
            // ->groupBy(self::$main_table.'.delegate_petty_m_i
//            ->whereIn(self::$main_table.'.delegate_id',explode(',',$userDelegateIds))
            ->orderBy(self::$main_table.'.delegate_petty_m_id', 'desc');

        if(isset($params['lang']))
            $query->selectRaw(
                self::$main_table.'.delegate_petty_m_id as m_id ,'.
                self::$main_table.'.branch_id ,
                branchs.name_'.$params['lang'].' as branch_name ,'.
                self::$main_table.'.finance_year_id ,
                finance_years.name as finance_year_name ,'.
                self::$main_table.'.daily_journal_m_id ,'.
                self::$main_table.'.delegate_id ,
                delegates.name_'.$params['lang'].' as delegate_name ,'.
                self::$main_table.'.source ,'.
                self::$main_table.'.action_date ,'.
                self::$main_table.'.total_amount ,'.
                self::$main_table.'.remaining_amount ,'.
                self::$main_table.'.delegate_petty_status,'.
                self::$main_table.'.delegate_petty_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user,
                users.user_full_name as added_by
            ');

        if(isset($params['active']))
            $query->where('delegate_petty_active', $params['active']);

        if(isset($params['status']))
            $query->where('delegate_petty_status', $params['status']);

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->leftjoin('delegate_petty_d', self::$main_table.'.delegate_petty_m_id', '=', 'delegate_petty_d.delegate_petty_m_id')
                ->where(function ($stmt) use($params) {
                $stmt->where('delegate_petty_d.description_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('delegate_petty_d.description_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('delegates.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('delegates.name_ar' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_delegate_petty_m($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('daily_types', self::$main_table.'.daily_type_id', '=', 'daily_types.daily_type_id')
            ->leftjoin('delegates', self::$main_table.'.delegate_id', '=', 'delegates.delegate_id')
            ->where('delegate_petty_m_id' , $id);

        $query->selectRaw('
            delegate_petty_m_id as m_id ,
            delegate_petty_m_id ,'.
            self::$main_table.'.branch_id ,'.
            self::$main_table.'.finance_year_id ,'.
            self::$main_table.'.daily_type_id ,
            daily_types.name_'.$lang.' as daily_type_name ,'.
            self::$main_table.'.delegate_id ,'.
            self::$main_table.'.daily_journal_m_id ,

            delegates.name_'.$lang.' as delegate_name ,'.
            self::$main_table.'.source ,'.
            self::$main_table.'.action_date ,'.
            self::$main_table.'.total_amount ,'.
            self::$main_table.'.remaining_amount ,'.
            self::$main_table.'.delegate_petty_status,'.
            self::$main_table.'.delegate_petty_active,'.
            self::$main_table.'.add_date,'.
            self::$main_table.'.add_user
        ');

        $data = $query->first();
        return $data;
    }

    public static function get_delegate_petty_d($id, $lang = null)
    {
        $query = DB::table('delegate_petty_d')
            ->leftjoin('expense_codes' , 'delegate_petty_d.expense_code_id' , '=' , 'expense_codes.expense_code_id')
            ->leftjoin('clearances_m' , 'clearances_m.clearance_id' , '=' , 'delegate_petty_d.clearance_id')
            ->leftjoin('customers' , 'customers.customer_id' , '=' , 'clearances_m.customer_id')
            ->where('delegate_petty_m_id', $id);

        if($lang != null){
            $query->selectRaw('
                delegate_petty_m_id as m_id ,
                delegate_petty_d_id as d_id ,
                delegate_petty_d.clearance_id ,
                delegate_petty_d.expense_code_id ,
                expense_codes.name_'.$lang.' as expense_code_name ,
                receipt_number ,
                amount_without_vat ,
                total_vat ,
                amount ,
                tax_id ,
                delegate_petty_d.invoice_id ,
                description_'.$lang.' as description ,
                description_en ,
                description_ar ,
                delegate_petty_d.add_date ,
                delegate_petty_d.treasury_id ,
                delegate_petty_d.add_user ,
                customers.name_'.$lang.' as customer_name,
                delegate_petty_d.file_path,
                delegate_petty_d.is_finance,
                concat(clearances_m.clearance_id  ," - " , customers.name_'.$lang.' , " - " ,clearances_m.shipping_policy_no) as label,
                delegate_petty_d.taxable
            ');
        }

        $data = $query->get();
        return $data;
    }

    public static function get_delegate_petty_d_by_d_id($id, $lang = null)
    {
        $query = DB::table('delegate_petty_d')
            ->where('delegate_petty_d_id', $id)
            ->selectRaw('file_path');


        $data = $query->first();
        return $data;
    }


    public static function get_delegate_petty_d_without_attachment($id, $lang = null)
    {
        $query = DB::table('delegate_petty_d')
            ->leftjoin('expense_codes' , 'delegate_petty_d.expense_code_id' , '=' , 'expense_codes.expense_code_id')
            ->leftjoin('delegate_petty_m' , 'delegate_petty_m.delegate_petty_m_id' , '=' , 'delegate_petty_d.delegate_petty_m_id')
            ->leftjoin('delegates' , 'delegates.delegate_id' , '=' , 'delegate_petty_m.delegate_id')
            ->whereNull('delegate_petty_d.file_path')
            ->where('delegate_petty_m.delegate_petty_active', 1)
            ->where('delegates.delegate_id', $id);

        if($lang != null){
            $query->selectRaw('
                delegate_petty_d.delegate_petty_m_id as m_id ,
                delegate_petty_d.delegate_petty_d_id as d_id ,
                delegate_petty_d.clearance_id ,
                expense_codes.name_'.$lang.' as expense_code_name ,
                receipt_number ,
                amount ,
                delegate_petty_d.file_path
            ');
        }

        $data = $query->get();
        return $data;
    }
    public static function add_delegate_petty_m($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function add_delegate_petty_d($seed)
    {
        return DB::table('delegate_petty_d')->insertGetId($seed);
    }

    public static function update_delegate_petty_m($seed, $id)
    {
        return DB::table(self::$main_table)->where('delegate_petty_m_id', $id)->update($seed);
    }

    public static function delete_delegate_petty_m($id)
    {
        return DB::table(self::$main_table)->where('delegate_petty_m_id', $id)->update(array('delegate_petty_active' => '0', 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }

    public static function delete_delegate_petty_d($id)
    {
        return DB::table('delegate_petty_d')->where('delegate_petty_m_id', $id)->delete();
    }

    public static function update_delegate_petty_d($id,$update)
    {
        return DB::table(self::$detail_table)->where('delegate_petty_d_id', $id)->update($update);
    }
}
