<?php

namespace App\Models\Main;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class Drafts extends Model
{

    protected $primaryKey = 'draft_id';

    public static $main_table = 'drafts';

    public static $rules = array(
        'branch_id'      => 'required|integer',
    );



    public static function all_drafts($params = null)
    {
        $query =  DB::table(self::$main_table)
            ->leftjoin('screen_subs','screen_subs.screen_code','drafts.screen_code')
            ->orderBy('draft_id');

        if( isset($params['lang']) )
            $query->selectRaw('
                    drafts.draft_id as id ,
                    drafts.branch_id ,
                    drafts.screen_code,
                    screen_subs.screen_sub_title_'.$params['lang'].' as screen_name,
                    drafts.form_data,
                    drafts.active ,
                    drafts.add_date
                  ');

        if(isset($params['active']))
            $query->where('drafts.active' , $params['active'] );

        if(isset($params['branch_id']))
            $query->where('drafts.branch_id' , $params['branch_id'] );

        if(isset($params['screen_code']))
            $query->where('drafts.screen_code' , $params['screen_code'] );

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_draft($id , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('draft_id' , $id);

        if($lang != null){
            $query->selectRaw('
                draft_id as id ,
                add_date
                  ');
        }
        $data = $query->first();

        return $data;
    }

    public static function add_draft($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_draft($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('draft_id' , $id )->update($seed);
    }

    public static function delete_draft($id)
    {
        return DB::table(self::$main_table)->where('draft_id' , $id )->update( array('active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
