<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class Reporting extends Model
{
    protected $primaryKey = 'report_m_id';

    public static $main_table = 'reports_m';

    public static $rules = array(
        
    );

    public static function all_reports($params = null)
    {
        $query =  DB::table(self::$main_table)
            ->leftjoin('screen_mains','screen_mains.screen_main_id','reports_m.screen_main_id')
            ->orderBy('report_m_id','desc');

        if($params['lang'] != null){
            $query->selectRaw('
                    report_m_id,
                    name_ar,
                    name_en,
                    name_'.$params['lang'].' as name,
                    file_ar,
                    file_en,
                    stored_procedure,
                    reports_m.screen_main_id,
                    screen_main_image,
                    screen_main_title_'.$params['lang'].' as screen_name

                ');
        }


        if(isset($params['word']) && strlen($params['word']) >  0){
            $query->where(function ($stmt) use($params) {
                $stmt->where('name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('name_en' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_report_m($id , $lang = null)
    {
        $query =  DB::table('reports_m')
            ->leftjoin('screen_mains','screen_mains.screen_main_id','reports_m.screen_main_id')
            ->where('report_m_id' , $id);

        if($lang != null){
            $query->selectRaw('
                report_m_id,
                name_ar,
                name_en,
                name_'.$lang.' as name,
                file_ar,
                file_en,
                stored_procedure,
                reports_m.screen_main_id,
                screen_main_image,
                screen_main_title_'.$lang.' as screen_name

            ');
        }

        $data = $query->first();
        return $data;

    }
    public static function get_report_d($id , $lang = null)
    {
        $query =  DB::table('reports_d')
            ->where('report_m_id' , $id)
            ->orderBy('report_order' , 'ASC');

        if($lang != null){
            $query->selectRaw('
                report_d_id,
                report_m_id,
                name_ar,
                name_en,
                name_'.$lang.' as name,
                value,
                lookup,
                data_type,
                input_default,
                input_type,
                required,
                report_order
            ');
        }

        $data = $query->get();

        return $data;

    }

    public static function add_report_m($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }
    public static function add_report_d($seed)
    {
        return DB::table('reports_d')->insertGetId($seed);
    }
    public static function update_report($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('report_m_id' , $id )->update($seed);
    }

    public static function delete_report($id)
    {
        return DB::table(self::$main_table)->where('report_m_id' , $id )->delete();
    }




}
