<?php

namespace App\Models\Sales;

use Illuminate\Database\Eloquent\Model;
use DB;

class ClearanceInvoiceDraft extends Model
{

    protected $primaryKey = 'clearance_id';

    public static $main_table = 'clearance_invoices_draft_m';
    public $table             = 'clearance_invoices_draft_m';

    public static $rules = array(
        'name_en'               => 'required|string' ,
        'name_ar'               => 'required|string' ,
        'mobile'                => 'required|string' ,
        'vat_register_number'   => 'required|string' ,
    );

    public static function all_invoices($params = null)
    {
        $query =  DB::table(self::$main_table)
            ->leftjoin('daily_journal_m' , 'clearance_invoices_draft_m.daily_journal_m_id' , '=' , 'daily_journal_m.daily_journal_m_id')
            ->leftjoin('clearances_m' , 'clearance_invoices_draft_m.clearance_id' , '=' , 'clearances_m.clearance_id')
            ->leftjoin('clearances_d' , 'clearance_invoices_draft_m.clearance_id' , '=' , 'clearances_d.clearances_m_id')
            ->leftjoin('customers' , 'clearance_invoices_draft_m.customer_id' , '=' , 'customers.customer_id')
            ->leftjoin('customers_info' , 'customers_info.customer_id' , '=' , 'clearance_invoices_draft_m.customer_id')
            ->leftjoin('ports' , 'clearance_invoices_draft_m.port_id' , '=' , 'ports.port_id')
            ->leftjoin('users' , 'users.id' , '=' , self::$main_table.'.add_user')
            ->orderBy('clearance_invoices_draft_m.invoice_id','DESC')
            ->groupBy('clearance_invoices_draft_m.invoice_id');


        if( isset($params['lang']) )
            $query->selectRaw('
                clearance_invoices_draft_m.invoice_id as id ,
                clearance_invoices_draft_m.clearance_id ,
                customers.name_'.$params['lang'].' as customer_name  ,
                MAX(customers_info.customers_info_email) as customer_email,
                ports.port_name_'.$params['lang'].' as port_name  ,
                clearance_invoices_draft_m.invoice_date ,
                clearance_invoices_draft_m.daily_journal_m_id ,
                clearance_invoices_draft_m.daily_journal_m_id as daily_journal_id ,
                daily_journal_m.year_ser as journal_code ,
                total_delegate,
                total_cost ,
                total_amount ,
                total_journals ,
                total_cost_without_vat ,
                doc_id  ,
                doc_code  ,
                draft  ,
                clearance_invoices_draft_m.vat_per ,
                clearance_invoices_draft_m.add_date,
                users.user_full_name as added_by
            ');

        if(isset($params['active']))
            $query->where('invoice_active' , $params['active'] );

        if(isset($params['from_id']))
            $query->where('clearance_invoices_draft_m.invoice_id' ,'>=', $params['from_id'] );

        if(isset($params['to_id']))
            $query->where('clearance_invoices_draft_m.invoice_id' ,'<=' , $params['to_id'] );

        if(isset($params['draft']))
            $query->where('draft' , $params['draft'] );

        if(isset($params['clearance_id']))
            $query->where(self::$main_table.'.clearance_id' , $params['clearance_id'] );

        if(isset($params['is_customs_clearance']))
            $query
                ->leftjoin('clearance_invoices_draft_d' , 'clearance_invoices_draft_m.invoice_id' , '=' , 'clearance_invoices_draft_d.invoice_m_id')
                ->where( 'clearance_invoices_draft_d.revenue_id', '='  , 1)->where('clearance_invoices_draft_d.amount_without_vat' ,'>',0);


        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('customers.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('customers.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearance_invoices_draft_m.invoice_id' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearance_invoices_draft_m.clearance_id' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearances_d.container_number' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearances_m.majira_policy' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearances_m.shipping_policy_no' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearances_m.customs_declaration_number' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('ports.port_name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('ports.port_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.clearance_id'  , $params['word']);
            });

        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_invoice($id , $lang = null)
    {
        $query =  DB::table(self::$main_table)
            ->leftjoin('clearances_m' , 'clearance_invoices_draft_m.clearance_id' , '=' , 'clearances_m.clearance_id')
            ->leftjoin('customers' , 'clearance_invoices_draft_m.customer_id' , '=' , 'customers.customer_id')
            ->leftjoin('customers as sub_customers' , 'clearances_m.sub_customer_id' , '=' , 'sub_customers.customer_id')
            ->leftjoin('customers_info' , 'customers_info.customer_id' , '=' , 'clearance_invoices_draft_m.customer_id')
            ->leftjoin('ports' , 'clearance_invoices_draft_m.port_id' , '=' , 'ports.port_id')
            ->where('clearance_invoices_draft_m.invoice_id' , $id);

        if( $lang != null ){
            $query->selectRaw('
                clearances_m.branch_id ,
                clearance_invoices_draft_m.invoice_active ,
                clearance_invoices_draft_m.customer_id ,
                MAX(customers_info.customers_info_email) as customer_email,
                clearance_invoices_draft_m.invoice_id as id ,
                clearance_invoices_draft_m.finance_year_id,
                clearance_invoices_draft_m.clearance_id ,
                sub_customers.name_'.$lang.' as sub_customer_name  ,
                customers.name_'.$lang.' as customer_name  ,
                ports.port_name_'.$lang.' as port_name  ,


                customers.vat_register_number ,
                customers.address ,
                customers.account_tree_id as customer_account_tree_id ,
                clearance_invoices_draft_m.invoice_date ,
                clearance_invoices_draft_m.daily_journal_m_id ,

                clearances_m.file_no ,
                clearance_invoices_draft_m.goods_type ,
                clearances_m.customs_declaration_date ,
                clearances_m.customs_declaration_number ,
                clearances_m.customs_declaration_file ,
                clearances_m.withdrawal_note ,
                clearances_m.classification_id ,
                clearances_m.action_date ,
                clearances_m.shipping_policy_no ,
                clearances_m.clearance_type ,
                clearances_m.container_count ,
                clearances_m.container_small ,
                clearances_m.container_big ,
                clearances_m.is_lcl ,
                clearances_m.pledge_number,
                clearances_m.pledge_date,

                total_delegate,
                total_cost ,
                total_amount ,
                total_journals ,
                total_cost_without_vat ,
                vat_per ,
                clearance_invoices_draft_m.add_date,
                doc_id  ,
                doc_code  ,
                clearances_m.note  ,
                draft
            ');
        }

        $data = $query->first();

        return $data;
    }

    public static function get_invoice_delegate_d($id , $lang)
    {
        return DB::table('clearance_invoices_draft_delegate_d')
            ->leftjoin('expense_codes' , 'clearance_invoices_draft_delegate_d.expense_code_id' , '=' , 'expense_codes.expense_code_id')
            ->where('invoice_m_id' , $id )
            ->selectRaw('
                clearance_invoices_delegate_d_id as d_id ,
                clearance_invoices_draft_delegate_d.expense_code_id ,
                expense_codes.name_'.$lang.' as expense_code_name ,
                expense_codes.name_ar ,
                expense_codes.name_en ,
                receipt_number ,
                expense_codes.account_tree_id as expense_code_account_tree_id ,
                from_type ,
                petty_ids ,
                multi_ids ,
                amount ,
                description_'.$lang.' as description ,
                clearance_invoices_draft_delegate_d.add_date
            ')->get();
    }


    public static function get_invoice_journals($id , $lang)
    {
        return DB::table('clearance_invoices_draft_journal')
            // ->leftjoin('account_tree' , 'clearance_invoices_draft_delegate_d.account_tree_id' , '=' , 'account_tree.account_tree_id')
            ->where('invoice_m_id' , $id )
            ->selectRaw('
                clearance_invoices_journal_id as d_id ,
                invoice_m_id as m_id ,
                clearance_invoices_draft_journal.account_tree_id ,
                amount
            ')
            ->get();
    }




    public static function get_invoice_d($id , $lang = null)
    {
        return DB::table('clearance_invoices_draft_d')
            ->leftjoin('revenues' , 'clearance_invoices_draft_d.revenue_id' , '=' , 'revenues.revenue_id')
            ->where('invoice_m_id' , $id)
            ->get();

    }


    public static function get_next($id , $branch){
        return DB::table(self::$main_table)
            ->where('invoice_id' , '>' , $id)
            ->where('invoice_active' , 1)
            ->selectRaw('invoice_id as m_id')
            ->first();
    }

    public static function get_previous($id, $branch){
        return DB::table(self::$main_table)
            ->where('invoice_id' , '<' , $id)
            ->where('invoice_active' , 1)
            ->selectRaw('invoice_id as m_id')
            ->orderBy('invoice_id' , 'desc')
            ->first();
    }

    public static function get_last($branch){
        return DB::table(self::$main_table)
            ->where('invoice_active' , 1)
            ->selectRaw('invoice_id as m_id')
            ->orderBy('invoice_id' , 'desc')
            ->first();
    }



    public static function add_invoice($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function add_invoice_d($seed)
    {
        return DB::table('clearance_invoices_draft_d')->insert($seed);
    }

    public static function delete_invoice_d($invoice_m_id)
    {
        return DB::table('clearance_invoices_draft_d')->where('invoice_m_id',$invoice_m_id)->delete();
    }

    public static function update_invoice($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('invoice_id' , $id )->update($seed);
    }

    public static function delete_invoice($id)
    {
        return DB::table(self::$main_table)->where('invoice_id' , $id )->update( array('invoice_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
    public static function hard_delete_invoice($id)
    {
        return DB::table(self::$main_table)->where('invoice_id' , $id )->delete();
    }

    public static function all_attachment($lang,$clearance_id){
        return  DB::table('clearance_attachments')
            ->leftJoin('delegate_petty_d','clearance_attachments.clearance_id','=','delegate_petty_d.clearance_id')
            ->leftjoin('clearance_file_names' , 'clearance_attachments.clearance_file_name_id' , '=' , 'clearance_file_names.id')
            ->where('clearance_attachments.clearance_id',$clearance_id)
            ->orWhere('delegate_petty_d.clearance_id',$clearance_id)
            ->selectRaw('clearance_file_names.name_'.$lang.' as file_name ,clearance_attachments.file_path,clearance_attachments.file_type,clearance_attachment_id as id,clearance_attachments.clearance_id')
            ->get();
    }

    public static function clearance_attachment($lang,$clearance_id){
        return  DB::table('clearance_attachments')
            ->leftjoin('clearance_file_names' , 'clearance_attachments.clearance_file_name_id' , '=' , 'clearance_file_names.id')
            ->where('clearance_attachments.clearance_id',$clearance_id)
            ->selectRaw('"clearance_attachment" as type_attachment  , clearance_file_names.name_'.$lang.' as file_name ,clearance_attachments.file_path,clearance_attachment_id as id')
            ->get();
    }
    public static function delegate_petty_attachment($lang,$clearance_id){
        return  DB::table('delegate_petty_d')
            ->where('delegate_petty_d.clearance_id',$clearance_id)
            ->selectRaw('"delegate_attachment" as type_attachment , "" as file_name,delegate_petty_d.file_path,delegate_petty_d_id as id')
            ->get();
    }
    public static function invoice_attachments($lang,$invoice_id){
        return  DB::table('invoice_attachments')
            ->where('invoice_attachments.invoice_type',"draft")
            ->where('invoice_attachments.invoice_id',$invoice_id)
            ->selectRaw('"invoice_attachment" as type_attachment , file_name,invoice_attachments.file_path,id')
            ->get();
    }
}
