<?php

namespace App\Models\Sales;

use Illuminate\Database\Eloquent\Model;
use DB;

class CovenantType extends Model
{
    protected $primaryKey     = 'covenant_type_id';
    public static $main_table = 'covenant_types';
    public $table             = 'covenant_types';

    public static $rules = array(
        'name_en' => 'required|string',
        'name_ar' => 'required|string',
    );

    public static function all_covenant_types($params = null)
    {
        $query = DB::table(self::$main_table)
            ->orderBy(self::$main_table.'.covenant_type_id', 'desc');

        if(isset($params['lang']))
            $query->selectRaw('
                covenant_type_id as id ,'.
                self::$main_table.'.name_'.$params['lang'].' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.covenant_type_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        if(isset($params['active']))
            $query->where('covenant_type_active', $params['active']);

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->where(function ($stmt) use($params) {
                $stmt->where('name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('covenant_type_id' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_covenant_type($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('covenant_type_id', $id);

        if($lang != null)
            $query->selectRaw('
                covenant_type_id as id ,'.
                self::$main_table.'.name_'.$lang.' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.covenant_type_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        $data = $query->first();
        return $data;
    }

    public static function add_covenant_type($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_covenant_type($seed, $id)
    {
        return DB::table(self::$main_table)->where('covenant_type_id', $id)->update($seed);
    }

    public static function delete_covenant_type($id)
    {
        return DB::table(self::$main_table)->where('covenant_type_id', $id)->update(array('covenant_type_active' => 0, 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }
}
