<?php

namespace App\Models\Sales;

use App\Models\Main\ScreenSub;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class Customer extends Model
{

    protected $primaryKey = 'customer_id';

    public static $main_table = 'customers';
    public  $table             = 'customers';
    public static $inputs             = "customers_inputs";

    public static $rules = array(
        'name_en'               => 'required|string|unique:customers' ,
        'name_ar'               => 'required|string|unique:customers' ,
        'mobile'                => 'nullable' ,
        'vat_register_number'   => 'required' ,
        'account_tree_id'       => 'required|numeric',
        'extra_inputs'          => 'nullable|array',
        'is_main'               => 'required|in:0,1',
        'customer_main_id'      => 'nullable',

    );



    public static function get_inputs(){
        $inputs = self::$inputs;
        $get_columns = array_column(DB::select("SHOW COLUMNS FROM {$inputs}"),'Field');
//        $get_specific_columns = array_diff($get_columns, ['customers_inputs_id']);
        return $get_columns;
    }


    public static function all_customers($params = null)
    {
        $query =  DB::table(self::$main_table)
            ->leftjoin('account_tree' , self::$main_table.'.account_tree_id' , '=' , 'account_tree.account_tree_id')
            ->leftjoin('users' , self::$main_table.'.add_user' , '=' , 'users.id')
            ->leftJoin('customers as main_customers' ,self::$main_table.'.customer_main_id' ,'=','main_customers.customer_id')
            ->orderBy('customers.customer_id' , 'desc');

        if( isset($params['lang']) )
            $query->selectRaw('
                customers.customer_id as id , 
                customers.delegate_id , '.
                self::$main_table.'.name_'.$params['lang'].' as name  , 
                customers.name_ar , customers.name_en , '.
                self::$main_table.'.mobile , 
                account_tree.name_'.$params['lang'].' as account_tree_name , '.
                self::$main_table.'.address ,account_tree.account_tree_id , '.
                self::$main_table.'.customer_type , '.self::$main_table.'.id_number , '.
                self::$main_table.'.email , '.self::$main_table.'.vat_register_number ,'.
                self::$main_table.'.credit_period , '.self::$main_table.'.notes , '.
                self::$main_table.'.alireda_code,customers.customer_main_id , 
                main_customers.name_'.$params['lang'] .' as customer_main_name ,
                users.user_full_name as added_by ,  
                customers.is_main , 
                customers.customer_token');

        if(isset($params['active']))
            $query->where('customers.customer_active' , $params['active'] );


        if(isset($params['is_main']))
            $query->where('customers.is_main' , $params['is_main'] );


        if(isset($params['customer_main_id']))
            $query->where('customers.customer_main_id' , $params['customer_main_id'] );

        if(isset($params['delegate_id']) && is_array($params['delegate_id']) && count($params['delegate_id']) > 0 ){
            $query->whereIn('customers.delegate_id' , $params['delegate_id'] );
        }elseif(isset($params['delegate_id']) && !is_array($params['delegate_id']) ){
            $query->where('customers.delegate_id' , $params['delegate_id'] );
        }


        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where(self::$main_table.'.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('account_tree.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('account_tree.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.customer_id' , 'LIKE' , "%{$params['word']}%");
            });

        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_customer($id , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->leftJoin('customer_files' ,self::$main_table.'.customer_id' ,'=','customer_files.customer_id')
            ->leftJoin('customers as main_customers' ,self::$main_table.'.customer_main_id' ,'=','main_customers.customer_id')
            ->leftJoin('delegates' ,self::$main_table.'.delegate_id' ,'=','delegates.delegate_id')
            ->where(self::$main_table.'.customer_id' , $id);

        if($lang != null){
            $query->selectRaw(self::$main_table.'.customer_id as id ,
                customers.name_ar  ,
                customers.name_en  ,
                customers.mobile ,
                customers.address ,
                customers.account_tree_id ,
                customers.customer_type ,
                customers.id_number ,
                customers.email ,
                customers.vat_register_number ,
                customers.credit_period ,
                customers.notes ,
                customers.customer_active ,
                customers.customer_order ,
                customers.customer_status ,
                customers.notes ,
                customers.credit_period ,
                customers.issuing_invoice ,
                customers.other_file ,
                customers.iban_file ,
                customers.commercial_register_file ,
                customers.prices_list_file  ,
                delegates.delegate_id ,
                delegates.name_'.$lang.' as delegate_name,
                customers.alireda_code,
                customers.customer_main_id,
                main_customers.name_'.$lang.' as customer_main_name,
                customers.is_main ,
                customers.add_building_no ,
                customers.add_street ,
                customers.add_sec_no ,
                customers.add_district ,
                customers.add_postcode ,
                customers.add_city ,
                customers.add_country ,
                customers.customer_token ,
                customers.token_expired
                                       ');
        }

        $data = $query->first();

        return $data;
    }

    public static function add_customer($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_customer($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('customer_id' , $id )->update($seed);
    }

    public static function delete_customer($id)
    {
        return DB::table(self::$main_table)->where('customer_id' , $id )->update( array('customer_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }

    public static function check_customer_name($lang , Request $request){

    }

    public static function get_dynamic_inputs(){
        $screen_sub_id = DB::table('screen_subs')->where('table_name',self::$main_table)->first()->screen_sub_id ?? '';
        return DB::table('inputs')->where('screen_sub_ids',$screen_sub_id)->get();
    }

//    public static function get_dynamic_columns(){
//        $screen_sub_id = DB::table('screen_subs')->where('table_name',self::$main_table)->first()->screen_sub_id;
//        return DB::table('inputs')->where('screen_sub_ids',$screen_sub_id)->select('input_label_ar','input_label_en')->get();
//    }
}
