<?php

namespace App\Models\Sales;

use Illuminate\Database\Eloquent\Model;
use DB;

class CustomsItem extends Model
{
    protected $primaryKey     = 'customs_item_id';
    public static $main_table = 'customs_items';
    public $table             = 'customs_items';

    public static $rules = array(
        'item_name_ar'  => 'required|string',
        'name_ar'       => 'required|string',
        'item_type'     => 'required|numeric'
    );

    public static function all_customs_items($params = null)
    {
        $query = DB::table(self::$main_table)
            ->orderBy(self::$main_table.'.customs_item_id', 'desc');

        if(isset($params['lang']))
            $query->selectRaw('
                customs_item_id as id ,'.
                self::$main_table.'.name_'.$params['lang'].' as name ,'.
                self::$main_table.'.item_name_'.$params['lang'].' as item_name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.item_type ,'.
                self::$main_table.'.slug ,'.
                self::$main_table.'.customs_item_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        if(isset($params['active']))
            $query->where('customs_item_active', $params['active']);

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->where(function ($stmt) use($params) {
                $stmt->where(self::$main_table.'.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.item_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.item_name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.slug' , 'LIKE' , $params['word'])
                    ->orWhere(self::$main_table.'.item_type' , 'LIKE' , $params['word'])
                    ->orWhere(self::$main_table.'.customs_item_id' , $params['word']);
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_customs_item($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('customs_item_id', $id);

        if($lang != null)
            $query->selectRaw('
                customs_item_id as id ,'.
                self::$main_table.'.name_'.$lang.' as name ,'.
                self::$main_table.'.item_name_'.$lang.' as item_name ,'.
                self::$main_table.'.item_name_ar ,'.
                self::$main_table.'.item_name_en ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.item_type ,'.
                self::$main_table.'.slug ,'.
                self::$main_table.'.customs_item_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        $data = $query->first();
        return $data;
    }

    public static function add_customs_item($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_customs_item($seed, $id)
    {
        return DB::table(self::$main_table)->where('customs_item_id', $id)->update($seed);
    }

    public static function delete_customs_item($id)
    {
        return DB::table(self::$main_table)->where('customs_item_id', $id)->update(array('customs_item_active' => 0, 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }
}
