<?php

namespace App\Models\Sales;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class NoticeCreditor extends Model
{
    protected $primaryKey = 'notice_creditor_m_id';

    public static $main_table = 'notice_creditors_m';

    public static function all_notice_creditors($params = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('clearance_invoices_m' , 'clearance_invoices_m.invoice_id' , '=' , 'notice_creditors_m.invoice_m_id')
            ->leftjoin('branchs', self::$main_table.'.branch_id', '=', 'branchs.branch_id')
            ->leftjoin('finance_years', self::$main_table.'.finance_year_id', '=', 'finance_years.finance_year_id')
            ->leftjoin('users', self::$main_table.'.add_user', '=', 'users.id')
            ->leftjoin('customers', 'clearance_invoices_m.customer_id', '=', 'customers.customer_id')
            ->leftjoin('daily_journal_m', self::$main_table.'.daily_journal_id', '=', 'daily_journal_m.daily_journal_m_id')
            ->where('notice_creditors_m.active', 1)
            ->orderBy(self::$main_table.'.notice_creditor_m_id', 'desc')
            ->groupBy('notice_creditor_m_id');

        if($params['lang'] != null){
            $query->selectRaw('
                notice_creditors_m.notice_creditor_m_id ,
                notice_creditors_m.branch_id,
                notice_creditors_m.finance_year_id,
                notice_creditors_m.daily_journal_id,
                notice_creditors_m.description,

                daily_journal_m.year_ser as journal_code,
                notice_creditors_m.invoice_date,
                clearance_invoices_m.customer_id ,
                customers.name_'.$params['lang'].' as customer_name ,
                customers.vat_register_number,
                clearance_invoices_m.invoice_date as clearance_invoice_date,
                notice_creditors_m.invoice_m_id,
                notice_creditors_m.notice_type_id,
                notice_creditors_m.total_vat,
                notice_creditors_m.total_without_vat,
                notice_creditors_m.total_with_vat
            ');
        }

        if(isset($params['branch']) && is_array($params['branch'])){
            $query->whereIn(self::$main_table.'.branch_id', $params['branch']);
        }elseif(isset($params['branch'])){
            $query->where(self::$main_table.'.branch_id', $params['branch']);
        }

        if(isset($params['finance_year_id']) && is_array($params['finance_year_id'])){
            $query->whereIn(self::$main_table.'.finance_year_id', $params['finance_year_id']);
        }elseif(isset($params['finance_year_id'])){
            $query->where(self::$main_table.'.finance_year_id', $params['finance_year_id']);
        }

        if(isset($params['shift'])){
            $query->where(self::$main_table.'.shift_id', $params['shift']);
        }

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->where(function ($stmt) use($params) {
                $stmt->where(self::$main_table.'.notice_creditor_m_id', 'LIKE', "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.invoice_m_id', 'LIKE', "%{$params['word']}%")
                    ->orWhere('customers.name_ar', 'LIKE', "%{$params['word']}%")
                    ->orWhere('customers.name_en', 'LIKE', "%{$params['word']}%")
                    ->orWhere('customers.mobile', 'LIKE', "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_notice_creditor_m($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('clearance_invoices_m' , 'clearance_invoices_m.invoice_id' , '=' , 'notice_creditors_m.invoice_m_id')
            ->leftjoin('branchs', self::$main_table.'.branch_id', '=', 'branchs.branch_id')
            ->leftjoin('finance_years', self::$main_table.'.finance_year_id', '=', 'finance_years.finance_year_id')
            ->leftjoin('users', self::$main_table.'.add_user', '=', 'users.id')
            ->leftjoin('clearances_m' , 'clearance_invoices_m.clearance_id' , '=' , 'clearances_m.clearance_id')
            ->leftjoin('ports' , 'clearance_invoices_m.port_id' , '=' , 'ports.port_id')
            ->leftjoin('customers', 'clearance_invoices_m.customer_id', '=', 'customers.customer_id')
            ->leftjoin('daily_journal_m', self::$main_table.'.daily_journal_id', '=', 'daily_journal_m.daily_journal_m_id')
            ->orderBy(self::$main_table.'.notice_creditor_m_id', 'desc')
            ->where('notice_creditors_m.notice_creditor_m_id' , $id)
            ->groupBy('notice_creditor_m_id');

        if($lang != null){
            $query->selectRaw('
                notice_creditors_m.notice_creditor_m_id ,
                notice_creditors_m.branch_id,
                notice_creditors_m.finance_year_id,
                notice_creditors_m.daily_journal_id,
                notice_creditors_m.description,
                daily_journal_m.year_ser as journal_code,
                notice_creditors_m.invoice_date,
                clearance_invoices_m.customer_id ,
                customers.name_'.$lang.' as customer_name ,
                customers.vat_register_number,
                clearances_m.shipping_policy_no ,
                clearances_m.customs_declaration_number ,
                ports.port_name_'.$lang.' as port_name  ,
                customers.address ,
                customers.account_tree_id as customer_account_tree_id ,
                clearance_invoices_m.clearance_id ,
                clearance_invoices_m.invoice_date as clearance_invoice_date,
                notice_creditors_m.invoice_m_id,
                notice_creditors_m.notice_type_id,
                notice_creditors_m.total_vat,
                notice_creditors_m.total_without_vat,
                notice_creditors_m.total_with_vat ,
                notice_creditors_m.active ,
                notice_creditors_m.add_date

            ');
        }

        $data = $query->first();
        return $data;
    }

    public static function get_notice_creditor_d($id, $lang = null)
    {
        $query = DB::table('notice_creditors_d')
            ->leftJoin('clearance_invoices_d','clearance_invoices_d.clearance_invoices_d_id','notice_creditors_d.invoice_d_id')
            ->leftJoin('revenues','clearance_invoices_d.revenue_id','revenues.revenue_id')
            ->selectRaw('
                notice_creditors_d.notice_creditor_m_id as m_id ,
                notice_creditors_d.notice_creditor_d_id as d_id ,
                notice_creditors_d.amount_with_vat ,
                notice_creditors_d.amount_without_vat ,
                notice_creditors_d.vat_amount ,
                notice_creditors_d.invoice_d_id as clearance_invoices_d_id ,
                clearance_invoices_d.current_vat ,
                clearance_invoices_d.current_with_vat ,
                clearance_invoices_d.current_without_vat ,
                clearance_invoices_d.vat_per ,
                revenues.account_tree_id ,
                revenues.revenue_name_'.($lang ?? 'en').' as revenue_name
            ')
            ->where('notice_creditor_m_id', $id);

        $data = $query->get();
        return $data;
    }

    public static function add_notice_creditor_m($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function add_notice_creditor_d($seed)
    {
        return DB::table('notice_creditors_d')->insertGetId($seed);
    }

    public static function update_notice_creditor_m($seed, $id)
    {
        return DB::table(self::$main_table)->where('notice_creditor_m_id', $id)->update($seed);
    }

    public static function update_notice_creditor_d($seed, $id)
    {
        return DB::table('notice_creditors_d')->where('notice_creditor_d_id', $id)->update($seed);
    }

    public static function delete_return_invoice_m($id)
    {
        return DB::table(self::$main_table)->where('notice_creditor_m_id', $id)->update(array('notice_creditor_active' => 0));
    }

    public static function delete_return_invoice_d($id)
    {
        return DB::table('notice_creditors_d')->where('notice_creditor_d_id', $id)->delete();
    }
}
