<?php

namespace App\Models\Sales;

use App\Models\Main\ScreenSub;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Schema;

class Revenue extends Model
{

    protected $primaryKey = 'revenue_id';

    public static $main_table = 'revenues';
    public  $table             = 'revenues';

    public static $rules = array(
        'revenue_name_ar'               => 'required|string' ,
        'revenue_name_en'               => 'required|string' ,
        'account_tree_id'                => 'nullable' ,
    );



    public static function all_revenues($params = null)
    {
        $query =  DB::table(self::$main_table)
            ->leftjoin('account_tree', self::$main_table.'.account_tree_id', '=', 'account_tree.account_tree_id')
            ->orderBy('revenue_id' , 'desc');

        if( isset($params['lang']) )
            $query->selectRaw('revenue_id as id , revenue_name_'.$params['lang'].' as name , revenue_name_ar , revenue_name_en , account_tree.account_tree_id , account_tree.name_'.$params['lang'].' as account_tree_name ');



        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('revenue_name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('revenue_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('account_tree.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('account_tree.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('revenue_id' , 'LIKE' , "%{$params['word']}%");
            });

        }
        if(isset($params['active']))
            $query->where('revenues.revenues_active' , $params['active'] );

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_revenue($id , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('account_tree', self::$main_table.'.account_tree_id', '=', 'account_tree.account_tree_id')
            ->where(self::$main_table.'.revenue_id' , $id);

        if($lang != null){
            $query->selectRaw(self::$main_table.'.revenue_id as id , revenues.revenue_name_ar  , revenues.revenue_name_en  , account_tree.account_tree_id ,  account_tree.name_'.$lang.' as account_tree_name ');
        }

        $data = $query->first();

        return $data;
    }

    public static function add_revenue($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_revenue($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('revenue_id' , $id )->update($seed);
    }

    public static function delete_revenue($id)
    {
        return DB::table(self::$main_table)->where('revenue_id' , $id)->update(array('revenues_active' => 0, 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }

}
