<?php

namespace App\Models\Trans;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class Agent extends Model
{
    protected $primaryKey = 'agent_id';

    public static $main_table = 'agents';
    public $table             = 'agents';

    public static $rules = array(
        'agent_name_ar' => 'required|string|unique:agents',
        'agent_name_en' => 'required|string|unique:agents',
        'area_id'       => 'required|exists:areas,area_id',
    );


    public static function all_agents($params = null)
    {
        $query = DB::table(self::$main_table)
            ->leftJoin('areas', self::$main_table . '.area_id', '=', 'areas.area_id')
            ->orderBy('agent_id', 'desc');

        if (isset($params['lang']))
            $query->selectRaw('agent_id as id , agent_name_' . $params['lang'] . ' as agent_name  , agents.mobile ,areas.area_id, areas.area_name_' . $params['lang']. ' as area_name , agents.add_date');

        if (isset($params['active']))
            $query->where('agent_active', $params['active']);


        if (isset($params['word']) && strlen($params['word']) > 0) {

            $query->where(function ($stmt) use ($params) {
                $stmt->where('agent_name_ar', 'LIKE', "%{$params['word']}%")
                    ->orWhere('agent_name_en', 'LIKE', "%{$params['word']}%")
                    ->orWhere('area_name_ar', 'LIKE', "%{$params['word']}%")
                    ->orWhere('area_name_en', 'LIKE', "%{$params['word']}%");
            });

        }

        if (isset($params['rows'])) {
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        } else {
            $data = $query->get();
        }

        return $data;
    }

    public static function get_agent($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->join('areas', self::$main_table . '.area_id', '=', 'areas.area_id')
            ->where('agent_id', $id);

        if ($lang != null) {
            $query->selectRaw('agent_id as id , agent_name_ar,agent_name_en  , agents.mobile , areas.area_name_'.$lang. ' as area_name , areas.area_id, agents.add_date');
        }

        $data = $query->first();

        return $data;
    }

    public static function add_agent($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_agent($seed, $id)
    {
        return DB::table(self::$main_table)->where('agent_id', $id)->update($seed);
    }

    public static function delete_agent($id)
    {
        return DB::table(self::$main_table)->where('agent_id', $id)->update(array('agent_active' => '0', 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }
}
