<?php

namespace App\Models\Trans;

use Illuminate\Database\Eloquent\Model;
use DB;

class AreaPrice extends Model
{

    protected $primaryKey = 'area_price_id';

    public static $main_table = 'area_prices';

    public static $rules = array(
        'from_area_id'      => 'required|numeric' ,
        'to_area_id'        => 'required|numeric' ,
        'driver_price_in'   => 'required|numeric' ,
        'driver_price_out'  => 'required|numeric' ,
        'trip_price_in'     => 'required|numeric' ,
        'trip_price_out'    => 'required|numeric' 

    );



    public static function all_area_prices($params = null)
    {
        $query =  DB::table(self::$main_table)
            ->leftjoin('areas as from_areas' , self::$main_table.'.from_area_id' , '=' , 'from_areas.area_id')
            ->leftjoin('areas as to_areas' , self::$main_table.'.to_area_id' , '=' , 'to_areas.area_id');

        if( isset($params['lang']) )
            $query->selectRaw('
                from_areas.area_name_'.$params['lang'].' as from_area_name ,
                to_areas.area_name_'.$params['lang'].' as to_area_name ,
                area_price_id as id , 
                from_area_id  , 
                to_area_id , 
                driver_price_out as driver_price , 
                trip_price_out as trip_price , 
                driver_price_in , 
                driver_price_out , 
                trip_price_in , 
                trip_price_out
                
                '); 

        if(isset($params['active']))
            $query->where('area_price_active' , $params['active'] );

        if(isset($params['from_area']))
            $query->where('from_area_id' , $params['from_area'] );

        if(isset($params['to_area']))
            $query->where('to_area_id' , $params['to_area'] );


        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('from_areas.area_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('from_areas.area_name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('to_areas.area_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('to_areas.area_name_ar' , 'LIKE' , "%{$params['word']}%");
            });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_area_price($id , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('area_price_id' , $id)
            ->leftjoin('areas as from_areas' , self::$main_table.'.from_area_id' , '=' , 'from_areas.area_id')
            ->leftjoin('areas as to_areas' , self::$main_table.'.to_area_id' , '=' , 'to_areas.area_id');

        if($lang != null){
            $query->selectRaw('
                from_areas.area_name_'.$params['lang'].' as from_area_name ,
                to_areas.area_name_'.$params['lang'].' as to_area_name ,
                area_price_id as id , 
                from_area_id  , 
                to_area_id , 
                driver_price_in , 
                driver_price_out , 
                trip_price_in , 
                trip_price_out'); 
        }

        $data = $query->first();

        return $data;
    }

    public static function add_area_price($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_area_price($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('area_price_id' , $id )->update($seed);
    }

    public static function delete_area_price($id)
    {
        return DB::table(self::$main_table)->where('area_price_id' , $id )->update( array('area_price_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
