<?php

namespace App\Models\Trans;

use Illuminate\Database\Eloquent\Model;
use DB;

class Port extends Model
{

    protected $primaryKey = 'port_id';

    public static $main_table = 'ports';

    public static $rules = array(
        'port_name_en'      => 'required|string' ,
        'port_name_ar'      => 'required|string' ,
        'area_id'           => 'required|numeric',
        'branch_id'           => 'required|numeric'
    );



    public static function all_ports($params = null)
    {
        $query =  DB::table(self::$main_table);

        if( isset($params['lang']) )
            $query->selectRaw('port_id as id , port_name_'.$params['lang'].' as name  , area_id');

        if(isset($params['active']))
            $query->where('port_active' , $params['active'] );

        if(isset($params['branch']))
            $query->where('branch_id' , $params['branch'] );


        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('port_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('port_name_ar' , 'LIKE' , "%{$params['word']}%");
            });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_port($id , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('port_id' , $id);

        if($lang != null){
            $query->selectRaw('port_id as id , port_name_'.$lang.' as name  , area_id');
        }

        $data = $query->first();

        return $data;
    }

    public static function add_port($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_port($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('port_id' , $id )->update($seed);
    }

    public static function delete_port($id)
    {
        return DB::table(self::$main_table)->where('port_id' , $id )->update( array('port_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
