<?php

namespace App\Models\Trans;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class Trip extends Model
{


    // protected static $otherDB = session()->get('otherDB');
    protected $primaryKey = 'trip_id';

    public static $main_table = 'trips';

    public static $rules = array(
        'customer_id'           => 'required|numeric' ,
        'trip_type'             => 'required|numeric|min:1|max:3' ,
        'from_port_id'          => 'nullable|numeric',
        'from_wearhouse_id'     => 'nullable|numeric',
        'from_area_id'          => 'nullable|numeric',
        'to_port_id'            => 'nullable|numeric',
        'to_wearhouse_id'       => 'nullable|numeric',
        'to_area_id'            => 'nullable|numeric',
        'trip_date'             => 'required|date',
        'car_id'                => 'required|numeric',
        'trip_price'            => 'required|numeric',
        'company_bears_expenses'         =>'nullable|in:0,1',
        'to_agent_id'           => 'nullable|exists:agents,agent_id',
        // 'clearance_m_id'           => 'nullable|exists:clearances_m,clearance_id',
        // 'clearance_d_id'           => 'nullable|exists:clearances_d,clearances_d_id'
    );



    public static function all_trips($params = null)
    {
        $otherDB = session()->get('otherDB');
        // return $otherDB;
        $query =  DB::table(self::$main_table)
            ->orderBy('trip_id' , 'desc')
            ->leftjoin('customers' , self::$main_table.'.customer_id' , '=' , 'customers.customer_id')
            ->leftjoin($otherDB.'.customers as amwag_cust' , self::$main_table.'.customer_id' , '=' , 'amwag_cust.customer_id')
            ->leftjoin($otherDB.'.agents as to_agents' , self::$main_table.'.to_agent_id' , '=' , 'to_agents.agent_id')
            ->leftjoin('drivers' , self::$main_table.'.driver_id' , '=' , 'drivers.driver_id')
            ->leftjoin('cars' , self::$main_table.'.car_id' , '=' , 'cars.car_id')
            ->leftjoin($otherDB.'.clearances_d' , self::$main_table.'.clearance_d_id' , '=' , 'clearances_d.clearances_d_id' )
            ->leftjoin($otherDB.'.clearances_m' , 'clearances_d.clearances_m_id' , '=' , 'clearances_m.clearance_id' )

            ->leftjoin('areas as from_areas' , self::$main_table.'.from_area_id' , '=' , 'from_areas.area_id')
            ->leftjoin('areas as to_areas' , self::$main_table.'.to_area_id' , '=' , 'to_areas.area_id')
            ->leftjoin('areas as final_dest' , self::$main_table.'.final_dest_id' , '=' , 'from_areas.area_id')

            ->leftjoin('ports as from_ports' , self::$main_table.'.from_port_id' , '=' , 'from_ports.port_id')
            ->leftjoin('ports as to_ports' , self::$main_table.'.to_port_id' , '=' , 'to_ports.port_id')

            ->leftjoin('wearhouses as from_wearhouses' , self::$main_table.'.from_wearhouse_id' , '=' , 'from_wearhouses.wearhouse_id')
            ->leftjoin('wearhouses as to_wearhouses' , self::$main_table.'.to_wearhouse_id' , '=' , 'to_wearhouses.wearhouse_id');


        if( isset($params['lang']) )
            $query->selectRaw('
                trip_id as id ,

                ifnull( if( trips.clearance_m_id > 0 , amwag_cust.name_'.$params['lang'].' , customers.name_'.$params['lang'].' ) , trips.customer_name) as customer_name ,

                if( trips.clearance_m_id > 0 , amwag_cust.vat_register_number , if(trips.customer_id = 1 , trips.customer_vat_number , customers.vat_register_number ) ) as vat_register_number ,

                if( trips.clearance_m_id > 0 , amwag_cust.mobile , customers.mobile ) as customer_mobile ,

                if( trips.clearance_d_id > 0 , clearances_d.container_number, trips.container_number ) as container_number ,
                if( trips.clearance_d_id > 0 , clearances_d.container_size, trips.container_size ) as container_size ,


                from_areas.area_name_'.$params['lang'].' as from_area_name ,
                to_areas.area_name_'.$params['lang'].' as to_area_name ,

                from_ports.port_name_'.$params['lang'].' as from_port_name ,
                to_ports.port_name_'.$params['lang'].' as to_port_name ,

                from_wearhouses.wearhouse_name_'.$params['lang'].' as from_wearhouse_name ,
                to_wearhouses.wearhouse_name_'.$params['lang'].' as to_wearhouse_name ,

                to_agents.agent_name_'.$params['lang'].' as to_agent_name ,

                drivers.driver_name_'.$params['lang'].' as driver_name ,
                cars.car_name_'.$params['lang'].' as car_name ,
                cars.car_code ,
                clearances_d.recommended_return_date,
                clearances_d.trans_vat_amount,
                clearances_d.trans_amount_without_vat,
                clearances_d.trans_amount_with_vat,
                clearances_d.archive_id ,
                clearances_d.return_date ,
                clearances_d.recommended_return_date  as agent_date,
                clearances_m.is_lcl ,
                clearances_m.port_id ,
                trips.is_empty ,
                trips.clearance_m_id ,
                trips.clearance_d_id ,
                trip_type ,
                "" as trip_notes ,
                trip_road ,
                from_port_id ,
                from_wearhouse_id ,
                from_area_id ,
                final_dest_id ,
                final_dest.area_name_'.$params['lang'].' as final_dest_name ,

                to_port_id ,
                to_wearhouse_id ,
                to_area_id ,
                to_agent_id ,
                trip_date ,
                trips.related_trans_date,
                trip_price  ,
                driver_price ,
                trips.total_price ,
                trips.invoice_id,
                company_bears_expenses
             ');

        if(isset($params['active']))
            $query->where('trip_active' , $params['active'] );

        if(isset($params['ids']))
            $query->whereIn('trips.trip_id' , $params['ids'] );

        if(isset($params['customer']) && $params['customer'] == 0 ){
            $query->where('trips.trip_type' , 1 );

        }elseif(isset($params['customer'])){
            $query->where('trips.customer_id' , $params['customer'] );
        }

        if(isset($params['port_id']))
            $query->where('clearances_m.port_id' , $params['port_id'] );
                // ->orWhere('trips.clearance_m_id' , 0);

        if(isset($params['branch_id']))
            $query->where('trips.branch_id' , $params['branch_id'] );

        if(isset($params['invoice_id']))
            $query->where('trips.invoice_id' , $params['invoice_id'] );

        if(isset($params['driver_id']))
            $query->where('trips.driver_id' , $params['driver_id'] );

        if(isset($params['car_id']))
            $query->where('trips.car_id' , $params['car_id'] );

        if(isset($params['from']))
            $query->where(self::$main_table.'.trip_date' , '>=' , $params['from']." 00:00:00" );

        if(isset($params['to']))
            $query->where(self::$main_table.'.trip_date' , '<=' , $params['to']." 23:59:59" );

        if(isset($params['day_date']))
            $query->where(self::$main_table.'.trip_date' , 'like' , $params['day_date']."%" );

        if(isset($params['driver_paid']))
            $query->where('trips.driver_paid' , $params['driver_paid'] );

        if(isset($params['is_returned']) && $params['is_returned'] == 1)
            $query->where('clearances_d.archive_id' ,'>', 0);

        if(isset($params['in']))
            $query->whereIn('trips.trip_road' , [1,4] );

        if(isset($params['out']))
            $query->whereIn('trips.trip_road' , [3,5] );

        if(isset($params['first']))
            $query->distinct('container_number')->groupBy('container_number');

        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('customers.name_ar' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('customers.name_en' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('from_areas.area_name_ar' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('from_areas.area_name_en' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('drivers.driver_name_ar' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('drivers.driver_name_en' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('trips.container_number' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('trips.container_number' , $params['word']);
                    $stmt->orWhere('trips.container_size' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('clearances_d.container_number' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('clearances_d.container_size' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('trips.clearance_m_id' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('car_name_ar' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('car_name_en' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('to_wearhouses.wearhouse_name_ar' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('to_wearhouses.wearhouse_name_en' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('to_ports.port_name_ar' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('to_ports.port_name_en' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('to_areas.area_name_ar' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('to_areas.area_name_en' , 'LIKE' , "%{$params['word']}%");
                    $stmt->orWhere('trips.trip_id' ,$params['word']);
            });

        }



        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_trip($id , $lang = null)
    {
        $otherDB = session()->get('otherDB');

        $query = DB::table(self::$main_table)
            ->leftjoin('customers' , self::$main_table.'.customer_id' , '=' , 'customers.customer_id')
            ->leftjoin($otherDB.'.customers as amwag_cust' , self::$main_table.'.customer_id' , '=' , 'amwag_cust.customer_id')
            ->leftjoin('drivers' , self::$main_table.'.driver_id' , '=' , 'drivers.driver_id')
            ->leftjoin('cars' , self::$main_table.'.car_id' , '=' , 'cars.car_id')
            ->leftjoin($otherDB.'.clearances_d' , self::$main_table.'.clearance_d_id' , '=' , 'clearances_d.clearances_d_id' )
            ->leftjoin($otherDB.'.clearances_m' , 'clearances_d.clearances_m_id' , '=' , 'clearances_m.clearance_id' )
            ->leftjoin('areas as to_area', self::$main_table . '.to_area_id', '=', 'to_area.area_id')
            ->leftjoin('areas as final_dest' , self::$main_table.'.final_dest_id' , '=' , 'final_dest.area_id')

            ->where('trip_id' , $id);

        if($lang != null){
            $query->selectRaw('
                trip_id ,
                trips.branch_id ,
                clearance_m_id ,
                clearance_d_id ,

                if( trips.clearance_m_id > 0 , amwag_cust.name_'.$lang.' , customers.name_'.$lang.' ) as customer_name ,
                if( trips.clearance_m_id > 0 , amwag_cust.vat_register_number , customers.vat_register_number ) as customer_vat_number ,
                if( trips.clearance_m_id > 0 , amwag_cust.mobile , customers.mobile ) as customer_mobile ,
                if( trips.clearance_d_id > 0 , clearances_d.container_number, trips.container_number ) as container_number ,
                if( trips.clearance_d_id > 0 , clearances_d.container_size, trips.container_size ) as container_size ,
                if( trips.clearance_d_id > 0 , clearances_d.seal_number, null ) as seal_number ,
                clearances_m.shipping_policy_no ,
                clearances_d.recommended_return_date,
                trips.is_empty ,
                clearances_m.is_lcl ,
                trips.driver_id ,
                trips.car_id ,
                drivers.driver_name_'.$lang.' as driver_name ,
                cars.car_name_'.$lang.' as car_name ,

                trip_type ,
                trip_road ,
                trips.customer_id ,
                trip_id as id ,
                from_port_id ,
                from_wearhouse_id ,
                from_area_id ,
                to_port_id ,
                to_wearhouse_id ,
                to_area_id ,
                final_dest_id ,
                final_dest.area_name_'.$lang.' as final_dest_name ,
                trip_date ,
                related_trans_date,
                trip_price  ,
                driver_price ,
                total_price,
                company_bears_expenses,
                customs_declaration_number,
                to_area.area_name_'.$lang.' as  to_area_name
             ');
        }

        $data = $query->first();

        return $data;
    }

    public static function add_trip($seed)
    {
        return DB::table(self::$main_table)->lockForUpdate()->insertGetId($seed);
    }

    public static function update_trip($seed ,  $id)
    {

        $seed['update_user'] = auth()->guard('api')->user()->id ?? 0;
        $seed['update_date'] = date("Y-m-d H:i:s") ;
        return DB::table(self::$main_table)->where('trip_id' , $id )->update($seed);
    }

    public static function delete_trip($id)
    {
        return DB::table(self::$main_table)->where('trip_id' , $id )->delete();
        // return DB::table(self::$main_table)->where('trip_id' , $id )->update( array('trip_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
