import Vue from "vue";
import axios from "axios";
import VueAxios from "vue-axios";
import JwtService from "./jwt.service";
import router from '../router'
import Alerts from "../services/alerts";



/**
 * Service to call HTTP request via Axios
 */
const ApiService = {

	// init() {
	// 	const lang = localStorage.getItem('lang');
	// 	Vue.use(VueAxios, axios);
	// 	Vue.axios.defaults.baseURL = "http://localhost:20002/apsi/v1/"+lang+"/";
	// 	this.setHeader();
	// },

	setHeader() {
		Vue.axios.defaults.headers.common["Authorization"] = `Bearer ${JwtService.getToken()}`;
		Vue.axios.defaults.headers.common["access-token"] = process.env.MIX_APP_API_TOKEN;
		Vue.axios.defaults.headers.common["Accept"] = 'application/json';
	},

	setError(status, title = 'error', error) {
		if (status == 401) {
			router.push({ name: 'Login' });
		} else {
			Alerts.ShowAlert('error', title, error);
		}
	},

	query(resource, params) {
		return Vue.axios.get(resource, params).catch(error => {
			// console.log(error);
			throw new Error(`[KT] ApiService ${error}`);
		});
	},

	/**
	 * Send the GET HTTP request
	 * @param resource
	 * @param slug
	 * @returns {*}
	 */
	get(resource, slug = "") {

		// return new Promise(resolve => {
		//   Vue.axios.get(`${resource}/${slug}`).then(({ data }) => {
		//     if(data.check){
		//       resolve(data.data);
		//     }else{
		//       this.setError(100 , 'Error' ,data.msg);
		//     }

		//   }).catch(error => {
		//     this.setError(error.response.status , error);
		//     return '';
		//   });
		// });


		return Vue.axios.get(`${resource}?${slug}`).catch(error => {



			this.setError(error.response.status , 'error' , error);
			return '';
		});
	},

	/**
	 * Set the POST HTTP request
	 * @param resource
	 * @param params
	 * @returns {*}
	 */
	post(resource, params) {

		// return new Promise(resolve => {
		//   Vue.axios.post(`${resource}`, params).then(({ data }) => {
		//     if(data.check){
		//       resolve(data.data);
		//     }else{
		//       this.setError(100 , 'Error' ,data.msg);
		//     }

		//   }).catch(error => {
		//     this.setError(error.response.status , error);
		//     return '';
		//   });
		// });

		return Vue.axios.post(`${resource}`, params).catch(error => {
			this.setError(error);
		});
	},

	/**
	 * Send the UPDATE HTTP request
	 * @param resource
	 * @param slug
	 * @param params
	 * @returns {IDBRequest<IDBValidKey> | Promise<void>}
	 */
	update(resource, slug, params) {
		return Vue.axios.put(`${resource}/${slug}`, params);
	},

	/**
	 * Send the PUT HTTP request
	 * @param resource
	 * @param params
	 * @returns {IDBRequest<IDBValidKey> | Promise<void>}
	 */
	put(resource, params) {
		return Vue.axios.put(`${resource}`, params);
	},

	/**
	 * Send the DELETE HTTP request
	 * @param resource
	 * @returns {*}
	 */
	delete(resource) {
		return Vue.axios.delete(resource).catch(error => {
			// console.log(error);
			throw new Error(`[RWV] ApiService ${error}`);
		});
	}
};

export default ApiService;
