import ApiService from "../services/api.service";
import JwtService from "../services/jwt.service";
import Alerts from "../services/alerts";
import router from '../router'
import Vue from "vue";

// action types
export const LOGIN = "login";
export const LOGOUT = "logout";

// mutation types
export const PURGE_AUTH = "logOut";
export const SET_AUTH = "setUser";
export const SET_ERROR = "setError";

const state = {
	errors: null,
	user: JSON.parse(localStorage.getItem('userData')),
	isAuthenticated: !!JwtService.getToken(),
	UserToken: JwtService.getToken(),
};

const getters = {
	currentUser(state) {
		return state.user;
	},
	getBranchs(state) {
		return state.user.user.branchs;
	},
	getBranch(state) {
		if(!!state.user.user.shift){
			return state.user.user.shift.branch_id;
		}else{
			return 0;
		}
	},
	getPermissions(state) {
		return state.user.user.mypermissions;
	},
	getShift(state) {
		return state.user.user.shift;
	},
	isShiftStarted(state) {
		return !!state.user.user.shift;
	},
	isAuthenticated(state) {
		return state.isAuthenticated;
	}
};

const actions = {
	[LOGIN](context, credentials) {
		return new Promise(resolve => {
			ApiService.post("login", credentials)
				.then(({ data }) => {
					if (data.check) {
						context.commit(SET_AUTH, data.data);
						resolve(data);
					} else {
						context.commit(SET_ERROR, data.msg);
					}
				})
				.catch(({ response }) => {
					context.commit(SET_ERROR, response.data.errors);
				});
		});
	},
	[LOGOUT](context) {
		context.commit(PURGE_AUTH);
	},
	updateAuthData(context) {
		console.log('updateAuthData');
		context.commit('updateAuthData');
	},
};

const mutations = {
	updateAuthData(state ,context){
		ApiService.setHeader();
		ApiService.get("getMyProfile")
			.then(({ data }) => {
				if (data.check) {
					localStorage.setItem('userData' , JSON.stringify(data.data));
					JwtService.saveToken(data.data.api_token);

				} else {
					context.commit(SET_ERROR, data.msg);
				}
			})
			.catch(({ response }) => {
				context.commit(SET_ERROR, response.data.errors);
			});


		state.user = JSON.parse(localStorage.getItem('userData'));
	},
	[SET_ERROR](state, error) {
		state.errors = error;
		Alerts.ShowAlert('error', 'Auth faild !', state.errors);
	},
	[SET_AUTH](state, user) {
		// console.log('user', state);
		state.isAuthenticated = true;
		state.user = user;
		state.errors = {};
		localStorage.setItem('userData' , JSON.stringify(user));
		JwtService.saveToken(user.api_token);
		router.push({ name: 'Home' });

	},
	[PURGE_AUTH](state) {
		state.isAuthenticated = false;
		state.user = {};
		localStorage.setItem('userData' , '[]');
		state.errors = {};
		JwtService.destroyToken();
		router.push({ name: 'Login' });

	}
};

export default {
	state,
	actions,
	mutations,
	getters
};
