import axios from "axios";
import VueAxios from "vue-axios";
import JwtService from "./jwt.service";
import Swal from 'sweetalert2/dist/sweetalert2.js'

export default {

  install(Vue, store) {
    
    Vue.use(VueAxios, axios);
    const lang = localStorage.getItem('lang');
    Vue.axios.defaults.baseURL = process.env.MIX_APP_URL+"/api/v1/"+lang+"/";
    Vue.axios.defaults.headers.common["access-token"] = process.env.MIX_APP_API_TOKEN;
    Vue.axios.defaults.headers.common["Accept"] = 'application/json';

    Vue.prototype.$toDecimal = (number) => {
      if(typeof number != 'number')
        number = parseFloat(number);
      return (number.toFixed(2)).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
    }

    Vue.prototype.$printInvoice = (url , id) => {
      var frame1 = document.createElement('iframe');
			frame1.name = "frame1";
			frame1.style.position = "absolute";
			frame1.style.top = "-10000000px";
			frame1.src = process.env.MIX_APP_URL+'/'+url+'/'+id;
			document.body.appendChild(frame1);
			// var frameDoc = frame1.contentWindow ? frame1.contentWindow : frame1.contentDocument.document ? frame1.contentDocument.document : frame1.contentDocument;
			// frameDoc.document.open();
			// frameDoc.document.write(`<html><head><title>${title}</title>`);
			// frameDoc.document.write('</head><body>');
			// frameDoc.document.write(contents);
			// frameDoc.document.write(`<iframe src="http://localhost:20002/invoiceByQty"></iframe>`);
			// frameDoc.document.write(`<p src="http://localhost:20002/invoiceByQty"></p>`);
			// frameDoc.document.write('</body></html>');
			// frameDoc.document.close();
      // frameDoc.r
			setTimeout(function () {
				window.frames["frame1"].focus();
				window.frames["frame1"].print();
				document.body.removeChild(frame1);
			}, 5000);
			return false;
    }



    Vue.prototype.$alerts = {
      ShowAlert(type, title, msg) {
        Swal.fire({
          title: title,
          text: msg,
          icon: type,
          confirmButtonText: 'Ok'
        });
      },
    };
    
    Vue.prototype.$api = {
      updateToken(){
        Vue.axios.defaults.headers.common["Authorization"] = `Bearer ${JwtService.getToken()}`;
        Vue.axios.defaults.headers.common["currentBranch"] = localStorage.getItem('currentBranch');
      },
      setError(status, title = 'error', error) {
        // console.log('status' ,status);
        if (status == 401) {
          store.dispatch('logout');
        } else {
          Vue.prototype.$alerts.ShowAlert('error', title, error);
        }
      },
      get(resource, slug = "") {
        this.updateToken();
        return new Promise(resolve => {
          Vue.axios.get(`${resource}?${slug}`).then(({ data }) => {
              resolve(data);
          }).catch(error => {
            store.dispatch('endLoading');
            this.setError(error.response.status , error);
            return '';
          });
        });

        // return Vue.axios.get(`${resource}/${slug}`).catch(error => {
        //   return this.setError(error.response.status , 'error' , error);
        // });
      },
      post(resource, params) {
        this.updateToken();
        return new Promise(resolve => {
          Vue.axios.post(`${resource}`, params).then(({ data }) => {
            resolve(data);
          }).catch(error => {
            store.dispatch('endLoading');
            this.setError(error.response.status , error);
            return '';
          });
        });
    
        // return Vue.axios.post(`${resource}`, params).catch(error => {
        //   this.setError(error);
        // });
      },
    };
 }
}